/* File: maptools3.js
 * All functions relating to gathering info from the World Map
 */

dt.mapTools = {
    /* Method: checkHash
     * Check if the user is currently viewing the world map. Return true if they are
     * otherwise display a message and return false.
     * 
     */
    checkHash: function () {
        var newHash = window.location.hash;
        if (newHash.indexOf('?')) {
            var bits = newHash.split('?');
            newHash = bits[0];
        }
        var bits = newHash.split('/');
        if (bits[1] === "World") {
            if (bits[2] === 'Map') {
                return true;
            }
        }
        window.alert("You must be viewing the world map to use this tool.");
        return false;
    },

    showSettings: function () {
        // Load the form with the current settings
        var sects = document.querySelectorAll("fieldset.cbox");
        var sectNames = ["noteDisplay","fiveSq","tenSq"];
        for(var i=0;i<sects.length;i++){
            var thisSect = sects[i];
            thisSect.querySelector(".current").style.backgroundColor = dt[PlayerId].mapDisplay[sectNames[i]].colour;
            thisSect.querySelector("." + dt[PlayerId].mapDisplay[sectNames[i]].onoff).checked = true;
        }
        dt.showTool("#colourSetter");
    },

    saveSettings: function (e) {
        var tgt = e.target;
        var p = tgt.dataset.p;
        switch (p) {
            case "Notes":
                sect = "noteDisplay";
                break;
            case "fiveSq":
                sect = "fiveSq";
                break;
            case "tenSq":
                sect = "tenSq";
                break;
        }
        if (tgt.dataset.c) {
            dt[PlayerId].mapDisplay[sect].colour = tgt.dataset.c;
            tgt.parentNode.parentNode.previousElementSibling.style.backgroundColor = tgt.dataset.c;
        }
        if (tgt.dataset.state) {
            dt[PlayerId].mapDisplay[sect].onoff = tgt.dataset.state;
        }
//        var c = tgt.dataset.c;
//        var cont = document.querySelector(".p");
//        var currentDisp = cont.querySelector("div.current");
//        var currentState = cont.querySelector(".onoff input[name='onoff']:checked");

        // querySelector('input[name="rate"]:checked').value;
//        var p = tgt.parentNode.parentNode.previousElementSibling;
        console.log(p);
//        currentDisp.style.backgroundColor = c;
        locStorage.setItem(PlayerId + ":mapDisplay", JSON.stringify(dt[PlayerId].mapDisplay));
        var rubbish = "";

    },

    addNotes: function () {
        // Get the canvas container
        var mDiv = document.getElementById("mapDiv");
        // Get the canvas element for the cities
        var cCanvas = document.getElementById("mapCities");
        // Create the canvas for the notes
        var mapNotes = document.createElement("canvas");
        // <canvas id="mapSov" width="500" height="500" style="position:absolute;top:11px;left:15px" unselectable="on"></canvas
        mapNotes.setAttribute("id", "notesCanvas");
        mapNotes.setAttribute("width", "500");
        mapNotes.setAttribute("height", "500");
        mapNotes.setAttribute("style", "position:absolute;top:11px;left:15px");
        mapNotes.setAttribute("unselectable", "on");
    },
    /* Function: noteDispUpdate
     * 
     * @return {undefined}
     */
    noteDispUpdate: function () {
        if (dt[PlayerId].mapDisplay.noteDisplay.onoff == "on") {
            if (window.location.href.startsWith("https://elgea.illyriad.co.uk/#/World/Map")) {
                document.querySelector("#mapNotes").classList.replace("nosee", "see");
            }
        } else {
            if (window.location.href.startsWith("https://elgea.illyriad.co.uk/#/World/Map")) {
                document.querySelector("#mapNotes").classList.replace("see", "nosee");
            }
        }
//       locStorage.setItem(PlayerId + ":mapDisplay", JSON.stringify(dt[PlayerId].mapDisplay));
        var rubbish = "";
    },
    townRadDispToggle: function () {
        var currentState = document.querySelector("#townRadDispToggle").textContent;
        var rd = document.querySelector("#townRadDispToggle");
        if (currentState == "Enable 10sq radius display") {
            rd.textContent = "Disable 10sq radius display";
            if (window.location.href.startsWith("https://elgea.illyriad.co.uk/#/World/Map")) {
//            document.querySelector("#townRadDispToggle").textContent = "Disable 10sq radius display";
                document.querySelector("#townRad").classList.replace("nosee", "see");
            }
            dt[PlayerId].mapDisplay.tenSq = "see";
        } else {
            rd.textContent = "Enable 10sq radius display";
            if (window.location.href.startsWith("https://elgea.illyriad.co.uk/#/World/Map")) {
//          document.querySelector("#townRadDispToggle").textContent = "Enable 10sq radius display";
                document.querySelector("#townRad").classList.replace("see", "nosee");
            }
            dt[PlayerId].mapDisplay.tenSq = "nosee";

        }
        locStorage.setItem(PlayerId + ":mapDisplay", JSON.stringify(dt[PlayerId].mapDisplay));
        var rubbish = "";
    },

    /* Function: getVisTowns
     * Get the coords of all towns visible on the map
     */
    getVisTowns: function () {
        var tList = Object.keys(mapData.t);
        this.addCircle(tList, 10);

    },

    addCircle: function (cList, rad) {
        var sFactor = 500 / ((mapData.zoom * 2) + 1);
        var cx = mapData.x;
        var cy = mapData.y;

        for (i = 0; i < cList.length; i++) {
            thisTown = cList[i].split("|");
            x = thisTown[1];
            y = thisTown[0];
            xCol = mapData.zoom + (x - cx);
            yCol = mapData.zoom - (y - cy);
//            console.log("xCol ",xCol," yCol ",yCol);
            if (dt[PlayerId].mapDisplay.tenSq.onoff == "on") {
                var t = document.querySelector("#tenSq");
                var ctx = t.getContext('2d');
                dt.drawCircle2(ctx, xCol, yCol, sFactor, 10, dt[PlayerId].mapDisplay.tenSq.colour);
            }
            if (dt[PlayerId].mapDisplay.fiveSq.onoff == "on") {
                var t = document.querySelector("#fiveSq");
                var ctx = t.getContext('2d');
                dt.drawCircle2(ctx, xCol, yCol, sFactor, 5, dt[PlayerId].mapDisplay.fiveSq.colour);
            }
            
//ctx.arc(x, y, radius, startAngle, endAngle);
        }
    },

    addTownRadius: function () {
        var mDiv = document.getElementById('mapDiv');
        // Get the canvas element for the cities
        var cCanvas = document.getElementById('mapCities');
        // Create the canvas for the notes
        var townRad = document.createElement('canvas');
        townRad.setAttribute('id', 'townRadCanvas');
        townRad.setAttribute('width', '500');
        townRad.setAttribute('height', '500');
        townRad.setAttribute('style', 'position:absolute;top:11px;left:15px');
        townRad.setAttribute('unselectable', 'on');
        mDiv.insertBefore(cCanvas, townRad);
        townRad = document.getElementById("townRadCanvas");
        var ctx = townRad.getContext('2d');
        // Get the coords of any towns on the map
        var visibleTowns = Object.keys(mapData.t);
        // Loop through each of the towns
        for (t = 0; t < visibleTowns.length; t++) {
//    console.log(mapData.t[visibleTowns[t]]);
            thisTown = {
                x: parseInt(mapData.t[visibleTowns[t]].t.X),
                y: parseInt(mapData.t[visibleTowns[t]].t.Y)
            };
//    console.log(thisTown);
            s = 2 * curMapZoom + 1;
            ctx.scale(500 / s, 500 / s),
                    ctx.translate(-curMapX + curMapZoom, curMapY + curMapZoom);
            ctx.arc(thisTown.x, thisTown.y, 10, 0, 2 * Math.PI);
            ctx.fillStyle = 'red';
            ctx.fill();
        }
    },
    /* Method: collectScan
     * Display the buttons and table for the scan of collactable items.
     */
    collectScan: function () {
        if (this.checkHash()) {
            dt.showTool("#scanCollectableHtml");
        }
    },
    toggleCollect(e) {
        var t = e.target;
        var tId = t.id;
//    console.log(t,tId,t.value,t.checked);
        if (t.checked) {
            console.log("Will show " + tId);
        } else {
            console.log("Will not show " + tId);
        }
        dt.sCollectables[tId] = t.checked;
    },
    /* Function: scanCollectTbl
     * Scan the visible map for any items that can be collected by trade units or
     *  military/NPC camps and display them in a table.
     * 
     * Calls:
     * <addElement2>
     */
    scanCollectTbl: function () {
        // Hex codes for sov colours
        // #A5CE38 - Confed
        // 
        dt.found = {};
        var outputTable = document.querySelector('#collectable tbody');
        // Get the limits for the map.
        var centerCoords = [parseInt(mapData.x), parseInt(mapData.y)];
        var zoomLevel = parseInt(mapData.zoom);
        var xRange = [centerCoords[0] - zoomLevel, centerCoords[0] + zoomLevel];
        if (xRange[0] > xRange[1]) {
            var t = xRange[0];
            xRange[0] = xRange[1];
            xRange[1] = t;
        }
        var yRange = [centerCoords[1] - zoomLevel, centerCoords[1] + zoomLevel];
        if (yRange[0] > yRange[1]) {
            var t = yRange[0];
            yRange[0] = yRange[1];
            yRange[1] = t;
        }
        // Get keys for all the elments of interest from mapData
        var vanKeys = Object.keys(mapData.n);
        var chmsKeys = Object.keys(mapData.d);
        var campKeys = Object.keys(mapData.c);
        var sovKeys = Object.keys(mapData.s);
        // Step through the squares
        for (var thisY = yRange[0]; thisY < yRange[1] + 1; thisY++) {
            for (var thisX = xRange[0]; thisX < xRange[1] + 1; thisX++) {
                var thisSquare = {cotters: [], herbalists: [], miners: [], skinners: [], caravans: [], camps: []};
                var thisKey = thisY + '|' + thisX;
                // Check if there is a caravan collectable
                if ($.inArray(thisKey, vanKeys) > -1) {
                    if (mapData.n[thisKey].hasOwnProperty('rImg')) {
                        var t = mapData.n[thisKey].rImg;
                        var title = mapData.n[thisKey].d;
                        thisSquare.caravans.push([['resIcon', t.split('/')[1] + '_24'], [title]]);
                        dt.found[thisKey] = thisSquare;
                    } else {
                        var t = mapData.n[thisKey].hImg;
                        // "map/flags/T--1"
                        var bits = t.split('/');
                        var numBits = bits.length;
                        t = 'dt-' + bits[numBits - 1];
                        var title = mapData.n[thisKey].rd + 'harvesters';
                        thisSquare.caravans.push([[t], [title]]);
                        dt.found[thisKey] = thisSquare;
                    }
                }
                // Check if there is a encampment collectable
                if ($.inArray(thisKey, campKeys) > -1) {
                    var index = mapData.c[thisKey].i;
                    var title = mapData.c[thisKey].d;
                    if (parseInt(index) == index) {
                        // "//assets.illyriad.net/img/map/flags/44.png"
                        var imgTag = '<img class="simg" src="//assets.illyriad.net/img/map/flags/' + index + '.png"  alt="" title="' + title + '">';
//                        var imgTag = '<img src="http://cdn7.illyriad.com/img/icons/res' + index + '.png"  alt="" title="' + title + '">';
                        thisSquare.camps.push(['dtresIcon', imgTag]);
                    } else {
                        var imgTag = '<img class="dtcamp" src="//assets.illyriad.net/img/map/flags/' + index + '.png"  alt="" title="' + title + '">';
//                        var imgTag = '<img class="dtcamp" src="//assets.illyriad.net/img/map/flags/encampment2.png"  alt="" title="' + title + '">';
                        thisSquare.camps.push(['dtresIcon', imgTag]);
                    }
                    dt.found[thisKey] = thisSquare;
                }
                // Check for cotter, miner, herbalist or skinner items
                if ($.inArray(thisKey, chmsKeys) > -1) {
                    var flags = mapData.d[thisKey];
                    var flagArray = flags.split('|');
                    for (i = 0; i < 9; i++) {
                        if (parseInt(flagArray[i]) === 1) {
                            switch (i) {
                                case 0:
                                    thisSquare.cotters.push([['resIcon', 'hide_24'], ['Hides']]);
                                    break;
                                case 1:
                                    thisSquare.cotters.push([['itemsprite', 'herbs_24'], ['Herbs']]);
                                    break;
                                case 2:
                                    thisSquare.cotters.push([['base', 'crystals_24'], ['Minerals']]);
                                    break;
                                case 3:
                                    thisSquare.cotters.push([['dtequip'], ['Equipment']]);
                                    break;
                                case 4:
                                    thisSquare.skinners.push([['resIcon', 'elemairsalt_24'], ['Elemental Salts']]);
                                    break;
                                case 5:
                                    thisSquare.herbalists.push([['resIcon', 'spices_24'], ['Rare Herbs']]);
                                    break;
                                case 6:
                                    thisSquare.miners.push([['resIcon', 'gems_24'], ['Rare Minerals']]);
                                    break;
                                case 7:
                                    thisSquare.cotters.push([['resIcon', 'grapes_24'], ['Grapes']]);
                                    break;
                                case 8:
                                    thisSquare.skinners.push([['resIcon', 'paws_24'], ['Animal Parts']]);
                                    break;
                            }
                        }
                    }
                    dt.found[thisKey] = thisSquare;
                }
            } // end of x loop
        } // end of y loop
        var b = "";
        var foundkeys = Object.keys(dt.found);
        var numFound = foundkeys.length;
        for (var i = 0; i < numFound; i++) {
            thisKey = foundkeys[i];
            var cArray = thisKey.split('|');
            var href = 'href="#/World/Map/' + cArray[1] + '/' + cArray[0] + '/5"';

            var thisSquare = dt.found[thisKey];
// check if there are any collectables to be displayed in this row
            if (this.showRow(thisSquare)) {
                // Add a row to the table
                var row = outputTable.insertRow(-1);
                var newCell = row.insertCell(0);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);

                newElem.textContent = dt.coordSwap(thisKey);
                var newCell = row.insertCell(1);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);

                // Build the Caravan cell
                var numEntries = thisSquare.caravans.length;
                if (numEntries > 0 && dt.sCollectables.sVan) {
                    for (var n = 0; n < numEntries; n++) {
                        var classes = thisSquare.caravans[n][0].join(' ');
                        var title = thisSquare.caravans[n][1];
                        var newSpan = dt.addElement2(newElem, 'span', ['class', 'title'], [classes, title]);
                        newSpan.setAttribute('title', title);
                    }
                }
                var newCell = row.insertCell(2);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                // Build the Cotters cell
                var numEntries = thisSquare.cotters.length;
                if (numEntries > 0 && dt.sCollectables.sCott) {
                    for (var n = 0; n < numEntries; n++) {
                        var thistitle = thisSquare.cotters[n][1];
                        var classes = thisSquare.cotters[n][0].join(' ');
                        var newSpan = dt.addElement2(newElem, 'span', ['class'], [classes]);
                        newSpan.setAttribute('title', thistitle);
                        if (thistitle == "Equipment") {
                            newSpan.innerHTML = '<img class="simg" src="//assets.illyriad.net/img/icons/equip_48.png" />';
                        }
                    }
                }
                var newCell = row.insertCell(3);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                // Build the Skinners cell
                var numEntries = thisSquare.skinners.length;
                if (numEntries > 0 && dt.sCollectables.sSkin) {
                    for (var n = 0; n < numEntries; n++) {
                        var thistitle = thisSquare.skinners[n][1];
                        var classes = thisSquare.skinners[n][0].join(' ');
                        var newSpan = dt.addElement2(newElem, 'span', ['class'], [classes]);
                        newSpan.setAttribute('title', thistitle);
                    }
                }
                var newCell = row.insertCell(4);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                // Build the Herbalists cell
                var numEntries = thisSquare.herbalists.length;
                if (numEntries > 0 && dt.sCollectables.sHerb) {
                    for (var n = 0; n < numEntries; n++) {
                        var thistitle = thisSquare.herbalists[n][1];
                        var classes = thisSquare.herbalists[n][0].join(' ');
                        var newSpan = dt.addElement2(newElem, 'span', ['class'], [classes]);
                        newSpan.setAttribute('title', thistitle);
                    }
                }
                var newCell = row.insertCell(5);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                // Build the Miners cell
                var numEntries = thisSquare.miners.length;
                if (numEntries > 0 && dt.sCollectables.sMin) {
                    for (var n = 0; n < numEntries; n++) {
                        var thistitle = thisSquare.miners[n][1];
                        var classes = thisSquare.miners[n][0].join(' ');
                        var newSpan = dt.addElement2(newElem, 'span', ['class'], [classes]);
                        newSpan.setAttribute('title', thistitle);
                    }
                }
                var newCell = row.insertCell(6);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                // Build the Encampments cell
                var numEntries = thisSquare.camps.length;
                if (numEntries > 0 && dt.sCollectables.sCamp) {
                    for (var n = 0; n < numEntries; n++) {
                        var newSpan = dt.addElement2(newElem, 'span', [], []);
                        newSpan.innerHTML = thisSquare.camps[n][1];
                    }
                }
                var newCell = row.insertCell(7);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                var sourceCoords = [townX, townY];
                var coords = thisKey.split('|');
                coords[0] = parseInt(coords[0]);
                coords[1] = parseInt(coords[1]);
                var dist = dt.hypot(sourceCoords, coords.reverse()).toFixed(1);
                newElem.textContent = dist;
                // Add the sov status
                var newCell = row.insertCell(8);
                newElem = dt.addElement2(newCell, 'a', ['onclick'], [href]);
                // Check if there is an entry in the sovKeys array for this location
                if (sovKeys.indexOf(thisKey) > -1) {
                    var thisSov = mapData.s[thisKey];
                    // Create the title for this element
                    var title = thisSov.a + " " + thisSov.rd;
                    var newSpan = dt.addElement2(newElem, 'span', ['style', 'title'], ['background-color: yellow', title]);
                    newSpan.innerHTML = thisSov.pn;
                }
            }
        }
//		})
        // Having built the table we need to check the height of it and adjust the window height if needed.
        var id = "#scanCollectableHtml";
        var winHeight = window.innerHeight;
        var maxHt = parseInt(0.75 * winHeight);
        var el = document.querySelector('div.ui-dialog > ' + id).parentElement;

        var intElemOffsetHeight = el.offsetHeight;
        var currHeight = $(id).dialog("option", "height");
        if (intElemOffsetHeight > maxHt) {
            $(id).dialog("option", "height", maxHt);
        }
        dt.adjustTool("#scanCollectableHtml");

    },
    showRow: function (thisSq) {
        var rubbish = "";
        var s = 0;
        if (thisSq.caravans.length > 0 && dt.sCollectables.sVan) {
            s = 1;
        }
        if (thisSq.camps.length > 0 && dt.sCollectables.sCamp) {
            s = 1;
        }
        if (thisSq.cotters.length > 0 && dt.sCollectables.sCott) {
            s = 1;
        }
        if (thisSq.herbalists.length > 0 && dt.sCollectables.sHerb) {
            s = 1;
        }
        if (thisSq.miners.length > 0 && dt.sCollectables.sMin) {
            s = 1;
        }
        if (thisSq.skinners.length > 0 && dt.sCollectables.sSkin) {
            s = 1;
        }
        return s;
    },
    clearCollectTbl: function () {
        dt.found = {};
//		document.querySelector('#dt_input').setAttribute('class', 'nosee');
//	$( "#dt_output" ).load( dt.server+"/html/durctools3.html #scanCollectableHtml" );
//$( "#dt_output" ).load(  dt.server+"/html/durctools3.html #scanCollectableHtml", function() {
//  alert( "Load was performed." );
//});
// html body div#durcTools3.see div#dt_output.see div#scanCollectableHtml table#collectable.see thead tr
        var tblBody = document.querySelectorAll('#collectable tbody');
//var element = document.getElementById("top");
        dt.removeRows(document.querySelector('#collectable'));
//		while (tblBody.firstChild) {
//			tblBody.removeChild(element.firstChild);
//		}
//for(var i=1;i<tblRows.length;i++){

//}
//		document.querySelector('#dt_output').setAttribute('class', 'see');

    },
    tapeMeasure: function () {
        if (this.checkHash()) {
            dt.showTool("#distMeasureHtml");
            this.tapeMeasure.start = [0, 0];
            this.tapeMeasure.end = [0, 0];
        }
    },
    getCoords: function () {
        // Find which options are currently selected and set the coords accordingly.
        var startAt = document.querySelector('input[name="startSource"]:checked').value;
        var endAt = document.querySelector('input[name="endSource"]:checked').value;
        switch (startAt) {
            case 'Current Town':
                dt.mapTools.tapeMeasure.start[0] = dt[PlayerId].towns[CurrentTown].x;
                dt.mapTools.tapeMeasure.start[1] = dt[PlayerId].towns[CurrentTown].y;
                break;
            case 'Map Center':
                dt.mapTools.tapeMeasure.start[0] = mapData.x;
                dt.mapTools.tapeMeasure.start[1] = mapData.y;
                break;
            case 'Enter Coords':
                var t = document.getElementById('startX').value;
                dt.mapTools.tapeMeasure.start[0] = t.replace("X", "");
                var t = document.getElementById('startY').value;
                dt.mapTools.tapeMeasure.start[1] = t.replace("Y", "");
                break;
        }
        switch (endAt) {
            case 'Current Town':
                dt.mapTools.tapeMeasure.end[0] = dt[PlayerId].towns[CurrentTown].x;
                dt.mapTools.tapeMeasure.end[1] = dt[PlayerId].towns[CurrentTown].y;
                break;
            case 'Map Center':
                dt.mapTools.tapeMeasure.end[0] = mapData.x;
                dt.mapTools.tapeMeasure.end[1] = mapData.y;
                break;
            case 'Enter Coords':
                var t = document.getElementById('endX').value;
                dt.mapTools.tapeMeasure.end[0] = t.replace("X", "");
                var t = document.getElementById('endY').value;
                dt.mapTools.tapeMeasure.end[1] = t.replace("Y", "");
                break;
        }
        dt.mapTools.calcHypot();
    },
    calcHypot: function () {
        var dist = dt.hypot(this.tapeMeasure.start, this.tapeMeasure.end).toFixed(2);
        var resultText = 'Distance between [' + this.tapeMeasure.start[0] + '|' + this.tapeMeasure.start[1] + '] and [' + this.tapeMeasure.end[0] + '|' + this.tapeMeasure.end[1] + '] is ' + dist + ' squares.';
        document.getElementById('result').textContent = resultText;
    },
    toggleManEntry: function (where) {
        var startX = document.getElementById('startX');
        var startY = document.getElementById('startY');
        var endX = document.getElementById('endX');
        var endY = document.getElementById('endY');
        switch (where) {
            case 'startTown':
                startX.setAttribute('disabled', 'disabled');
                startY.setAttribute('disabled', 'disabled');
                break;
            case 'startMap':
                startX.setAttribute('disabled', 'disabled');
                startY.setAttribute('disabled', 'disabled');
                break;
            case 'startManual':
                startX.removeAttribute('disabled', 'disabled');
                startY.removeAttribute('disabled', 'disabled');
                break;
            case 'endTown':
                endX.setAttribute('disabled', 'disabled');
                endY.setAttribute('disabled', 'disabled');
                break;
            case 'endMap':
                endX.setAttribute('disabled', 'disabled');
                endY.setAttribute('disabled', 'disabled');
                break;
            case 'endManual':
                endX.removeAttribute('disabled', 'disabled');
                endY.removeAttribute('disabled', 'disabled');
                break;
        }
    },
    /* Function: resScan
     * Display and enable the form for input of the ranges for basic resources.
     * 
     */
    resScan: function () {
        if (this.checkHash()) {
            var ranges = {minW: [0, 25, 0], minC: [0, 25, 0], minI: [0, 25, 0], minS: [0, 25, 0], minF: [0, 25, 0], maxW: [0, 25, 25], maxC: [0, 25, 25], maxI: [0, 25, 25], maxS: [0, 25, 25], maxF: [0, 25, 25]};
// Display the tool window with the form for setting the search parameters.
            dt.showTool("#spotScanHtml");
            var everythingLoaded = setInterval(function () {
                if (/loaded|complete/.test(document.readyState)) {
                    clearInterval(everythingLoaded);

//		$(document).ready(function resScanLoaded() {
                    var resOptKeys = Object.keys(ranges);
                    for (var i = 0; i < resOptKeys.length; i++) {
                        var thisOpt = document.getElementById(resOptKeys[i]);
                        var min = ranges[resOptKeys[i]][0];
                        var max = ranges[resOptKeys[i]][1];
                        var use = ranges[resOptKeys[i]][2];
                        for (var val = min; val <= max; val++) {
                            var newOpt = dt.addElement2(thisOpt, 'option', ['value'], [val]);
                            newOpt.textContent = val;
                            if (val == use) {
                                newOpt.setAttribute('selected', '');
                            }
                        }
//                                        var newOpt = dt.addElement2(thisOpt, 'option', ['value'], "99");
//                                        newOpt.textContent = '-';
                    }
//			dt.outputArea.innerHtml = "";
                    //	durc_input.innerHTML = durc.contents.resRange;
                    cbtAcceptOpt = document.getElementById('sqTerrain');
                    centAcceptOpt = document.getElementById('cTerrain');
                    var t = Object.keys(dt.cbtTerrains);
                    var l = t.length;
                    for (i = 0; i < l; i++) {
                        option = document.createElement('option');
                        option.setAttribute('value', t[i]);
                        option.appendChild(document.createTextNode(t[i]));
                        cbtAcceptOpt.appendChild(option);
                    }
                    for (i = 0; i < l; i++) {
                        option = document.createElement('option');
                        option.setAttribute('value', t[i]);
                        option.appendChild(document.createTextNode(t[i]));
                        centAcceptOpt.appendChild(option);
                    }
//			dt.inputArea.setAttribute('class', 'see');
//			dt.outputArea.setAttribute('class', 'nosee');
//		});

//    init(); // this is the function that gets called when everything is loaded
                }
            }, 10);

        }
    },
    /*Function: resourceScan
     * Checks the visible map for squares that match given resource values and 
     * terrain classes (both on the square and adjacent).
     */
    resourceScan: function () {
//	dt.callerId(resourceScan.caller);
        //dt.ajaxReq("resTable",'dt_output');

// Now wait for the map etc to have finished loading
        var everythingLoaded = setInterval(function () {
            if (/loaded|complete/.test(document.readyState)) {
                clearInterval(everythingLoaded);

//	$(document).ready(function resourceScanLoaded() {
                // Check if the sq matches a required terrain
                var outArea = document.querySelector('#spotScanResultsHtml tbody');
//		outArea.setAttribute(['class'], ['see']);
//		var newElem = dt.addElement2(outArea, 'p', [], []);
//		newElem.textContent = "Checking terrain classes...";
                var wanted = [];
                var possible = [];
                var resLimits = [];
                // Get the options that have been selected
                var x = document.getElementById('resLimits');
                for (i = 0; i < 10; i++) {
                    resLimits[i] = parseInt(x.elements[i].value);
                }

                var t = x.elements[10];
                if (t.options[0].selected == true) {
                    wanted = Object.keys(dt.cbtTerrains);
                } else {
                    for (var i = 0; i < t.options.length; i++) {
                        if (t.options[i].selected == true) {
                            var name = t.options[i].value;
                            wanted.push(name);
                        }
                    }
                }
                // Get the area to be checked (1 sq inside the map limits)
                var newElem = dt.addElement2(outArea, 'p', [], []);
                newElem.textContent = "Checking resource levels...";
                var centerCoords = [parseInt(mapData.x), parseInt(mapData.y)];
                var zoomLevel = parseInt(mapData.zoom) - 1;
                var xRange = [centerCoords[0] - zoomLevel, centerCoords[0] + zoomLevel];
                if (xRange[0] > xRange[1]) {
                    var t = xRange[0];
                    xRange[0] = xRange[1];
                    xRange[1] = t;
                }
                var yRange = [centerCoords[1] - zoomLevel, centerCoords[1] + zoomLevel];
                if (yRange[0] > yRange[1]) {
                    var t = yRange[0];
                    yRange[0] = yRange[1];
                    yRange[1] = t;
                }
                // Get the info from the mapData object.
                var mapInfo = mapData.data;
                // Filter the squares for those with acceptable terrain types.
                for (var thisY = yRange[0]; thisY < yRange[1] + 1; thisY++) {
                    for (var thisX = xRange[0]; thisX < xRange[1] + 1; thisX++) {
                        var thisKey = thisY + '|' + thisX;
                        if (mapData.data.hasOwnProperty(thisKey)) {
                            var terrainCode = mapData.data[thisKey].i;
                            if (wanted.lastIndexOf(terrain[terrainCode].combat) > -1) {
                                possible.push(thisKey);
                            }
                        }
                    }
                }
                // Filter the remaing squares for acceptable resource levels.
                var rubbish = '';
                var matchRes = [];
                var numSqs = possible.length;
                for (var thisSq = 0; thisSq < numSqs; thisSq++) {
                    var theRes = mapData.data[possible[thisSq]].rs;
                    var resArray = theRes.split('|');
                    var matched = 0;
                    for (var i = 0; i < 5; i++) {
                        // Check each of the resources
                        if (resArray[i] >= resLimits[i] && resArray[i] <= resLimits[i + 5]) {
                            matched = matched + 1;
                        }
                    }
                    if (matched == 5) {
                        matchRes.push(possible[thisSq]);
                    }
                }
                // Check the squares adjacent to each of the remaining squares for the terrain type
                var newElem = dt.addElement2(outArea, 'p', [], []);
                newElem.textContent = "Checking surrounding squares...";
                var t = x.elements[11];
                if (t.options[0].selected == true) {
                    wanted = Object.keys(dt.cbtTerrains);
                } else {
                    for (var i = 0; i < t.options.length; i++) {
                        if (t.options[i].selected == true) {
                            var name = t.options[i].value;
                            wanted.push(name);
                        }
                    }
                }
                var numSqs = matchRes.length;
                fullMatch = [];
                for (var i = 0; i < numSqs; i++) {
                    var coords = matchRes[i].split('|');
                    var xRange = [parseInt(coords[1]) - 1, parseInt(coords[1]) + 1];
                    if (xRange[0] > xRange[1]) {
                        var t = xRange[0];
                        xRange[0] = xRange[1];
                        xRange[1] = t;
                    }
                    var yRange = [parseInt(coords[0]) - 1, parseInt(coords[0]) + 1];
                    if (yRange[0] > yRange[1]) {
                        var t = yRange[0];
                        yRange[0] = yRange[1];
                        yRange[1] = t;
                    }
                    var matchCount = 0;
                    for (var thisY = yRange[0]; thisY < yRange[1] + 1; thisY++) {
                        for (var thisX = xRange[0]; thisX < xRange[1] + 1; thisX++) {
                            var thisKey = thisY + '|' + thisX;
                            if (thisKey != matchRes[i]) {
                                var terrainCode = mapData.data[thisKey].i;
                                if (wanted.lastIndexOf(terrain[terrainCode].combat) > -1) {
                                    ++matchCount;
                                }
                            }
                        }
                    }
                    if (matchCount == 8) {
                        fullMatch.push(matchRes[i]);
                    }
                }
                var rubbish = '';
                var outString = fullMatch.join(', ');
//		outArea.textContent=outString;
                var numMatches = fullMatch.length;
                outArea.innerHTML = '';
                var resTable = document.querySelector("#spotScanResultsHtml table.oddeven tbody");
                for (var i = 0; i < numMatches; i++) {
                    var row = resTable.insertRow(-1);
                    var cArray = fullMatch[i].split('|');
                    var href = 'href="#/World/Map/' + cArray[1] + '/' + cArray[0] + '/5"';

                    var cell = row.insertCell(0);
                    var newElem = dt.addElement2(cell, 'a', ['onclick'], [href]);
                    newElem.textContent = dt.coordSwap(fullMatch[i]);
                    cell = row.insertCell(1);
                    newElem = dt.addElement2(cell, 'a', ['onclick'], [href]);
                    var tName = terrain[mapData.data[fullMatch[i]].i].combat
                    newElem.textContent = tName;
                    cell = row.insertCell(2);
                    newElem = dt.addElement2(cell, 'a', ['onclick'], [href]);
                    newElem.textContent = mapData.data[fullMatch[i]].rs;
                    cell = row.insertCell(3);
                    newElem = dt.addElement2(cell, 'span', [], []);
                    newElem.textContent = 'DETAILED';
                    cell = row.insertCell(4);
                    newElem = dt.addElement2(cell, 'span', [], []);
                    newElem.textContent = 'SAVE';

                }
                dt.adjustTool("#spotScanHtml");
                var fullString = JSON.stringify(fullMatch);
            }
//	}
        }, 10);

    },
    clearResTbl: function () {
        dt.removeRows(document.querySelector("#spotScanResultsHtml table"));
    }
};

dt.scanFactionTroops = function () {
    var msg = "Scanning for faction troops.";
    dt.logEntry(msg);
//	var found = false;
    var everythingLoaded = setInterval(function () {
        if (/loaded|complete/.test(document.readyState)) {
            clearInterval(everythingLoaded);
//	 dt.production.setup();
//	 if(!dt[PlayerId].settings.chkFreq){
//		 dt[PlayerId].settings.chkFreq = 300000;
//	 }
//	dt.prodCheckTimer = window.setInterval(dt.production.updateQueues, dt[PlayerId].settings.chkFreq);
//	 
////    init(); // this is the function that gets called when everything is loaded
//  }
//}, 10);	
            var found = false;
            $(document).ready(function scanFactionTroopsLoaded() {
                var camps = Object.keys(mapData.c);
                for (var i = 0; i < camps.length; i++) {
                    var t = mapData.c[camps[i]].d;
                    if (t.includes('Occupying')) {
                        var v = t.substring(0, 3);
                        switch (v) {
                            case 'You':
                            case 'Con':
                            case 'NAP':
                            case 'Neu':
                            case 'All':
                                // Not interested in these
                                break;
                            default:
                                var s = camps[i];
                                var who = t.replace(' Occupying Force', '');
//						dt.factionTroops[s] = {whereat: s, name: who, by: PlayerId, when: Date()};
                                dt.ajaxFactionTroops({whereat: s, name: who, by: PlayerId, when: Date()});
//			dt.ajaxFactionTroops(dt.factionTroops);
//			dt.factionTroops = {};
                                found = true;
                        }
                    }
                }
                if (found === true) {
//			dt.ajaxFactionTroops(dt.factionTroops);
//			dt.factionTroops = {};
                }
            })
        }
    }, 10);

    var msg = "Faction troop scan finished.";
    dt.logEntry(msg);

};
