/* File: production6.js
 * Functions to manage production queues and alerts.
 * Data on the queues is stored in the format:
 * {bId:buildingId,Building:buildingName,item:itemName,endsat:endingTimestamp}
 * The data is no longer stored in separate areas. rather those areas will be recreated for display purposes.
 */
dt.production = {

    setup: function () {
        dt.production.fullCompleteCheck();
        dt.production.updateFlag();
    },

    /* Method: parseBldgProd
     * Parse the intercept of a 'BuildingProduction' page
     * 
     */
    parseBldgProd: function (xhr, event, bits) {
        if (!docfrag) {
            var docfrag = document.createDocumentFragment();
            var d = document.createElement("div");
            docfrag.appendChild(d);
        }
        d.innerHTML = xhr.responseText;
        var infodivs = d.querySelectorAll("div.info");
        // The building
        var bId = event.currentTarget.URL.split("/")[6];
        var bName = dt.prodBuildings[bId].name;
        var prodArea = dt.prodBuildings[bId].class;
        var thisQ = [];
        if (infodivs.length > 1) {
            // The queued items
            var queues = infodivs[1].querySelectorAll("tr");
            for (let r = 1; r < queues.length; r++) {
                var thisR = queues[r].querySelectorAll("td");
                var itemid = thisR[0].textContent;
                var itemName = thisR[1].textContent;
                var data = thisR[5].querySelector("div").getAttribute('data').split('|');
                var startDate = new Date(parseInt(data[0]));
                var endDate = new Date(parseInt(data[1]));
                var thisI = {'bId': bId, "type": dt.prodBuildings[bId].class, 'Building': bName, 'item': itemName, endsAt: data[1]};
                thisQ.push(thisI);
            }
            data = startDate = endDate = docfrag = null;
        }
//        console.log(dt[PlayerId].towns[CurrentTown].name, bName + " Production: ", thisQ);
        this.compareQs(thisQ, dt[PlayerId].towns[CurrentTown].prod, CurrentTown, bId);
        this.updateQs(thisQ, dt[PlayerId].towns[CurrentTown].prod, CurrentTown);
//        dt[PlayerId].towns[CurrentTown].prod = townProd;
        var rubbish = "";

    },
    /* Method: parseTownProd
     * Parse the intercept of a 'Production' page
     */
    parseTownProd: function (xhr, event, bits) {
        if (!docfrag) {
            var docfrag = document.createDocumentFragment();
            var d = document.createElement("div");
            docfrag.appendChild(d);
        }
        d.innerHTML = xhr.responseText;
        var infodiv = d.getElementsByClassName("info")[0];

        var townProd = [];
        var fieldsets = infodiv.querySelectorAll("fieldset");
        var numFieldsets = fieldsets.length;
        var thisTPBs = [];
        for (var i = 0; i < numFieldsets; i++) {
            var thisFieldset = fieldsets[i];
            var bId = thisFieldset.querySelector("a").getAttribute("href").split('/')[3];
            thisTPBs.push(bId);
            var bName = dt.prodBuildings[bId].name;
            // Process the rows to find any items being produced
            // The queued items
            var trs = thisFieldset.querySelectorAll("tr");
            for (let r = 1; r < trs.length; r++) {
                var thisR = trs[r].querySelectorAll("td");
                var itemid = thisR[0].textContent;
                var itemName = thisR[1].textContent;
                var data = thisR[5].querySelector("div").getAttribute('data').split('|');
                var startDate = new Date(parseInt(data[0]));
                var endDate = new Date(parseInt(data[1]));
                var thisI = {'bId': bId, "type": dt.prodBuildings[bId].class, 'Building': bName, 'item': itemName, endsAt: data[1]};
                townProd.push(thisI);
            }
        }
//        var key = CurrentTown + ':prod';
//        locStorage.setItem(key, JSON.stringify(dt[PlayerId].towns[CurrentTown].prod));
//        data = startDate = endDate = docfrag = null;
//        console.log(dt[PlayerId].towns[CurrentTown].name + " Town Production: ", townProd);
        this.compareQs(townProd, dt[PlayerId].towns[CurrentTown].prod, CurrentTown, thisTPBs);
        this.updateQs(townProd, dt[PlayerId].towns[CurrentTown].prod, CurrentTown);
//        dt[PlayerId].towns[CurrentTown].prod = townProd;
        var rubbish = "";
//        return bldgs;

    },

    /* Method: parseUnitProd
     * 
     */
    parseUnitProd: function (xhr, event, bits) {
        if (!docfrag) {
            var docfrag = document.createDocumentFragment();
            var d = document.createElement("div");
            docfrag.appendChild(d);
        }
        d.innerHTML = xhr.responseText;
        var infodivs = d.querySelectorAll("div.info");
        // The building
        var bId = event.currentTarget.URL.split("/")[6];
        var bName = dt.prodBuildings[bId].name;
        var prodArea = dt.prodBuildings[bId].class;
        var thisQ = [];
        if (infodivs.length > 1) {
            // The queued items
            var queues = infodivs[1].querySelectorAll("tr");
            for (let r = 1; r < queues.length; r++) {
                var thisR = queues[r].querySelectorAll("td");
                var itemid = thisR[0].textContent;
                var itemName = thisR[1].textContent;
                var data = thisR[5].querySelector("div").getAttribute('data').split('|');
                var startDate = new Date(parseInt(data[0]));
                var endDate = new Date(parseInt(data[1]));
                var thisI = {'bId': bId, "type": dt.prodBuildings[bId].class, 'Building': bName, 'item': itemName, endsAt: data[1]};
                thisQ.push(thisI);
            }
            data = startDate = endDate = docfrag = null;
        }
        console.log(dt[PlayerId].towns[CurrentTown].name, bName + " Production: ", thisQ);
        this.compareQs(thisQ, dt[PlayerId].towns[CurrentTown].prod, CurrentTown, bId);
        this.updateQs(thisQ, dt[PlayerId].towns[CurrentTown].prod, CurrentTown);

        var rubbish = "";
    },

    /* Check complete times
     * A simple check of the 'endsAt' property against the current datetime.
     * 
     * Parameters:
     * tId - integer. The id of the town to check.
     * list - array. An array holding the indexes of the elements in the src to check. If not present then check all elements.
     */
    checkComplete: function (tId, list) {
        var currentTS = Date.now();
        var finished = [];
        var townProd = dt[PlayerId].towns[tId].prod;
        for (let i = 0; i < townProd.length; i++) {
            if (list) {
                if (townProd[i].endsAt < currentTS && list.includes(i)) {
                    finished.push(i);
                }
            } else {
                if (townProd[i].endsAt < currentTS) {
                    finished.push(i);
                }
            }
        }
        finished.reverse();
        finished.forEach(function (element) {
//            console.log(element);
            dt.production.makeNotify(tId, element, townProd);
        });
    },
    /* Check prod queues in all towns for completed entries.
     * 
     */
    fullCompleteCheck: function () {
        for (t = 0; t < dt[PlayerId].towns.list.length; t++) {
            dt.production.checkComplete(dt[PlayerId].towns.list[t]);
        }
    },

    updateQs: function (newQ, oldQ, tId) {
//        console.log("Updating queues at " + dt[PlayerId].towns[tId].name);
//        console.log(dt[PlayerId].towns[tId].prod);
        for (let i = 0; i < newQ.length; i++) {
            var newE = newQ[i];
            var matched = false;
            for (let j = 0; j < oldQ.length; j++) {
                if (JSON.stringify(newE) === JSON.stringify(oldQ[j])) {
                    matched = true;
                }
            }
            if (matched === false) {
//                console.log("No match for ", newQ[i]);
                if (dt[PlayerId].towns[tId].prod === {}) {
                    dt[PlayerId].towns[tId].prod = [];
                }
                dt[PlayerId].towns[tId].prod.push(newE);
//                this.makeNotify(tId, i, oldQ);
            }
        }
//        console.log(dt[PlayerId].towns[tId].prod);
        locStorage.setItem(tId + ":prod", JSON.stringify(dt[PlayerId].towns[tId].prod));
    },
    compareQs: function (newQ, oldQ, tId, bId) {
//        console.log("Comparing queues at " + dt[PlayerId].towns[tId].name);
        for (let i = 0; i < oldQ.length; i++) {
            if (bId) {
                if (oldQ[i].bId === bId) {
                    var oldE = oldQ[i];
                    var matched = false;
                    for (let j = 0; j < newQ.length; j++) {
                        if (JSON.stringify(oldE) === JSON.stringify(newQ[j])) {
                            matched = true;
                        }
                    }
                    if (matched === false) {
                        console.log("No match for ", oldQ[i]);
                        this.makeNotify(tId, i, oldQ);
                    }
                }
            } else {
                var oldE = oldQ[i];
                var matched = false;
                for (let j = 0; j < newQ.length; j++) {
                    if (JSON.stringify(oldE) === JSON.stringify(newQ[j])) {
                        matched = true;
                    }
                }
                if (matched === false) {
                    console.log("No match for ", oldQ[i]);
                    dt.production.makeNotify(tId, i, oldQ);
                }
            }
        }
    },
    /* Make a notify entry and remove the entry from the town production array
     * 
     */
    makeNotify: function (tId, index, townProd) {
        thisE = townProd[index];
//        console.log("Notify done for ", thisE, " town id: " + tId);
        var prodN = {logged: true, seen: false, "body": "Production of <i>" + thisE.item + "</i> completed in <i>" + thisE.Building + "</i> at <i>" + dt[PlayerId].towns[tId].name + "</i>."};
        dt[PlayerId].prodNotices.unshift(prodN);
        dt.production.updateFlag();
        dt[PlayerId].towns[tId].prod.splice(index, 1);
        locStorage.setItem(tId + ":prod", JSON.stringify(dt[PlayerId].towns[tId].prod));
        locStorage.setItem([PlayerId] + ":prodNotices", JSON.stringify(dt[PlayerId].prodNotices));
    },
    /* Method: updateFlag
     * Updates the count of unseen production notifications.
     */
    updateFlag: function () {
        var unseen = 0;
        dt[PlayerId].prodNotices.forEach(function (entry) {
            if (entry.seen === false) {
                unseen++;
            }
        });
        document.getElementById("prodflag").textContent = unseen;
        document.getElementById("prodflag").setAttribute("title", unseen + " unread Production Alerts");
        if (dt.popupStates.AllQs) {
            dt.showTool("#showAllQueuesHtml", "AllQs");
        }
    },
    showProdAlerts: function () {
        var prodAlertPopUp = document.getElementById("prodAlertsHtml");
        var tBody = prodAlertPopUp.getElementsByTagName("tbody")[1];
        while (tBody.rows.length > 0) {
            tBody.deleteRow(0);
        }
        var numAlerts = dt[PlayerId].prodNotices.length;
        for (i = 0; i < numAlerts; i++) {
//        dt[PlayerId].prodNotices.forEach(function (item) {
            var item = dt[PlayerId].prodNotices[i];
            var row = tBody.insertRow(-1);
            var cell = row.insertCell(0);
            // <input type="checkbox" id="cbox2" value="second_checkbox" checked="checked">
            var cBox = dt.addElement2(cell, "input", ["type", "value"], ["checkbox", i]);
            cBox.setAttribute("onchange", 'dt.production.alertDelHandler(' + i + ')');
            cell = row.insertCell(1);
            if (!item.seen) {
                item.seen = true;
                cell.innerHTML = "<strong>" + item.body + "</strong>";
            } else {
                cell.innerHTML = item.body;
            }
        }
        ;
        var key = PlayerId + ':prodNotices';
        locStorage.setItem(key, JSON.stringify(dt[PlayerId].prodNotices));
        var key = CurrentTown + ':prod';
        locStorage.setItem(key, JSON.stringify(dt[PlayerId].towns[CurrentTown].prod));
        var unseen = 0;
        document.getElementById("prodflag").textContent = unseen;
        document.getElementById("prodflag").setAttribute("title", unseen + " unread Production Alerts");
        dt.popupStates.ProdNotices = true;
        dt.showTool("#prodAlertsHtml", "ProdNotices");
    },
    showIdle: function () {
        dt.popupStates.IdleQs = true;
        if (!document.getElementById("showIdleBldgsHtml")) {
            dt.addElement(document.getElementsByTagName("body")[0], "div", ["class", "id", "title"], ["nosee", "showIdleBldgsHtml", "Idle Production Buildings"]);
            var t = document.getElementById("showIdleBldgsHtml");
            t.innerHTML = this.idleQs;
        }
        var twns = dt[PlayerId].towns.list;
        var idleTbl = document.getElementById('idleQueuesTbl');
        TableSort.init('idleQueuesTbl');
        var tBody = idleTbl.getElementsByTagName("tbody")[0];
        while (tBody.rows.length > 0) {
            tBody.deleteRow(0);
        }
        var idle = [];
        for (var t = 0; t < twns.length; t++) {
            console.log(dt[PlayerId].towns[twns[t]].name);
            var thisTprod = dt[PlayerId].towns[twns[t]].prod;
            var bldgs = [];
            var k = Object.keys(dt.prodBuildings);
            k.forEach(function (bId) {
                bldgs.push(dt.prodBuildings[bId].name)
            });
            var busy = [];
            thisTprod.forEach(function (thisProd) {
                busy[thisProd.bId] = thisProd.Building;
                var ind = bldgs.indexOf(thisProd.Building);
                if (ind > -1) {
                    bldgs.splice(ind, 1);
                }
                console.log(thisProd);
            });
            for (let i = 0; i < bldgs.length; i++) {
                idle.push([dt[PlayerId].towns[twns[t]].name, bldgs[i]]);
            }
            console.log(busy, bldgs, idle);

//            var thisTownProdBldgs = Object.keys(dt[PlayerId].towns[twns[t]].prod.items);
            for (var i = 0; i < idle.length; i++) {
//                if (dt[PlayerId].towns[twns[t]].prod.items[bldgId].endsAt === "Empty") {
                var row = tBody.insertRow(-1);
                var cell = row.insertCell(0);
                cell.textContent = idle[i][0];
                cell = row.insertCell(1);
                cell.textContent = idle[i][1];
//                }
            }
        }
        if (dt.popupStates.IdleQs) {
            dt.showTool("#showIdleBldgsHtml", "IdleQs");
        }
    },

    getTownProdBldgs: function (tId) {
        var tBldgs = dt[PlayerId].towns[tId].plots.internal;
        var thisTownBldgs = []
        for (var p in tBldgs) {
            if (tBldgs.hasOwnProperty(p)) {
                thisTownBldgs.push(tBldgs[p][0]);
            }
        }
        var k = Object.keys(dt.prodBuildings);
//        console.log("Town bldgs ", thisTownBldgs);
        var twnProdBldgs = [];
        k.forEach(function (ind) {
            if (thisTownBldgs.includes(dt.prodBuildings[ind].name)) {
//                twnProdBldgs.push(dt.prodBuildings[ind]);
                twnProdBldgs[ind] = dt.prodBuildings[ind];
            }
        });
        return twnProdBldgs;
    },

    showAllQueues: function () {
        $("#allQsTable").tabs();

        var twns = dt[PlayerId].towns.list;
        var allTbl = document.getElementById('allQsTbl');
        var itemsTbl = document.getElementById('itemsQsTbl');
        var diploTbl = document.getElementById('diploQsTbl');
        var milTbl = document.getElementById('milQsTbl');
        var tradeTbl = document.getElementById('tradeQsTbl');
        var prodTabs = {
            "all": {"tbl": allTbl, "tblId": "allQsTbl"},
            "items": {"tbl": itemsTbl, "tblId": "itemsQsTbl"},
            "diplo": {"tbl": diploTbl, "tblId": "diploQsTbl"},
            "mil": {"tbl": milTbl, "tblId": "milQsTbl"},
            "trade": {"tbl": tradeTbl, "tblId": "tradeQsTbl"}
        };
        var tabKeys = Object.keys(prodTabs);
        for (var i = 0; i < tabKeys.length; i++) {
            tbl = prodTabs[tabKeys[i]].tbl;
            TableSort.init(prodTabs[tabKeys[i]].tblId);
            var tBody = document.querySelector("#" + prodTabs[tabKeys[i]].tblId + " tbody");
//            var tBody = tbl.getElementsByTagName("tbody")[0];
            while (tBody.rows.length > 0) {
                tBody.deleteRow(0);
            }
        }

        var tabContent = "items";
        var tBody1 = prodTabs.all.tbl.getElementsByTagName("tbody")[0];
        var tBody2 = prodTabs.items.tbl.getElementsByTagName("tbody")[0];

        for (var t = 0; t < twns.length; t++) {
            // Get the town prod queues
            var tQs = dt[PlayerId].towns[twns[t]].prod;
            // Get the production bldgs in this town
            var twnProdBldgs = dt.production.getTownProdBldgs(twns[t]);
            dt.production.makeTblRows(twnProdBldgs, tQs, dt[PlayerId].towns[twns[t]], prodTabs);
        }

// TableSort.init('someTable')
        var k = Object.keys(prodTabs);
        for (let i = 0; i < k.length; i++) {
            TableSort.init(prodTabs[k[i]].tblId);
        }

//        var qsTabsDiv = this.queueTabs("#showAllQueuesHtml");
//        if (dt.popupStates.AllQs) {
//            dt.showTool(qsTabsDiv, "AllQs");
//        }
//            dt.showTool("#showAllQueuesHtml", "AllQs");
        dt.showTool("#allQsTable");
    },

    makeTblRows: function (twnProdBldgs, tQs, twn, prodTabs) {
        twnProdBldgs.forEach(function (thisB, bId) {
//            console.log("Checking: ",twn.name,thisB,bId);
            var active = false;
            var k = Object.keys(tQs);
            for (qI = 0; qI < k.length; qI++) {
//                console.log(tQs[qI]);
                if (tQs[qI].bId == bId) {
//                    console.log("Queue for current building...");
                    active = true;
                    var tz = Intl.DateTimeFormat().resolvedOptions().timeZone;
                    var loc = Intl.DateTimeFormat().resolvedOptions().locale;
                    var options = {weekday: 'short', year: 'numeric', month: 'short', day: '2-digit', hour: '2-digit', minute: '2-digit', second: '2-digit'};
                    let d = new Date(tQs[qI].endsAt / 1);
                    let iEnd = d.toLocaleString(undefined, {weekday: 'short', year: 'numeric', month: 'short', day: '2-digit', hour: '2-digit', minute: '2-digit', second: '2-digit', timeZone: 'UTC'});
                    let lEnd = d.toLocaleString(loc, {weekday: 'short', year: 'numeric', month: 'short', day: '2-digit', hour: '2-digit', minute: '2-digit', second: '2-digit', timeZone: tz});
//                    var r = [twn.name, tQs[qI].Building, tQs[qI].item, tQs[qI].endsAt, d.toLocaleString('en-GB', { timeZone: 'UTC' }), d.toLocaleString()];
                    var r = [twn.name, tQs[qI].Building, tQs[qI].item, Number(tQs[qI].endsAt), iEnd, lEnd];
                    dt.production.addRows(r, thisB.class, prodTabs);

                }
            }
            if (active === false) {
                var r = [twn.name, thisB.name, , 0, , ];
                dt.production.addRows(r, thisB.class, prodTabs);
            }
//            console.log(thisB.class, r);
//            dt.production.addRows(r, thisB.class, prodTabs);
        });
    },
    addRows: function (r, tabName, prodTabs) {
        var tBody1 = document.querySelector("#" + prodTabs.all.tblId + " tbody");
        var tBody2 = document.querySelector("#" + prodTabs[tabName].tblId + " tbody");
        var row1 = tBody1.insertRow(-1);
        var row2 = tBody2.insertRow(-1);
        for (var c = 0; c < 6; c++) {
            var cell1 = row1.insertCell(c);
            var cell2 = row2.insertCell(c);
            cell1.textContent = r[c];
            cell2.textContent = r[c];
            if (c === 3) {
                cell1.classList.add("tiny");
                cell2.classList.add("tiny");
            }
        }

        var rubbish = "";
    },
    alertDelHandler: function (index) {
        var arr = dt[PlayerId].prodNotices;
//        var index = event.target.value;
        if (index > -1) {
            arr.splice(index, 1);
        }
        this.showProdAlerts();
        var key = PlayerId + ':prodNotices';
        locStorage.setItem(key, JSON.stringify(dt[PlayerId].prodNotices));
    },

};
dt.prodBuildings = {"8": {"class": "mil", "name": "Barracks"}, "9": {"class": "trade", "name": "Marketplace"}, "10": {"class": "items", "name": "Brewery"}, "12": {"class": "diplo", "name": "Consulate"}, "19": {"class": "items", "name": "Common Ground"}, "21": {"class": "items", "name": "Paddock"}, "28": {"class": "items", "name": "Fletcher"}, "31": {"class": "items", "name": "Spearmaker"}, "32": {"class": "items", "name": "Blacksmith"}, "33": {"class": "items", "name": "Forge"}, "34": {"class": "items", "name": "Tannery"}, "35": {"class": "items", "name": "Saddlemaker"}, "36": {"class": "items", "name": "Siege Workshop"}, "37": {"class": "items", "name": "Bookbinder"}};

dt.prodCheckTimer = window.setInterval(dt.production.fullCumpleteCheck, 60000);
