/*
 * cBuildings - object. Holds details of the buildings needed to make a crafted item. Organised by item id.
 * itemid:{name:name of item, bldgs:[array of bldg ids needed]}
 * 
 * npcFamilies - object. Holds the details of what units belong to a particular group.
 * Family Name:[id of unit,id of unit...] - repeated for each family
 * 
 * equipmentTypes - object. Shows whether a crefted item is a mount, weapon or armour. For each area there is an object holding an array of equipment ids.
 * 
 * cpi - object. {1:"i=3|1",2:"i=3|2",3:"i=3|3",4:"i=3|4",5:"i=3|5",6:"i=3|6",7:"i=3|7",8:"i=3|8",9:"i=3|9",10:"i=3|10",11:"i=3|11",12:"i=3|12",17:"i=1|1",19:"i=1|2",21:"i=1|3",23:"i=1|4",25:"i=1|5",183:"i=4|1",295:"i=2|1",296:"i=2|2"}
 * 
 * icp - object. {"i=1|1":17,"i=1|2":19,"i=1|3":21,"i=1|4":23,"i=1|5":25,"i=2|1":295,"i=2|2":296,"i=3|1":1,"i=3|2":2,"i=3|3":3,"i=3|4":4,"i=3|5":5,"i=3|6":6,"i=3|7":7,"i=3|8":8,"i=3|9":9,"i=3|10":10,"i=3|11":11,"i=3|12":12,"i=4|1":183}
 * 
 * iClasses  - object. Holds details of the classes needed for the correct display of the base & advanced res icons with
 * the code of the item as the key.
 * item code:[class list as string, item title as string]
 * 
 * cEffects - object. Holds the effects that a crafted item has. For each effect there is an array holding the effect 
 * and area of effect.
 * itemid:{"effects":[[numerical effect, Area of effect]]}
 * 
 * cRecipes - object. Holds details on what is needed to craft an item. For each sub-item there is an array holding 
 * the qty needed & the code for the sub-item.
 * item id:{"recipe":[["1","i=3|1"],["5","i=4|1"],["5","c=186"],["10","c=416"],["1","c=417"]]}
 * 
 * cItems - object. Holds details of crafted items. Organised by the id of the item.
 * item id:{"name":name of item as string, "classes":classes for displaying item as string,"e": string holding data on the item with fields separated by |. Not known what the fields are any more."0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|25|0|0|0|0|0|1|0|0|0"}
 * 
 * unitLvls - object. Arrays of player units showing if they are t1 or t2.
 * {t1:[46,48,...],t2:[47,49,...]}
 * 
 * unitTypes - object. Arrays of player units showing if they are spear, infantry, ranged, cav or siege.
 * {spear:[46,47,...],inf:[56,57,...],bow:[66,67,...],cav:[76,77,...],siege:[86,87,...]}
 * 
 * mount - array. Ids of all crafted mounts.
 * 
 * armour - array. Ids of all crafted armours.
 * 
 * weapon - array. Ids of all crafted weapons.
 * 
 * allembineSites - object. Details for each of the allembine sites.
 * {1: {a: {coords: [ - 312, 717]}, name: 'Aldelbrecht'}
 * 
 * cbtTerrainMods object. Data for each of the main combat terrain types on how the unit types are affected.
 * {"Buildings": {atk: {cav: "-50", spear: "20", inf: "40", range: "5"}, def: {cav: "-45", spear: "+25", inf: "30", range: "15"}},
 * 
 * cbtTerrains - object. Details of the effects text for each of the main cbt terrain types.
 * {"Large Hill": {equip:['on Hills']}
 * 
 * biomes - object. Details of the effects text for each of the biome types.
 * {"Desert":{equip:['in Desert']},
 * 
 * tradeHubs - array. Details of each faction trade hubs.
 * [{"name":"Acies","regionId":"51","regionName":"High Hills","factionId":"99","factionName":"The New Light"}
 * 
 * terrainBonuses - object. Details of the bonuses for the different terrains
 *  {"Rocky Outcrop": {bldg: "Farrier", item: "Horses", change: "3"},
 */
dt.terrainBonuses = {"Rocky Outcrop": {bldg: "Farrier", item: "Horses", change: "3"}, "Fertile Ground": {bldg: "Farrier", item: "Horses", change: "1"}, "Bountiful Land": {bldg: "Cattle Rancher", item: "Cattle", change: "3"}, "Alluvial Plain": {bldg: "Cattle Rancher", item: "Cattle", change: "1"}, "Abundant Crops": {bldg: "Brewer's Yard", item: "Beer", change: "3"}, "Treacherous Mountains": {bldg: "Brewer's Yard", item: "Beer", change: "2"}, "Fertile Orchard": {bldg: "Brewer's Yard", item: "Beer", change: "1"}, "Rich Clay Seam": {bldg: "Papermill", item: "Books", change: "3"}, "Abundant Clay": {bldg: "Papermill", item: "Books", change: "2"}, "Wooded Land": {bldg: "Poleturner", item: "Spears", change: "3"}, "Light Woods": {bldg: "Poleturner", item: "Spears", change: "1"}, "Sharp Crags": {bldg: "Bladesmith", item: "Swords", change: "3"}, "Mountains": {bldg: "Bladesmith", item: "Swords", change: "1"}, "Thick Forest": {bldg: "Bowyer", item: "Bows", change: "3"}, "Forested Hilltop": {bldg: "Bowyer", item: "Bows", change: "1"}, "Turned Clay": {bldg: "Bridlemaker", item: "Saddles", change: "2"}, "Heavy Clay Seam": {bldg: "Bridlemaker", item: "Saddles", change: "1"}, "Clay Seam": {bldg: "Renderer", item: "Leather Armour", change: "3"}, "Exposed Clay": {bldg: "Renderer", item: "Leather Armour", change: "1"}, "Craggy Peaks": {bldg: "Armourer", item: "Chainmail", change: "3"}, "Stony Ground": {bldg: "Armourer", item: "Chainmail", change: "1"}, "Abundant Quarry": {bldg: "Plate Forger", item: "Plate Armour", change: "3"}, "Lonely Peaks": {bldg: "Plate Forger", item: "Plate Armour", change: "1"}, "Bleak Mountains": {bldg: "Finishing School", item: "Diplomats", change: "2"}, "Scrubland": {bldg: "Finishing School", item: "Diplomats", change: "1"}, "Wooded Glade": {bldg: "Training Ground", item: "Spearmen", change: "2"}, "Tundra": {bldg: "Training Ground", item: "Spearmen", change: "1"}, "Dense Forest": {bldg: "Target Range", item: "Archers", change: "2"}, "Clearing": {bldg: "Target Range", item: "Archers", change: "1"}, "Moor": {bldg: "Military Academy", item: "Swordsmen", change: "1"}, "Rich Quarry": {bldg: "Military Academy", item: "Swordsmen", change: "2"}, "Fertile Pasture": {bldg: "Jousting Yard", item: "Cavalry", change: "2"}, "Open Plains": {bldg: "Jousting Yard", item: "Cavalry", change: "1"}, "Landslip": {bldg: "Assembly Yard", item: "Seige Engines", change: "2"}, "Wooded Quarry": {bldg: "Assembly Yard", item: "Seige Engines", change: "1"}};
dt.tradeHubs = [{"name":"Acies","regionId":"51","regionName":"High Hills","factionId":"99","factionName":"The New Light"},{"name":"Aelif","regionId":"43","regionName":"Pamanyallpa","factionId":"57","factionName":"Windseekers"},{"name":"Aeris","regionId":"21","regionName":"Perrigor","factionId":"9","factionName":"Circle Of Five"},{"name":"Akualis","regionId":"33","regionName":"Larn","factionId":"34","factionName":"Demersae"},{"name":"Allon","regionId":"18","regionName":"Tor Carrock","factionId":"74","factionName":"Lannigolds"},{"name":"Alltudin","regionId":"46","regionName":"Pawanallpa","factionId":"92","factionName":"Llwcharion"},{"name":"Apaxy","regionId":"27","regionName":"Zanpur","factionId":"73","factionName":"Apaxu"},{"name":"Aqua","regionId":"15","regionName":"Middle Kingdom","factionId":"9","factionName":"Circle Of Five"},{"name":"Argmaenor","regionId":"31","regionName":"OCEAN","factionId":"100","factionName":"The Pirate Kings"},{"name":"Artain","regionId":"46","regionName":"Pawanallpa","factionId":"92","factionName":"Llwcharion"},{"name":"Augo","regionId":"45","regionName":"Clarien","factionId":"99","factionName":"The New Light"},{"name":"Azuria","regionId":"29","regionName":"Azura","factionId":"5","factionName":"Azure Throne"},{"name":"Bandarius","regionId":"16","regionName":"Mal Motsha","factionId":"72","factionName":"Pax Orcana"},{"name":"Banderstol","regionId":"68","regionName":"Kormandly","factionId":"103","factionName":"Virten"},{"name":"Baolan","regionId":"10","regionName":"Laoshin","factionId":"18","factionName":"Forbidden Empire"},{"name":"Belgorrian","regionId":"31","regionName":"OCEAN","factionId":"100","factionName":"The Pirate Kings"},{"name":"Bellingsy","regionId":"15","regionName":"Middle Kingdom","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Benfis","regionId":"13","regionName":"Meilla","factionId":"42","factionName":"Order of Allembine"},{"name":"Bepima","regionId":"29","regionName":"Azura","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Besnurk Barracks","regionId":"73","regionName":"Strendur","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Blackbriar","regionId":"30","regionName":"Djebeli","factionId":"59","factionName":"Unseelie Court"},{"name":"Boyaquira","regionId":"41","regionName":"Puchuallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Broadrick's Landing","regionId":"43","regionName":"Pamanyallpa","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Busillis","regionId":"14","regionName":"Lucerna","factionId":"16","factionName":"Empirium"},{"name":"Caer Morock","regionId":"9","regionName":"Norweld","factionId":"69","factionName":"Clan Reashag"},{"name":"Caerlea","regionId":"18","regionName":"Tor Carrock","factionId":"75","factionName":"Clan Bealagh"},{"name":"Cask","regionId":"8","regionName":"Fremorn","factionId":"39","factionName":"Outlaws"},{"name":"Castle of Mercy","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Catmont","regionId":"73","regionName":"Strendur","factionId":"103","factionName":"Virten"},{"name":"Centrum","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Cittacolumbae","regionId":"15","regionName":"Middle Kingdom","factionId":"16","factionName":"Empirium"},{"name":"City of Temples","regionId":"32","regionName":"Tallimar","factionId":"62","factionName":"Brotherhood of Kerala"},{"name":"Cleag's Haven","regionId":"59","regionName":"Newlands","factionId":"94","factionName":"Overoad Traders"},{"name":"Clegerick","regionId":"18","regionName":"Tor Carrock","factionId":"76","factionName":"Clan Moedagh"},{"name":"Cloghord's Haven","regionId":"57","regionName":"Coanhara","factionId":"94","factionName":"Overoad Traders"},{"name":"College of Silence","regionId":"52","regionName":"Westmarch","factionId":"97","factionName":"The College of Silence"},{"name":"Crowcaer","regionId":"8","regionName":"Fremorn","factionId":"61","factionName":"Treggar's Crows"},{"name":"Damquka","regionId":"12","regionName":"Taomist","factionId":"27","factionName":"Kanchalka"},{"name":"Deagh's Haven","regionId":"67","regionName":"Almenly","factionId":"94","factionName":"Overoad Traders"},{"name":"Detyrk Barracks","regionId":"73","regionName":"Strendur","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Dolentis","regionId":"14","regionName":"Lucerna","factionId":"7","factionName":"Fratri Hominis"},{"name":"Doloria","regionId":"14","regionName":"Lucerna","factionId":"7","factionName":"Fratri Hominis"},{"name":"Dolunhill","regionId":"63","regionName":"Silbeaur","factionId":"101","factionName":"The Tears Eternal"},{"name":"Drangu Kala","regionId":"2","regionName":"Kal Tirikan","factionId":"38","factionName":"Drangu Mundas"},{"name":"Dreadland Keep","regionId":"67","regionName":"Almenly","factionId":"89","factionName":"Houergerrt"},{"name":"Dunbar","regionId":"9","regionName":"Norweld","factionId":"68","factionName":"Guardians of Zera"},{"name":"Dunthaslea","regionId":"19","regionName":"The Western Realms","factionId":"32","factionName":"Lords Of The West"},{"name":"Durus","regionId":"45","regionName":"Clarien","factionId":"99","factionName":"The New Light"},{"name":"Dyadin","regionId":"25","regionName":"Arran","factionId":"82","factionName":"Dyadans"},{"name":"Eartholme","regionId":"24","regionName":"Lan Larosh","factionId":"12","factionName":"Kildarran"},{"name":"Ellesmere","regionId":"14","regionName":"Lucerna","factionId":"74","factionName":"Lannigolds"},{"name":"Elscrick","regionId":"68","regionName":"Kormandly","factionId":"103","factionName":"Virten"},{"name":"Eshtuk","regionId":"75","regionName":"Jurgor","factionId":"87","factionName":"Guul-Hai"},{"name":"Etilan","regionId":"19","regionName":"The Western Realms","factionId":"32","factionName":"Lords Of The West"},{"name":"Eyepool","regionId":"25","regionName":"Arran","factionId":"74","factionName":"Lannigolds"},{"name":"Fah Strigz Doraz","regionId":"16","regionName":"Mal Motsha","factionId":"71","factionName":"Crimson Skulls"},{"name":"Falkenburg","regionId":"3","regionName":"Wolgast","factionId":"36","factionName":"Northmen"},{"name":"Famak Barracks","regionId":"68","regionName":"Kormandly","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Freeport","regionId":"36","regionName":"Trome","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Fytorj Barracks","regionId":"73","regionName":"Strendur","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Gajik Festral","regionId":"5","regionName":"Qarosslan","factionId":"70","factionName":"Blood Reavers"},{"name":"Gajik Hinor","regionId":"4","regionName":"Ursor","factionId":"70","factionName":"Blood Reavers"},{"name":"Gajik Serun","regionId":"4","regionName":"Ursor","factionId":"70","factionName":"Blood Reavers"},{"name":"Galarwood","regionId":"67","regionName":"Almenly","factionId":"101","factionName":"The Tears Eternal"},{"name":"Garr","regionId":"2","regionName":"Kal Tirikan","factionId":"6","factionName":"Barbarians"},{"name":"Gilakek","regionId":"23","regionName":"Kumala","factionId":"4","factionName":"Atagek"},{"name":"Gjuk-Darak","regionId":"75","regionName":"Jurgor","factionId":"86","factionName":"Free Orcs"},{"name":"Gjuntak Barracks","regionId":"57","regionName":"Coanhara","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Glinntre","regionId":"18","regionName":"Tor Carrock","factionId":"75","factionName":"Clan Bealagh"},{"name":"Glory City","regionId":"27","regionName":"Zanpur","factionId":"41","factionName":"Raja Metu"},{"name":"Gordarill","regionId":"31","regionName":"OCEAN","factionId":"100","factionName":"The Pirate Kings"},{"name":"Gorn","regionId":"2","regionName":"Kal Tirikan","factionId":"6","factionName":"Barbarians"},{"name":"Gorwyn","regionId":"46","regionName":"Pawanallpa","factionId":"92","factionName":"Llwcharion"},{"name":"Great Mountain Home","regionId":"18","regionName":"Tor Carrock","factionId":"30","factionName":"Parvacones"},{"name":"Greybriar","regionId":"61","regionName":"Aindara","factionId":"103","factionName":"Virten"},{"name":"Grisj-hhuragk","regionId":"72","regionName":"Shardlands","factionId":"86","factionName":"Free Orcs"},{"name":"Grovinton","regionId":"13","regionName":"Meilla","factionId":"23","factionName":"Keepers of Golden Fields"},{"name":"Grujt Uruk","regionId":"76","regionName":"The Long White","factionId":"86","factionName":"Free Orcs"},{"name":"Hall of Contemplation","regionId":"67","regionName":"Almenly","factionId":"101","factionName":"The Tears Eternal"},{"name":"Hall of Reflection","regionId":"57","regionName":"Coanhara","factionId":"97","factionName":"The College of Silence"},{"name":"Halls of Care","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Hanlif","regionId":"37","regionName":"Rill Archipelago","factionId":"57","factionName":"Windseekers"},{"name":"Hastelbury","regionId":"13","regionName":"Meilla","factionId":"1","factionName":"Council of Illyria"},{"name":"Headagh's Haven","regionId":"69","regionName":"The Orken Coast","factionId":"94","factionName":"Overoad Traders"},{"name":"Hedgor's Haven","regionId":"64","regionName":"Fellandire","factionId":"94","factionName":"Overoad Traders"},{"name":"Helnsbere","regionId":"61","regionName":"Aindara","factionId":"103","factionName":"Virten"},{"name":"Hoataoto","regionId":"31","regionName":"OCEAN","factionId":"90","factionName":"Kapikami"},{"name":"Hohnsval","regionId":"2","regionName":"Kal Tirikan","factionId":"60","factionName":"Crimson Dawn"},{"name":"Holbaek","regionId":"2","regionName":"Kal Tirikan","factionId":"36","factionName":"Northmen"},{"name":"Hope","regionId":"8","regionName":"Fremorn","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Hope Ridge","regionId":"54","regionName":"Oarnamly","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Hoscarmel","regionId":"23","regionName":"Kumala","factionId":"74","factionName":"Lannigolds"},{"name":"Humayan","regionId":"27","regionName":"Zanpur","factionId":"41","factionName":"Raja Metu"},{"name":"Ignis","regionId":"21","regionName":"Perrigor","factionId":"9","factionName":"Circle Of Five"},{"name":"Ijek","regionId":"71","regionName":"Farshards","factionId":"87","factionName":"Guul-Hai"},{"name":"Ildibrook","regionId":"54","regionName":"Oarnamly","factionId":"101","factionName":"The Tears Eternal"},{"name":"Jallalabad","regionId":"27","regionName":"Zanpur","factionId":"53","factionName":"Udaiman"},{"name":"Jarru","regionId":"23","regionName":"Kumala","factionId":"35","factionName":"Jannu"},{"name":"Jhkej","regionId":"76","regionName":"The Long White","factionId":"86","factionName":"Free Orcs"},{"name":"Kachak","regionId":"72","regionName":"Shardlands","factionId":"86","factionName":"Free Orcs"},{"name":"Kajadum","regionId":"27","regionName":"Zanpur","factionId":"77","factionName":"Hashashin"},{"name":"Kala Mahlarg","regionId":"7","regionName":"Tamarin","factionId":"64","factionName":"Marauding Skullsplitters"},{"name":"Karakar","regionId":"29","regionName":"Azura","factionId":"21","factionName":"Skarakar"},{"name":"Kelsmouth","regionId":"32","regionName":"Tallimar","factionId":"28","factionName":"Kingdom of Tal"},{"name":"Kendbrook","regionId":"31","regionName":"OCEAN","factionId":"103","factionName":"Virten"},{"name":"Kepsburg","regionId":"17","regionName":"Keppen","factionId":"13","factionName":"Duchy of Keppen"},{"name":"Khafkar","regionId":"27","regionName":"Zanpur","factionId":"45","factionName":"Sultanate of Kazim"},{"name":"Kingsthrone","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Kolbarch","regionId":"51","regionName":"High Hills","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Kreyo","regionId":"56","regionName":"Gremont","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Kzrik","regionId":"9","regionName":"Norweld","factionId":"2","factionName":"Arakvar"},{"name":"Lacona","regionId":"25","regionName":"Arran","factionId":"51","factionName":"Triumvirate"},{"name":"Laghadd's Haven","regionId":"66","regionName":"Vindorel","factionId":"94","factionName":"Overoad Traders"},{"name":"Lanarrin","regionId":"24","regionName":"Lan Larosh","factionId":"81","factionName":"Lyrians"},{"name":"Landingstown","regionId":"31","regionName":"OCEAN","factionId":"95","factionName":"South Oceans Company"},{"name":"Lantellyn","regionId":"24","regionName":"Lan Larosh","factionId":"81","factionName":"Lyrians"},{"name":"Laresh","regionId":"21","regionName":"Perrigor","factionId":"74","factionName":"Lannigolds"},{"name":"Lasthold","regionId":"57","regionName":"Coanhara","factionId":"84","factionName":"Clan Dollogh"},{"name":"Lostwithiel","regionId":"3","regionName":"Wolgast","factionId":"19","factionName":"Fyrgis"},{"name":"Ludisfold","regionId":"31","regionName":"OCEAN","factionId":"103","factionName":"Virten"},{"name":"Madh Kala Uruk","regionId":"16","regionName":"Mal Motsha","factionId":"71","factionName":"Crimson Skulls"},{"name":"Madh-Vraken-Kalj","regionId":"69","regionName":"The Orken Coast","factionId":"85","factionName":"Drek-Hhakral"},{"name":"Mahareva","regionId":"58","regionName":"Lapo'a Lua","factionId":"90","factionName":"Kapikami"},{"name":"Mandaipur","regionId":"27","regionName":"Zanpur","factionId":"41","factionName":"Raja Metu"},{"name":"Maplehurst","regionId":"13","regionName":"Meilla","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Maraupoe","regionId":"31","regionName":"OCEAN","factionId":"90","factionName":"Kapikami"},{"name":"Marston","regionId":"9","regionName":"Norweld","factionId":"68","factionName":"Guardians of Zera"},{"name":"Mataba","regionId":"30","regionName":"Djebeli","factionId":"45","factionName":"Sultanate of Kazim"},{"name":"Matombo","regionId":"29","regionName":"Azura","factionId":"26","factionName":"Kadu"},{"name":"Memor","regionId":"51","regionName":"High Hills","factionId":"99","factionName":"The New Light"},{"name":"Mentis","regionId":"59","regionName":"Newlands","factionId":"99","factionName":"The New Light"},{"name":"Metaki","regionId":"29","regionName":"Azura","factionId":"26","factionName":"Kadu"},{"name":"Mons Equi","regionId":"15","regionName":"Middle Kingdom","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Moonvale","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Mourlake","regionId":"74","regionName":"Chulbran","factionId":"103","factionName":"Virten"},{"name":"Nachik","regionId":"27","regionName":"Zanpur","factionId":"73","factionName":"Apaxu"},{"name":"Nderurk Barracks","regionId":"74","regionName":"Chulbran","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Nemidas","regionId":"32","regionName":"Tallimar","factionId":"28","factionName":"Kingdom of Tal"},{"name":"Nessim","regionId":"22","regionName":"Kul Tar","factionId":"45","factionName":"Sultanate of Kazim"},{"name":"New Hope","regionId":"31","regionName":"OCEAN","factionId":"83","factionName":"Argiri"},{"name":"New Tal","regionId":"43","regionName":"Pamanyallpa","factionId":"96","factionName":"Tal Colonists"},{"name":"Newchapel","regionId":"68","regionName":"Kormandly","factionId":"103","factionName":"Virten"},{"name":"Newharbour","regionId":"33","regionName":"Larn","factionId":"17","factionName":"Kingdom of Larn"},{"name":"Newyns","regionId":"46","regionName":"Pawanallpa","factionId":"92","factionName":"Llwcharion"},{"name":"Norvasser","regionId":"61","regionName":"Aindara","factionId":"103","factionName":"Virten"},{"name":"Olog Ha","regionId":"32","regionName":"Tallimar","factionId":"52","factionName":"Zau Brulk"},{"name":"Olog Rraus","regionId":"32","regionName":"Tallimar","factionId":"52","factionName":"Zau Brulk"},{"name":"Omen","regionId":"40","regionName":"Calumnex","factionId":"102","factionName":"The Tower"},{"name":"Oramatete","regionId":"58","regionName":"Lapo'a Lua","factionId":"90","factionName":"Kapikami"},{"name":"Ordu Kush","regionId":"20","regionName":"Keshalia","factionId":"50","factionName":"Tipu Khan"},{"name":"Orelys","regionId":"13","regionName":"Meilla","factionId":"42","factionName":"Order of Allembine"},{"name":"Parataoti","regionId":"31","regionName":"OCEAN","factionId":"90","factionName":"Kapikami"},{"name":"Parlantis","regionId":"32","regionName":"Tallimar","factionId":"40","factionName":"Parlan"},{"name":"Patradan","regionId":"31","regionName":"OCEAN","factionId":"100","factionName":"The Pirate Kings"},{"name":"Pellimont","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Pencaster","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Pentateul","regionId":"33","regionName":"Larn","factionId":"17","factionName":"Kingdom of Larn"},{"name":"Persiflage","regionId":"9","regionName":"Norweld","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Poeghad's Haven","regionId":"51","regionName":"High Hills","factionId":"94","factionName":"Overoad Traders"},{"name":"Port Tal","regionId":"32","regionName":"Tallimar","factionId":"28","factionName":"Kingdom of Tal"},{"name":"Potentia","regionId":"14","regionName":"Lucerna","factionId":"16","factionName":"Empirium"},{"name":"Praekorn","regionId":"64","regionName":"Fellandire","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Princevale","regionId":"74","regionName":"Chulbran","factionId":"103","factionName":"Virten"},{"name":"Qainaqangma","regionId":"12","regionName":"Taomist","factionId":"27","factionName":"Kanchalka"},{"name":"Qal Mar","regionId":"22","regionName":"Kul Tar","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Qi Quien","regionId":"10","regionName":"Laoshin","factionId":"49","factionName":"Tien Zao"},{"name":"Qingdao","regionId":"10","regionName":"Laoshin","factionId":"56","factionName":"Wen Kun Dynasty"},{"name":"Rainswinton","regionId":"31","regionName":"OCEAN","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Rendsburg","regionId":"3","regionName":"Wolgast","factionId":"36","factionName":"Northmen"},{"name":"Riverpass","regionId":"17","regionName":"Keppen","factionId":"13","factionName":"Duchy of Keppen"},{"name":"Roetagh's Haven","regionId":"66","regionName":"Vindorel","factionId":"94","factionName":"Overoad Traders"},{"name":"Ronneby","regionId":"9","regionName":"Norweld","factionId":"37","factionName":"Nordlanders"},{"name":"Rosdin","regionId":"18","regionName":"Tor Carrock","factionId":"76","factionName":"Clan Moedagh"},{"name":"Royalport","regionId":"31","regionName":"OCEAN","factionId":"95","factionName":"South Oceans Company"},{"name":"Sanctuary","regionId":"72","regionName":"Shardlands","factionId":"93","factionName":"Mellorians"},{"name":"Sansouran","regionId":"24","regionName":"Lan Larosh","factionId":"80","factionName":"Illians"},{"name":"Santellya","regionId":"24","regionName":"Lan Larosh","factionId":"80","factionName":"Illians"},{"name":"Scara Gajup","regionId":"2","regionName":"Kal Tirikan","factionId":"78","factionName":"Foshnu Ob Wulpor"},{"name":"Scara Ulurl","regionId":"2","regionName":"Kal Tirikan","factionId":"78","factionName":"Foshnu Ob Wulpor"},{"name":"Scaravar","regionId":"29","regionName":"Azura","factionId":"21","factionName":"Skarakar"},{"name":"Scio","regionId":"59","regionName":"Newlands","factionId":"99","factionName":"The New Light"},{"name":"Scite","regionId":"15","regionName":"Middle Kingdom","factionId":"16","factionName":"Empirium"},{"name":"Sebring","regionId":"19","regionName":"The Western Realms","factionId":"32","factionName":"Lords Of The West"},{"name":"Selenze","regionId":"25","regionName":"Arran","factionId":"51","factionName":"Triumvirate"},{"name":"Seven Blessings","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Shearwall Market","regionId":"45","regionName":"Clarien","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Shelton","regionId":"14","regionName":"Lucerna","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Shingeng","regionId":"16","regionName":"Mal Motsha","factionId":"18","factionName":"Forbidden Empire"},{"name":"Shqur-Mich-Kalj","regionId":"69","regionName":"The Orken Coast","factionId":"85","factionName":"Drek-Hhakral"},{"name":"Shurthorn","regionId":"17","regionName":"Keppen","factionId":"13","factionName":"Duchy of Keppen"},{"name":"Sildarim","regionId":"5","regionName":"Qarosslan","factionId":"43","factionName":"Sillar"},{"name":"Sillarus","regionId":"5","regionName":"Qarosslan","factionId":"43","factionName":"Sillar"},{"name":"Silver Spires","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Sluice Haven","regionId":"8","regionName":"Fremorn","factionId":"39","factionName":"Outlaws"},{"name":"Slupsk","regionId":"9","regionName":"Norweld","factionId":"37","factionName":"Nordlanders"},{"name":"Snaresheath","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Southforge","regionId":"61","regionName":"Aindara","factionId":"103","factionName":"Virten"},{"name":"Ssaharin","regionId":"22","regionName":"Kul Tar","factionId":"31","factionName":"Sslipentin"},{"name":"Stoneberg","regionId":"73","regionName":"Strendur","factionId":"103","factionName":"Virten"},{"name":"Stormstone","regionId":"38","regionName":"Stormstone Island","factionId":"47","factionName":"Tenaril"},{"name":"Styne","regionId":"3","regionName":"Wolgast","factionId":"19","factionName":"Fyrgis"},{"name":"Suiqias","regionId":"41","regionName":"Puchuallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Swiftstead","regionId":"6","regionName":"Windlost","factionId":"48","factionName":"Tewhirrus"},{"name":"Syguf Barracks","regionId":"70","regionName":"Kingslands","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Tantark","regionId":"2","regionName":"Kal Tirikan","factionId":"8","factionName":"Tantarkim"},{"name":"Tarponsea","regionId":"29","regionName":"Azura","factionId":"5","factionName":"Azure Throne"},{"name":"Tattershill","regionId":"50","regionName":"Northmarch","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Tayaribe","regionId":"43","regionName":"Pamanyallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Temple of Compassion","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Tempus","regionId":"21","regionName":"Perrigor","factionId":"9","factionName":"Circle Of Five"},{"name":"Terra","regionId":"18","regionName":"Tor Carrock","factionId":"9","factionName":"Circle Of Five"},{"name":"Tewhirria","regionId":"6","regionName":"Windlost","factionId":"48","factionName":"Tewhirrus"},{"name":"The Barracks","regionId":"6","regionName":"Windlost","factionId":"63","factionName":"Bayne's Irregulars"},{"name":"The Barrier","regionId":"9","regionName":"Norweld","factionId":"58","factionName":"Seelie Court"},{"name":"The Brothers' Haven","regionId":"51","regionName":"High Hills","factionId":"94","factionName":"Overoad Traders"},{"name":"The Empty Halls","regionId":"71","regionName":"Farshards","factionId":"98","factionName":"The Lost Clans"},{"name":"The Loft","regionId":"8","regionName":"Fremorn","factionId":"10","factionName":"Daedalons"},{"name":"Thop Fatoft","regionId":"12","regionName":"Taomist","factionId":"25","factionName":"Sma Uruk"},{"name":"Thoroe's Haven","regionId":"57","regionName":"Coanhara","factionId":"94","factionName":"Overoad Traders"},{"name":"Thujn-Huckut-Kalj","regionId":"66","regionName":"Vindorel","factionId":"85","factionName":"Drek-Hhakral"},{"name":"Tiena","regionId":"25","regionName":"Arran","factionId":"51","factionName":"Triumvirate"},{"name":"Tilverdale","regionId":"17","regionName":"Keppen","factionId":"22","factionName":"Tinkers"},{"name":"Tor Dannu","regionId":"18","regionName":"Tor Carrock","factionId":"76","factionName":"Clan Moedagh"},{"name":"Tor Gorrak","regionId":"18","regionName":"Tor Carrock","factionId":"75","factionName":"Clan Bealagh"},{"name":"Torkurz","regionId":"75","regionName":"Jurgor","factionId":"86","factionName":"Free Orcs"},{"name":"Tractus","regionId":"59","regionName":"Newlands","factionId":"99","factionName":"The New Light"},{"name":"Trazurk","regionId":"76","regionName":"The Long White","factionId":"86","factionName":"Free Orcs"},{"name":"Trimek Barracks","regionId":"74","regionName":"Chulbran","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Tringar Trading Post","regionId":"1","regionName":"The Wastes","factionId":"66","factionName":"Trappers"},{"name":"Trottingham","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Tubriq","regionId":"26","regionName":"Turalia","factionId":"54","factionName":"Udan Tebriz"},{"name":"Tudeyll","regionId":"24","regionName":"Lan Larosh","factionId":"79","factionName":"Turyns"},{"name":"Tundale","regionId":"11","regionName":"Ragallon","factionId":"1","factionName":"Council of Illyria"},{"name":"Turyin","regionId":"24","regionName":"Lan Larosh","factionId":"79","factionName":"Turyns"},{"name":"Udalia","regionId":"26","regionName":"Turalia","factionId":"54","factionName":"Udan Tebriz"},{"name":"Uruk Vadokan","regionId":"13","regionName":"Meilla","factionId":"67","factionName":"Sundog Gith"},{"name":"Urukonium","regionId":"16","regionName":"Mal Motsha","factionId":"72","factionName":"Pax Orcana"},{"name":"Va Drangu","regionId":"3","regionName":"Wolgast","factionId":"38","factionName":"Drangu Mundas"},{"name":"Vdekurch","regionId":"76","regionName":"The Long White","factionId":"86","factionName":"Free Orcs"},{"name":"Vduch","regionId":"71","regionName":"Farshards","factionId":"86","factionName":"Free Orcs"},{"name":"Veem","regionId":"31","regionName":"OCEAN","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Veritas","regionId":"14","regionName":"Lucerna","factionId":"7","factionName":"Fratri Hominis"},{"name":"Verity City","regionId":"8","regionName":"Fremorn","factionId":"55","factionName":"Undying Flame"},{"name":"Votar","regionId":"13","regionName":"Meilla","factionId":"67","factionName":"Sundog Gith"},{"name":"Vrakun","regionId":"75","regionName":"Jurgor","factionId":"86","factionName":"Free Orcs"},{"name":"Windmire","regionId":"73","regionName":"Strendur","factionId":"103","factionName":"Virten"},{"name":"Winter Haven","regionId":"73","regionName":"Strendur","factionId":"94","factionName":"Overoad Traders"},{"name":"Woodsedge","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Wylvale","regionId":"67","regionName":"Almenly","factionId":"97","factionName":"The College of Silence"},{"name":"Xcrix","regionId":"9","regionName":"Norweld","factionId":"2","factionName":"Arakvar"},{"name":"Zaqcha","regionId":"41","regionName":"Puchuallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Zgudun Barracks","regionId":"73","regionName":"Strendur","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Zhuhai","regionId":"10","regionName":"Laoshin","factionId":"56","factionName":"Wen Kun Dynasty"},{"name":"Kolbarch","regionId":"51","regionName":"High Hills","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Kreyo","regionId":"56","regionName":"Gremont","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Kzrik","regionId":"9","regionName":"Norweld","factionId":"2","factionName":"Arakvar"},{"name":"Lacona","regionId":"25","regionName":"Arran","factionId":"51","factionName":"Triumvirate"},{"name":"Laghadd's Haven","regionId":"66","regionName":"Vindorel","factionId":"94","factionName":"Overoad Traders"},{"name":"Lanarrin","regionId":"24","regionName":"Lan Larosh","factionId":"81","factionName":"Lyrians"},{"name":"Landingstown","regionId":"31","regionName":"OCEAN","factionId":"95","factionName":"South Oceans Company"},{"name":"Lantellyn","regionId":"24","regionName":"Lan Larosh","factionId":"81","factionName":"Lyrians"},{"name":"Laresh","regionId":"21","regionName":"Perrigor","factionId":"74","factionName":"Lannigolds"},{"name":"Lasthold","regionId":"57","regionName":"Coanhara","factionId":"84","factionName":"Clan Dollogh"},{"name":"Lostwithiel","regionId":"3","regionName":"Wolgast","factionId":"19","factionName":"Fyrgis"},{"name":"Ludisfold","regionId":"31","regionName":"OCEAN","factionId":"103","factionName":"Virten"},{"name":"Madh Kala Uruk","regionId":"16","regionName":"Mal Motsha","factionId":"71","factionName":"Crimson Skulls"},{"name":"Madh-Vraken-Kalj","regionId":"69","regionName":"The Orken Coast","factionId":"85","factionName":"Drek-Hhakral"},{"name":"Mahareva","regionId":"58","regionName":"Lapo'a Lua","factionId":"90","factionName":"Kapikami"},{"name":"Mandaipur","regionId":"27","regionName":"Zanpur","factionId":"41","factionName":"Raja Metu"},{"name":"Maplehurst","regionId":"13","regionName":"Meilla","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Maraupoe","regionId":"31","regionName":"OCEAN","factionId":"90","factionName":"Kapikami"},{"name":"Marston","regionId":"9","regionName":"Norweld","factionId":"68","factionName":"Guardians of Zera"},{"name":"Mataba","regionId":"30","regionName":"Djebeli","factionId":"45","factionName":"Sultanate of Kazim"},{"name":"Matombo","regionId":"29","regionName":"Azura","factionId":"26","factionName":"Kadu"},{"name":"Memor","regionId":"51","regionName":"High Hills","factionId":"99","factionName":"The New Light"},{"name":"Mentis","regionId":"59","regionName":"Newlands","factionId":"99","factionName":"The New Light"},{"name":"Metaki","regionId":"29","regionName":"Azura","factionId":"26","factionName":"Kadu"},{"name":"Mons Equi","regionId":"15","regionName":"Middle Kingdom","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Moonvale","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Mourlake","regionId":"74","regionName":"Chulbran","factionId":"103","factionName":"Virten"},{"name":"Nachik","regionId":"27","regionName":"Zanpur","factionId":"73","factionName":"Apaxu"},{"name":"Nderurk Barracks","regionId":"74","regionName":"Chulbran","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Nemidas","regionId":"32","regionName":"Tallimar","factionId":"28","factionName":"Kingdom of Tal"},{"name":"Nessim","regionId":"22","regionName":"Kul Tar","factionId":"45","factionName":"Sultanate of Kazim"},{"name":"New Hope","regionId":"31","regionName":"OCEAN","factionId":"83","factionName":"Argiri"},{"name":"New Tal","regionId":"43","regionName":"Pamanyallpa","factionId":"96","factionName":"Tal Colonists"},{"name":"Newchapel","regionId":"68","regionName":"Kormandly","factionId":"103","factionName":"Virten"},{"name":"Newharbour","regionId":"33","regionName":"Larn","factionId":"17","factionName":"Kingdom of Larn"},{"name":"Newyns","regionId":"46","regionName":"Pawanallpa","factionId":"92","factionName":"Llwcharion"},{"name":"Norvasser","regionId":"61","regionName":"Aindara","factionId":"103","factionName":"Virten"},{"name":"Olog Ha","regionId":"32","regionName":"Tallimar","factionId":"52","factionName":"Zau Brulk"},{"name":"Olog Rraus","regionId":"32","regionName":"Tallimar","factionId":"52","factionName":"Zau Brulk"},{"name":"Omen","regionId":"40","regionName":"Calumnex","factionId":"102","factionName":"The Tower"},{"name":"Oramatete","regionId":"58","regionName":"Lapo'a Lua","factionId":"90","factionName":"Kapikami"},{"name":"Ordu Kush","regionId":"20","regionName":"Keshalia","factionId":"50","factionName":"Tipu Khan"},{"name":"Orelys","regionId":"13","regionName":"Meilla","factionId":"42","factionName":"Order of Allembine"},{"name":"Parataoti","regionId":"31","regionName":"OCEAN","factionId":"90","factionName":"Kapikami"},{"name":"Parlantis","regionId":"32","regionName":"Tallimar","factionId":"40","factionName":"Parlan"},{"name":"Patradan","regionId":"31","regionName":"OCEAN","factionId":"100","factionName":"The Pirate Kings"},{"name":"Pellimont","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Pencaster","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Pentateul","regionId":"33","regionName":"Larn","factionId":"17","factionName":"Kingdom of Larn"},{"name":"Persiflage","regionId":"9","regionName":"Norweld","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Poeghad's Haven","regionId":"51","regionName":"High Hills","factionId":"94","factionName":"Overoad Traders"},{"name":"Port Tal","regionId":"32","regionName":"Tallimar","factionId":"28","factionName":"Kingdom of Tal"},{"name":"Potentia","regionId":"14","regionName":"Lucerna","factionId":"16","factionName":"Empirium"},{"name":"Praekorn","regionId":"64","regionName":"Fellandire","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Princevale","regionId":"74","regionName":"Chulbran","factionId":"103","factionName":"Virten"},{"name":"Qainaqangma","regionId":"12","regionName":"Taomist","factionId":"27","factionName":"Kanchalka"},{"name":"Qal Mar","regionId":"22","regionName":"Kul Tar","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Qi Quien","regionId":"10","regionName":"Laoshin","factionId":"49","factionName":"Tien Zao"},{"name":"Qingdao","regionId":"10","regionName":"Laoshin","factionId":"56","factionName":"Wen Kun Dynasty"},{"name":"Rainswinton","regionId":"31","regionName":"OCEAN","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Rendsburg","regionId":"3","regionName":"Wolgast","factionId":"36","factionName":"Northmen"},{"name":"Riverpass","regionId":"17","regionName":"Keppen","factionId":"13","factionName":"Duchy of Keppen"},{"name":"Roetagh's Haven","regionId":"66","regionName":"Vindorel","factionId":"94","factionName":"Overoad Traders"},{"name":"Ronneby","regionId":"9","regionName":"Norweld","factionId":"37","factionName":"Nordlanders"},{"name":"Rosdin","regionId":"18","regionName":"Tor Carrock","factionId":"76","factionName":"Clan Moedagh"},{"name":"Royalport","regionId":"31","regionName":"OCEAN","factionId":"95","factionName":"South Oceans Company"},{"name":"Sanctuary","regionId":"72","regionName":"Shardlands","factionId":"93","factionName":"Mellorians"},{"name":"Sansouran","regionId":"24","regionName":"Lan Larosh","factionId":"80","factionName":"Illians"},{"name":"Santellya","regionId":"24","regionName":"Lan Larosh","factionId":"80","factionName":"Illians"},{"name":"Scara Gajup","regionId":"2","regionName":"Kal Tirikan","factionId":"78","factionName":"Foshnu Ob Wulpor"},{"name":"Scara Ulurl","regionId":"2","regionName":"Kal Tirikan","factionId":"78","factionName":"Foshnu Ob Wulpor"},{"name":"Scaravar","regionId":"29","regionName":"Azura","factionId":"21","factionName":"Skarakar"},{"name":"Scio","regionId":"59","regionName":"Newlands","factionId":"99","factionName":"The New Light"},{"name":"Scite","regionId":"15","regionName":"Middle Kingdom","factionId":"16","factionName":"Empirium"},{"name":"Sebring","regionId":"19","regionName":"The Western Realms","factionId":"32","factionName":"Lords Of The West"},{"name":"Selenze","regionId":"25","regionName":"Arran","factionId":"51","factionName":"Triumvirate"},{"name":"Seven Blessings","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Shearwall Market","regionId":"45","regionName":"Clarien","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Shelton","regionId":"14","regionName":"Lucerna","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Shingeng","regionId":"16","regionName":"Mal Motsha","factionId":"18","factionName":"Forbidden Empire"},{"name":"Shqur-Mich-Kalj","regionId":"69","regionName":"The Orken Coast","factionId":"85","factionName":"Drek-Hhakral"},{"name":"Shurthorn","regionId":"17","regionName":"Keppen","factionId":"13","factionName":"Duchy of Keppen"},{"name":"Sildarim","regionId":"5","regionName":"Qarosslan","factionId":"43","factionName":"Sillar"},{"name":"Sillarus","regionId":"5","regionName":"Qarosslan","factionId":"43","factionName":"Sillar"},{"name":"Silver Spires","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Sluice Haven","regionId":"8","regionName":"Fremorn","factionId":"39","factionName":"Outlaws"},{"name":"Slupsk","regionId":"9","regionName":"Norweld","factionId":"37","factionName":"Nordlanders"},{"name":"Snaresheath","regionId":"70","regionName":"Kingslands","factionId":"103","factionName":"Virten"},{"name":"Southforge","regionId":"61","regionName":"Aindara","factionId":"103","factionName":"Virten"},{"name":"Ssaharin","regionId":"22","regionName":"Kul Tar","factionId":"31","factionName":"Sslipentin"},{"name":"Stoneberg","regionId":"73","regionName":"Strendur","factionId":"103","factionName":"Virten"},{"name":"Stormstone","regionId":"38","regionName":"Stormstone Island","factionId":"47","factionName":"Tenaril"},{"name":"Styne","regionId":"3","regionName":"Wolgast","factionId":"19","factionName":"Fyrgis"},{"name":"Suiqias","regionId":"41","regionName":"Puchuallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Swiftstead","regionId":"6","regionName":"Windlost","factionId":"48","factionName":"Tewhirrus"},{"name":"Syguf Barracks","regionId":"70","regionName":"Kingslands","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Tantark","regionId":"2","regionName":"Kal Tirikan","factionId":"8","factionName":"Tantarkim"},{"name":"Tarponsea","regionId":"29","regionName":"Azura","factionId":"5","factionName":"Azure Throne"},{"name":"Tattershill","regionId":"50","regionName":"Northmarch","factionId":"65","factionName":"Illyria Trade Council"},{"name":"Tayaribe","regionId":"43","regionName":"Pamanyallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Temple of Compassion","regionId":"49","regionName":"Glanhad","factionId":"83","factionName":"Argiri"},{"name":"Tempus","regionId":"21","regionName":"Perrigor","factionId":"9","factionName":"Circle Of Five"},{"name":"Terra","regionId":"18","regionName":"Tor Carrock","factionId":"9","factionName":"Circle Of Five"},{"name":"Tewhirria","regionId":"6","regionName":"Windlost","factionId":"48","factionName":"Tewhirrus"},{"name":"The Barracks","regionId":"6","regionName":"Windlost","factionId":"63","factionName":"Bayne's Irregulars"},{"name":"The Barrier","regionId":"9","regionName":"Norweld","factionId":"58","factionName":"Seelie Court"},{"name":"The Brothers' Haven","regionId":"51","regionName":"High Hills","factionId":"94","factionName":"Overoad Traders"},{"name":"The Empty Halls","regionId":"71","regionName":"Farshards","factionId":"98","factionName":"The Lost Clans"},{"name":"The Loft","regionId":"8","regionName":"Fremorn","factionId":"10","factionName":"Daedalons"},{"name":"Thop Fatoft","regionId":"12","regionName":"Taomist","factionId":"25","factionName":"Sma Uruk"},{"name":"Thoroe's Haven","regionId":"57","regionName":"Coanhara","factionId":"94","factionName":"Overoad Traders"},{"name":"Thujn-Huckut-Kalj","regionId":"66","regionName":"Vindorel","factionId":"85","factionName":"Drek-Hhakral"},{"name":"Tiena","regionId":"25","regionName":"Arran","factionId":"51","factionName":"Triumvirate"},{"name":"Tilverdale","regionId":"17","regionName":"Keppen","factionId":"22","factionName":"Tinkers"},{"name":"Tor Dannu","regionId":"18","regionName":"Tor Carrock","factionId":"76","factionName":"Clan Moedagh"},{"name":"Tor Gorrak","regionId":"18","regionName":"Tor Carrock","factionId":"75","factionName":"Clan Bealagh"},{"name":"Torkurz","regionId":"75","regionName":"Jurgor","factionId":"86","factionName":"Free Orcs"},{"name":"Tractus","regionId":"59","regionName":"Newlands","factionId":"99","factionName":"The New Light"},{"name":"Trazurk","regionId":"76","regionName":"The Long White","factionId":"86","factionName":"Free Orcs"},{"name":"Trimek Barracks","regionId":"74","regionName":"Chulbran","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Tringar Trading Post","regionId":"1","regionName":"The Wastes","factionId":"66","factionName":"Trappers"},{"name":"Trottingham","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Tubriq","regionId":"26","regionName":"Turalia","factionId":"54","factionName":"Udan Tebriz"},{"name":"Tudeyll","regionId":"24","regionName":"Lan Larosh","factionId":"79","factionName":"Turyns"},{"name":"Tundale","regionId":"11","regionName":"Ragallon","factionId":"1","factionName":"Council of Illyria"},{"name":"Turyin","regionId":"24","regionName":"Lan Larosh","factionId":"79","factionName":"Turyns"},{"name":"Udalia","regionId":"26","regionName":"Turalia","factionId":"54","factionName":"Udan Tebriz"},{"name":"Uruk Vadokan","regionId":"13","regionName":"Meilla","factionId":"67","factionName":"Sundog Gith"},{"name":"Urukonium","regionId":"16","regionName":"Mal Motsha","factionId":"72","factionName":"Pax Orcana"},{"name":"Va Drangu","regionId":"3","regionName":"Wolgast","factionId":"38","factionName":"Drangu Mundas"},{"name":"Vdekurch","regionId":"76","regionName":"The Long White","factionId":"86","factionName":"Free Orcs"},{"name":"Vduch","regionId":"71","regionName":"Farshards","factionId":"86","factionName":"Free Orcs"},{"name":"Veem","regionId":"31","regionName":"OCEAN","factionId":"88","factionName":"Heirs of Voyldarn"},{"name":"Veritas","regionId":"14","regionName":"Lucerna","factionId":"7","factionName":"Fratri Hominis"},{"name":"Verity City","regionId":"8","regionName":"Fremorn","factionId":"55","factionName":"Undying Flame"},{"name":"Votar","regionId":"13","regionName":"Meilla","factionId":"67","factionName":"Sundog Gith"},{"name":"Vrakun","regionId":"75","regionName":"Jurgor","factionId":"86","factionName":"Free Orcs"},{"name":"Windmire","regionId":"73","regionName":"Strendur","factionId":"103","factionName":"Virten"},{"name":"Winter Haven","regionId":"73","regionName":"Strendur","factionId":"94","factionName":"Overoad Traders"},{"name":"Woodsedge","regionId":"15","regionName":"Middle Kingdom","factionId":"1","factionName":"Council of Illyria"},{"name":"Wylvale","regionId":"67","regionName":"Almenly","factionId":"97","factionName":"The College of Silence"},{"name":"Xcrix","regionId":"9","regionName":"Norweld","factionId":"2","factionName":"Arakvar"},{"name":"Zaqcha","regionId":"41","regionName":"Puchuallpa","factionId":"104","factionName":"Yuraquncha"},{"name":"Zgudun Barracks","regionId":"73","regionName":"Strendur","factionId":"91","factionName":"Kartur-Hhakral"},{"name":"Zhuhai","regionId":"10","regionName":"Laoshin","factionId":"56","factionName":"Wen Kun Dynasty"}];
dt.cbtTerrains = {
	"Plains": {equip:[]},
	"Buildings": {equip:[]},
	"Large Hill": {equip:['on Hills']},
	"Small Hill": {equip:['on Hills']},
	"Impassable": {equip:[]},
	"Large Mountain": {equip:['on Mountains']},
	"Small Mountain": {equip:['on Mountains']},
	"Large Forest": {equip:['in Forests']},
	"Small Forest": {equip:['in Forests']},
	"Fresh Water": {equip:[]},
	"Tidal Water": {equip:[]},
	"Shallow Salt Water": {equip:[]},
	"Ocean": {equip:[]},
	"Obsidian Mountains": {equip:[]}
};
dt.biomes = {
	"Desert":{equip:['in Desert']},
	"Arctic":{equip:['in Arctic']},
	"Forest":{equip:['in Forest']},
	"Jungle":{equip:['in Jungle']}
};
dt.cbtTerrainMods = {
	"Buildings": {atk: {cav: "-50", spear: "20", inf: "40", range: "5"}, def: {cav: "-45", spear: "+25", inf: "30", range: "15"}},
	"Large Forest": {atk: {cav: "-15", spear: "5", inf: "30", range: "-20"}, def: {cav: "-15", spear: "5", inf: "25", range: "-20"}},
	"Large Hill": {atk: {cav: "-15", spear: "0", inf: "10", range: "10"}, def: {cav: "-10", spear: "10", inf: "5", range: "15"}},
	"Large Mountain": {atk: {cav: "-30", spear: "0", inf: "15", range: "15"}, def: {cav: "-30", spear: "20", inf: "5", range: "30"}},
	"Plains": {atk: {cav: "30", spear: "-15", inf: "0", range: "0"}, def: {cav: "25", spear: "-15", inf: "0", range: "0"}},
	"Small Forest": {atk: {cav: "-10", spear: "10", inf: "30", range: "-10"}, def: {cav: "-10", spear: "10", inf: "30", range: "-10"}},
	"Small Hill": {atk: {cav: "0", spear: "0", inf: "0", range: "0"}, def: {cav: "0", spear: "5", inf: "5", range: "5"}},
	"Small Mountain": {atk: {cav: "-20", spear: "0", inf: "5", range: "10"}, def: {cav: "-15", spear: "15", inf: "5", range: "15"}}};
dt.allembineSites = {1: {a: {coords: [ - 312, 717]}, name: 'Aldelbrecht'}, 2: {a: {coords: [ - 608, - 189]}, name: 'Asmorn'}, 3: {a: {coords: [910, - 507]}, name: 'Bjander-Thegn'}, 4: {a: {coords: [ - 102, 1]}, b: {coords: [949, - 379]}, name: 'Braynard'}, 5: {a: {coords: [429, - 931]}, name: 'Burkhardt'}, 6: {a: {coords: [516, 505]}, name: 'Eberekt'}, 7: {a: {coords: [ - 485, - 736]}, b: {coords: [669, 794]}, name: 'Ferhonanth'}, 8: {a: {coords: [652, - 265]}, b: {coords: [ - 843, - 310]}, name: 'Fridholt'}, 9: {a: {coords: [ - 4, 10]}, name: 'Gunthar'}, 10: {a: {coords: [ - 377, - 104]}, name: 'Haestin'}, 11: {a: {coords: [356, 498]}, name: 'Keldar'}, 12: {a: {coords: [ - 157, - 298]}, name: 'Lambercht'}, 13: {a: {coords: [471, 228]}, b: {coords: [ - 711, 674]}, name: 'Lindricht'}, 14: {a: {coords: [232, 345]}, name: 'Oddric'}, 15: {a: {coords: [761, - 892]}, b: {coords: [ - 39, - 973]}, name: 'Osgaut'}, 16: {a: {coords: [ - 194, - 687]}, name: 'Raedborth'}, 17: {a: {coords: [719, 365]}, name: 'Regindalt'}, 18: {a: {coords: [22, - 85]}, name: 'Rikert'}, 19: {a: {coords: [ - 569, - 91]}, b: {coords: [136, 855]}, name: 'Tormand'}, 20: {a: {coords: [399, - 431]}, name: 'Ulvelaik'}};
dt.unitLvls = {t1:[46,48,50,52,56,58,60,62,66,68,70,72,76,78,80,82,86,88,90,92],t2:[47,49,51,53,57,59,61,63,67,69,71,73,77,79,81,83,87,89,91,94]};
dt.unitTypes = {spear:[46,47,48,49,50,51,52,53],inf:[56,57,58,59,60,61,62,63],bow:[66,67,68,69,70,71,72,73],cav:[76,77,78,79,80,81,82,83],siege:[86,87,88,89,90,91,92,94]};
dt.mount = [33,34,35,36,37,422,423,424,425,427,428,429,430,431];
dt.spear = [91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,478,479,480,481,482,483,484,485,486,487,488,489,490,491,492,500,497,499,495,496,493,494,498];
dt.swords = [40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,432,433,434,435,436,437,438,439,440,441,442,443,444,445,446,453,454,452,447,448,449,450,451];
dt.bows = [67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,455,456,457,458,459,460,461,462,463,464,465,466,467,468,469,475,476,472,473,470,471,477,474];
dt.leather = [120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,501,502,503,504,505,506,507,508,509,510,518,517,513,514,516,515,511,512];
dt.chainmail = [141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,519,520,521,522,523,524,525,526,527,528,536,535,533,529,530,534,532,531];
dt.platemail = [161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,537,538,539,540,541,542,543,544,545,546,554,553,551,547,548,549,550,552];
dt.armour = [120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,501,502,503,504,505,506,507,508,509,510,511,512,513,514,515,516,517,518,519,520,521,522,523,524,525,526,527,528,529,530,531,532,533,534,535,536,537,538,539,540,541,542,543,544,545,546,547,548,549,550,551,552,553,554];
dt.weapon = [40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,426,432,433,434,435,436,437,438,439,450,451,452,456,457,458,459,460,461,462,463,464,465,466,467,468,469,470,471,472,473,474,475,476,477,478,479,480,481,42,483,484,485,486,487,488,489,490,491,492,493,494,495,496,497,498,499,500];
dt.cItems={
    "33": {
        "name": "Draught Horse",
        "classes": "itemsprite draughthorse48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+25|0|0|0|0|0|1|0|0|0|0"
    },
    "34": {
        "name": "Heavy Warhorse",
        "classes": "itemsprite heavyhorse48",
        "e": "0|-5|-5|-5|-5|-5|-5|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+5|-5|+10|0|0|0|0|1|0|0|0|0"
    },
    "35": {
        "name": "Nimble Warhorse",
        "classes": "itemsprite nimblehorse48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+24|+24|+24|+24|-8|0|0|0|0|0|0|1|0|0|0|0"
    },
    "36": {
        "name": "Riding Horse",
        "classes": "itemsprite ridinghorse48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+25|0|0|0|0|0|1|0|0|0|0"
    },
    "37": {
        "name": "Steady Warhorse",
        "classes": "itemsprite steadyhorse48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+5|0|0|0|1|0|0|0|0"
    },
    "40": {
        "name": "Adventurer's Sword",
        "classes": "itemsprite adventurers48",
        "e": "0|0|0|0|-24|+60|-24|+20|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "41": {
        "name": "Barbarian Sword",
        "classes": "itemsprite barbarian48",
        "e": "0|0|0|0|-24|-24|+20|+60|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "42": {
        "name": "Battle Sword",
        "classes": "itemsprite battlesword48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+80|-16|-16|-16|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "43": {
        "name": "Bone Handled Sword",
        "classes": "itemsprite bonehandled48",
        "e": "0|+30|-8|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "44": {
        "name": "Chitin-Crafted Shortsword",
        "classes": "itemsprite chitincrafted48",
        "e": "0|0|0|0|+50|0|+40|0|0|0|-8|0|0|+10|0|0|0|-14|-14|-14|-14|-12|+15|0|0|0|1|0|0|0|0|0|0"
    },
    "45": {
        "name": "Dwarven Battle Axe",
        "classes": "itemsprite dwarvenaxe48",
        "e": "0|0|0|0|-4|-4|0|-2|0|0|+6|0|0|0|0|0|0|-4|-4|-4|-4|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "46": {
        "name": "Ebony-Hilt Sword",
        "classes": "itemsprite ebonyhilt48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-4|0|0|0|0|0|1|0|0|0|0|0|+120"
    },
    "47": {
        "name": "Elven Sword",
        "classes": "itemsprite elvishsword48",
        "e": "0|0|0|0|0|-4|-2|-2|0|0|0|+6|0|0|0|0|0|0|0|0|0|-2|0|0|0|0|1|0|0|0|0|0|0"
    },
    "48": {
        "name": "Fang-Barbed Sword",
        "classes": "itemsprite fangbarbed48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-12|-12|+90|-48|+20|0|0|0|0|1|0|0|0|0|0|0"
    },
    "49": {
        "name": "War Axe",
        "classes": "itemsprite waraxe48",
        "e": "0|0|0|0|0|0|0|0|0|0|+30|-8|-4|+10|0|0|0|-8|-8|-8|-8|+30|-10|0|0|0|1|0|0|0|0|0|0"
    },
    "50": {
        "name": "Light Sword",
        "classes": "itemsprite lightsword48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|+10|0|0|0|1|0|0|0|0|0|0"
    },
    "51": {
        "name": "Longsword",
        "classes": "itemsprite longsword48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|-16|+80|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "52": {
        "name": "Machete",
        "classes": "itemsprite machete48",
        "e": "0|-8|-8|+30|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "53": {
        "name": "Mountain-Tribe's Sword",
        "classes": "itemsprite mountaintribe48",
        "e": "0|0|0|0|+20|-24|+60|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "54": {
        "name": "Obsidian Blade",
        "classes": "itemsprite obsidiansword48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|-8|-8|0|0|0|+125|0|1|0|0|0|0|0|+50"
    },
    "55": {
        "name": "Orc Marauder's Sword",
        "classes": "itemsprite orcmarauder48",
        "e": "0|0|0|0|0|0|0|0|0|0|-24|-32|-24|+60|0|0|0|-28|-40|-24|-8|+80|-10|0|0|0|1|0|0|0|0|0|0"
    },
    "56": {
        "name": "Orcish Sword",
        "classes": "itemsprite orcishsword48",
        "e": "0|0|0|0|-2|0|-2|-4|0|0|0|0|0|+6|0|0|0|-2|-2|-2|-2|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "57": {
        "name": "Razor-Edged Sword",
        "classes": "itemsprite razoredged48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+10|+10|+10|+10|+10|0|0|0|0|1|0|0|0|0|0|0"
    },
    "58": {
        "name": "Reinforced Sword",
        "classes": "itemsprite reinforced48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+140|0|-8|-8|-8|-8|+10|0|0|0|0|1|0|0|0|0|0|0"
    },
    "59": {
        "name": "Sabre",
        "classes": "itemsprite sabre48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|+80|-16|-16|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "60": {
        "name": "Scimitar",
        "classes": "itemsprite scimitar48",
        "e": "0|-8|+30|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "61": {
        "name": "Seaxe",
        "classes": "itemsprite seax48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|+80|-16|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "62": {
        "name": "Short Sword",
        "classes": "itemsprite shortsword48",
        "e": "0|0|0|0|+60|+20|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0"
    },
    "63": {
        "name": "Silversteel Sword",
        "classes": "itemsprite silversteel48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+50|+25|+75|+150|0|0|0|0|1|0|0|0|0|0|0"
    },
    "64": {
        "name": "Spiked Sword",
        "classes": "itemsprite spikedsword48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|-16|-16|+40|0|0|0|0|1|0|0|0|0|0|0"
    },
    "65": {
        "name": "Traditional Sword",
        "classes": "itemsprite traditionalsword48",
        "e": "0|0|0|0|-2|-2|-4|0|0|0|0|0|+6|0|0|0|0|0|0|0|0|-2|0|0|0|0|1|0|0|0|0|0|0"
    },
    "67": {
        "name": "Arctic Bow",
        "classes": "itemsprite arcticbow48",
        "e": "0|+30|-8|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "68": {
        "name": "Chitin-Cored Bow",
        "classes": "itemsprite chitincored48",
        "e": "0|-32|-32|-32|0|0|0|0|0|0|0|0|0|0|0|0|0|+40|+40|+40|+40|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "69": {
        "name": "Desert Bow",
        "classes": "itemsprite desertbow48",
        "e": "0|-8|+30|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "70": {
        "name": "Ebony Bow",
        "classes": "itemsprite ebonybow48",
        "e": "0|0|0|0|0|0|0|0|-16|+30|0|0|0|0|0|-12|0|-4|-4|-4|-4|-8|0|0|0|0|0|1|0|0|0|0|+90"
    },
    "71": {
        "name": "Elven Spirit Bow",
        "classes": "itemsprite elvenspirit48",
        "e": "0|-36|-36|-36|0|0|0|0|+30|0|0|+120|0|0|0|0|0|-12|-12|-12|-12|-24|+5|0|0|0|0|1|0|0|0|0|0"
    },
    "72": {
        "name": "Hero's Bow",
        "classes": "itemsprite herosbow48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+135|0|-4|-4|-4|-4|-4|-10|0|0|0|0|1|0|0|0|0|0"
    },
    "73": {
        "name": "High-Power Bow",
        "classes": "itemsprite highpoweredbow48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|-16|+80|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "74": {
        "name": "Hillsman's Bow",
        "classes": "itemsprite mercbow48",
        "e": "0|0|0|0|-24|+60|-24|+20|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "75": {
        "name": "Hunter's Bow",
        "classes": "itemsprite huntersbow48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+120|0|0|-4|-4|-4|-4|-4|0|0|0|0|0|1|0|0|0|0|0"
    },
    "76": {
        "name": "Composite Bow",
        "classes": "itemsprite ironsprung48",
        "e": "0|-32|-32|-32|0|0|0|0|0|0|0|0|0|0|0|0|0|-10|-10|-10|-10|+75|-20|0|0|0|0|1|0|0|0|0|0"
    },
    "77": {
        "name": "Jungle Bow",
        "classes": "itemsprite junglebow48",
        "e": "0|-8|-8|+30|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "78": {
        "name": "Leopard Gut Bow",
        "classes": "itemsprite leopardgut48",
        "e": "0|-8|-8|+30|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|+5|0|0|0|0|1|0|0|0|0|0"
    },
    "79": {
        "name": "Light Bow",
        "classes": "itemsprite lightbow48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|+3|0|0|0|0|0|0|0|0|0|-8|+10|0|0|0|0|1|0|0|0|0|0"
    },
    "80": {
        "name": "Long-Draw Bow",
        "classes": "itemsprite longdraw48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|+80|-16|-16|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "81": {
        "name": "Mammoth Tusk Bow",
        "classes": "itemsprite mammothtusk48",
        "e": "0|+30|-8|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|1|0|0|0|0|0"
    },
    "82": {
        "name": "Marksman's Bow",
        "classes": "itemsprite marksmansbow48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+80|-16|-16|-16|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "83": {
        "name": "Mountain Bow",
        "classes": "itemsprite mountainbow48",
        "e": "0|0|0|0|+20|-24|+60|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "84": {
        "name": "Plains Bow",
        "classes": "itemsprite plainsbow48",
        "e": "0|0|0|0|-24|-24|+20|+60|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "85": {
        "name": "Poacher's Bow",
        "classes": "itemsprite poachersbow48",
        "e": "0|0|0|0|+20|+20|0|-12|0|0|0|0|0|0|+100|-36|0|-8|-8|-8|-8|0|+5|0|0|0|0|1|0|0|0|0|0"
    },
    "86": {
        "name": "Rapid-Draw Bow",
        "classes": "itemsprite rapiddraw48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|+80|-16|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "87": {
        "name": "Three-Wood Bow",
        "classes": "itemsprite threewood48",
        "e": "0|-32|-32|-32|0|0|0|0|0|0|0|+15|0|0|0|0|0|+25|+25|+25|+25|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "88": {
        "name": "War Bow",
        "classes": "itemsprite warbow48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+6|-15|0|0|0|0|1|0|0|0|0|0"
    },
    "89": {
        "name": "Woodsman's Bow",
        "classes": "itemsprite woodsmansbow48",
        "e": "0|0|0|0|+60|+20|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0"
    },
    "91": {
        "name": "Arctic Spear",
        "classes": "itemsprite arcticspear48",
        "e": "0|+30|-8|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "92": {
        "name": "Battle Spear",
        "classes": "itemsprite battlespear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|-16|+80|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "93": {
        "name": "Boar Spear",
        "classes": "itemsprite boarspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+120|0|0|-4|0|-4|-4|-16|0|0|0|1|0|0|0|0|0|0|0"
    },
    "94": {
        "name": "Defender's Spear",
        "classes": "itemsprite defendersspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+80|-16|-16|-16|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "95": {
        "name": "Desert Pike",
        "classes": "itemsprite desertpike48",
        "e": "0|0|+40|-16|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|+20|-16|-16|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "96": {
        "name": "Desert Spear",
        "classes": "itemsprite desertspear48",
        "e": "0|-8|+30|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "97": {
        "name": "Dragon Spear",
        "classes": "itemsprite dragonspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+150|0|-8|0|-8|-8|-16|-10|0|0|1|0|0|0|0|0|0|0"
    },
    "98": {
        "name": "Duelling Spear",
        "classes": "itemsprite duelingspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|-16|+80|-16|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "99": {
        "name": "Ebony Spear",
        "classes": "itemsprite ebonyspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-4|0|0|0|0|1|0|0|0|0|0|0|+60"
    },
    "100": {
        "name": "Fang-Barbed Spear",
        "classes": "itemsprite fangbarbedspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|-8|+60|-32|+10|0|0|0|1|0|0|0|0|0|0|0"
    },
    "101": {
        "name": "Fang-Tipped Spear",
        "classes": "itemsprite fangtippedspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+40|0|0|+20|-24|+40|-24|0|+5|0|0|1|0|0|0|0|0|0|0"
    },
    "102": {
        "name": "Forester's Spear",
        "classes": "itemsprite forestspear48",
        "e": "0|0|0|0|+60|+20|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "103": {
        "name": "Harpoon-Spear",
        "classes": "itemsprite harpoon48",
        "e": "0|+20|-4|-4|0|0|0|0|0|0|0|0|0|0|+100|0|0|-16|-16|-16|-16|-8|+5|0|0|1|0|0|0|0|0|0|0"
    },
    "104": {
        "name": "Hill-Tribe Spear",
        "classes": "itemsprite hilltribespear48",
        "e": "0|0|0|0|-24|+60|-24|+20|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "105": {
        "name": "Iron-Banded Spear",
        "classes": "itemsprite bandedspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|+10|0|+60|0|0|+40|0|+40|-4|-10|0|0|1|0|0|0|0|0|0|0"
    },
    "106": {
        "name": "Jungle Hunter's Spear",
        "classes": "itemsprite junglehunter48",
        "e": "0|-4|-4|+20|0|0|0|0|0|0|0|0|0|0|+90|0|0|-12|-12|-12|-12|-4|+10|0|0|1|0|0|0|0|0|0|0"
    },
    "107": {
        "name": "Jungle Spear",
        "classes": "itemsprite junglespear48",
        "e": "0|-8|-8|+30|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "108": {
        "name": "Light Spear",
        "classes": "itemsprite lightspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|+10|0|0|1|0|0|0|0|0|0|0"
    },
    "109": {
        "name": "Mountain Spear",
        "classes": "itemsprite mountainspear48",
        "e": "0|0|0|0|+20|-24|+60|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "110": {
        "name": "Obsidian-Tipped Spear",
        "classes": "itemsprite obsidiantipped48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|0|0|0|0|-8|0|0|0|+50|1|0|0|0|0|0|0|+20"
    },
    "111": {
        "name": "Pike",
        "classes": "itemsprite pike48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-16|+80|-16|-16|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "112": {
        "name": "Plainsman's Spear",
        "classes": "itemsprite plainsmansspear48",
        "e": "0|0|0|0|-24|-24|+20|+60|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0|0|0|0|0"
    },
    "113": {
        "name": "Razor-Edged Spear",
        "classes": "itemsprite razoredgedspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+10|+10|+10|+10|+10|0|0|0|1|0|0|0|0|0|0|0"
    },
    "114": {
        "name": "Silversteel Spear",
        "classes": "itemsprite silversteelspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+25|0|+25|+50|0|0|0|1|0|0|0|0|0|0|0"
    },
    "115": {
        "name": "Trident",
        "classes": "itemsprite trident48",
        "e": "0|-2|-2|+10|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|0|-4|+60|-4|0|0|0|1|0|0|0|0|0|0|0"
    },
    "116": {
        "name": "War Spear",
        "classes": "itemsprite warspear48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+6|-15|0|0|1|0|0|0|0|0|0|0"
    },
    "120": {
        "name": "Animal-Scale Armour",
        "classes": "itemsprite animalscale48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|+15|+120|+120|0|-4|-4|-4|-4|0|+10|0|0|0|0|0|0|1|0|0|+120"
    },
    "121": {
        "name": "Cloth-Backed Leather",
        "classes": "itemsprite clothbackedleath48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+120|-12|-12|-12|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "122": {
        "name": "Desert Armour",
        "classes": "itemsprite desertleather48",
        "e": "0|-8|+40|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "123": {
        "name": "Extra Heavy Armour",
        "classes": "itemsprite xheavyleather48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+100|+100|+100|+100|-8|-40|0|0|0|0|0|0|1|0|0|0"
    },
    "124": {
        "name": "Extra Light Armour",
        "classes": "itemsprite xlightleather48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|+5|0|+15|0|0|0|-10|-10|-10|-10|-10|+15|0|0|0|0|0|0|1|0|0|0"
    },
    "125": {
        "name": "Forester's Armour",
        "classes": "itemsprite forestersleath48",
        "e": "0|0|0|0|+60|+20|-24|-24|0|0|0|+40|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "126": {
        "name": "Fur-Lined Armour",
        "classes": "itemsprite furlinedleath48",
        "e": "0|+40|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "127": {
        "name": "Hardened Leather",
        "classes": "itemsprite hardenedleath48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-12|-12|+120|-12|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "128": {
        "name": "Heavy Leather",
        "classes": "itemsprite heavyleather48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+10|+10|+10|+10|-2|-10|0|0|0|0|0|0|1|0|0|0"
    },
    "129": {
        "name": "Highland Armour",
        "classes": "itemsprite highlandleather48",
        "e": "0|0|0|0|+20|-24|+60|-24|0|0|0|0|0|+40|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "130": {
        "name": "Jungle Armour",
        "classes": "itemsprite jungleleather48",
        "e": "0|-8|-8|+40|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "131": {
        "name": "Light Leather",
        "classes": "itemsprite lightleather48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|+10|0|+30|0|0|0|-4|-4|-4|-4|-4|+10|0|0|0|0|0|0|1|0|0|0"
    },
    "132": {
        "name": "Midnight Armour",
        "classes": "itemsprite midnightleather48",
        "e": "0|0|0|0|0|0|0|0|-12|+90|0|0|0|0|0|0|0|-4|-4|-4|-4|+10|0|0|0|0|0|0|0|1|0|0|0"
    },
    "133": {
        "name": "Over-Padded Armour",
        "classes": "itemsprite overpaddedleath48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+140|0|0|-16|-16|-16|-16|0|-10|0|0|0|0|0|0|1|0|0|0"
    },
    "134": {
        "name": "Plainsman's Armour",
        "classes": "itemsprite plainsleather48",
        "e": "0|0|0|0|-24|-24|+20|+60|0|0|0|0|+40|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "135": {
        "name": "Reinforced Leather",
        "classes": "itemsprite reinforcedleath48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-6|+24|-6|-6|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "136": {
        "name": "Splintmail",
        "classes": "itemsprite splintmail48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-12|-12|-12|+120|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "137": {
        "name": "Sun-Burnished Armour",
        "classes": "itemsprite sunburnished48",
        "e": "0|0|0|0|0|0|0|0|+90|-12|0|0|0|0|0|0|0|-4|-4|-4|-4|+10|0|0|0|0|0|0|0|1|0|0|0"
    },
    "138": {
        "name": "Upland Armour",
        "classes": "itemsprite uplandsleath48",
        "e": "0|0|0|0|-24|+60|-24|+20|0|0|+40|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0|0"
    },
    "139": {
        "name": "Vanguard's Armour",
        "classes": "itemsprite vanguardleath48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-12|-12|-12|-12|+100|-10|0|0|0|0|0|0|1|0|0|0"
    },
    "141": {
        "name": "Cloth-Backed Chainmail",
        "classes": "itemsprite clothbackedchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+80|-8|-8|-8|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "142": {
        "name": "Desert Chainmail",
        "classes": "itemsprite desertchain48",
        "e": "0|-8|+30|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "143": {
        "name": "Double-Weave Chainmail",
        "classes": "itemsprite doubleweave48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|+16|-4|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "144": {
        "name": "Extra Heavy Chainmail",
        "classes": "itemsprite xheavychain48",
        "e": "0|-8|-8|-8|0|0|0|0|0|0|0|0|0|0|0|0|0|+30|+30|+30|+30|-4|-40|0|0|0|0|0|0|0|1|0|0"
    },
    "145": {
        "name": "Extra Light Chainmail",
        "classes": "itemsprite xlightchain48",
        "e": "0|0|+25|+25|0|0|0|0|0|0|0|0|0|0|0|0|0|-10|-10|-10|-10|-10|+15|0|0|0|0|0|0|0|1|0|0"
    },
    "146": {
        "name": "Fur-Lined Chainmail",
        "classes": "itemsprite furlinedchain48",
        "e": "0|+30|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "147": {
        "name": "Heavy Chain Armour",
        "classes": "itemsprite heavychain48",
        "e": "0|-4|-4|-4|0|0|0|0|0|0|0|0|0|0|0|0|0|+4|+4|+4|+4|-2|-10|0|0|0|0|0|0|0|1|0|0"
    },
    "148": {
        "name": "Highland Chainmail",
        "classes": "itemsprite highlandchain48",
        "e": "0|0|0|0|+20|-24|+60|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "149": {
        "name": "Hillsman's Chainmail",
        "classes": "itemsprite hilltribechain48",
        "e": "0|0|0|0|-24|+60|-24|+20|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "150": {
        "name": "Jungle Chainmail",
        "classes": "itemsprite junglechain48",
        "e": "0|-8|-8|+30|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "151": {
        "name": "Light Chain Armour",
        "classes": "itemsprite lightchain48",
        "e": "0|0|+5|+5|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-4|-4|+10|0|0|0|0|0|0|0|1|0|0"
    },
    "152": {
        "name": "Master-Crafted Chainmail",
        "classes": "itemsprite masterchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|+50|0|0|0|0|+25|+25|+25|+25|-10|-10|0|0|0|0|0|0|0|1|0|0"
    },
    "153": {
        "name": "Over-Padded Chainmail",
        "classes": "itemsprite overpaddedchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+50|0|0|0|0|0|0|0|-10|0|0|0|0|0|0|0|1|0|0"
    },
    "154": {
        "name": "Plainsman's Chainmail",
        "classes": "itemsprite plainsmanschain48",
        "e": "0|0|0|0|-24|-24|+20|+60|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "155": {
        "name": "Reinforced Chainmail",
        "classes": "itemsprite reinforcedchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|+80|-8|-8|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "156": {
        "name": "Silversteel Chainmail",
        "classes": "itemsprite silversteelchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+75|+75|+75|+75|+40|0|0|0|0|0|0|0|0|1|0|0"
    },
    "157": {
        "name": "Thick-Ring Chainmail",
        "classes": "itemsprite thickringchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|-8|-8|+80|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "158": {
        "name": "Vanguard's Chainmail",
        "classes": "itemsprite vanguardchain48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-4|+5|0|0|0|0|0|0|0|0|1|0|0"
    },
    "159": {
        "name": "Woodsman's Chainmail",
        "classes": "itemsprite woodsmanschain48",
        "e": "0|0|0|0|+60|+20|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0|0"
    },
    "161": {
        "name": "Ancient Pattern Armour",
        "classes": "itemsprite ancientpattern48",
        "e": "0|-16|-16|-16|0|0|0|0|0|0|0|0|0|0|0|0|0|+50|+50|+50|+25|0|+5|0|0|0|0|0|0|0|0|1|0"
    },
    "162": {
        "name": "Chain-Edged Platemail",
        "classes": "itemsprite chainplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+16|-4|-4|-4|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "163": {
        "name": "Desert Platemail",
        "classes": "itemsprite desertplate48",
        "e": "0|-12|+30|-12|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "164": {
        "name": "Dwarven Champion's",
        "classes": "itemsprite dwarvenchamp48",
        "e": "0|-20|-20|-20|0|+30|+10|0|0|0|+40|-24|-24|-24|0|0|0|+70|+50|+70|+50|+20|-50|0|0|0|0|0|0|0|0|1|0"
    },
    "165": {
        "name": "Forester's Platemail",
        "classes": "itemsprite forestersplate48",
        "e": "0|0|0|0|+60|+20|-24|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "166": {
        "name": "Fur-Lined Platemail",
        "classes": "itemsprite furlinedplate48",
        "e": "0|+30|-36|-36|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "167": {
        "name": "Heavy Platemail",
        "classes": "itemsprite heavyplate48",
        "e": "0|-5|-5|-5|0|0|0|0|0|0|0|0|0|0|0|0|0|+4|+4|+4|+4|-2|-20|0|0|0|0|0|0|0|0|1|0"
    },
    "168": {
        "name": "Highland Platemail",
        "classes": "itemsprite highlandersplate48",
        "e": "0|0|0|0|+20|-24|+60|-24|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "169": {
        "name": "Jungle Platemail",
        "classes": "itemsprite jungleplate48",
        "e": "0|-12|-12|+30|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "170": {
        "name": "Light Platemail",
        "classes": "itemsprite lightplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-4|-4|+10|0|0|0|0|0|0|0|0|1|0"
    },
    "171": {
        "name": "Obsidian Platemail",
        "classes": "itemsprite obsidianplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-6|-4|-10|0|+200|0|0|0|0|0|0|1|+35"
    },
    "172": {
        "name": "Plainsman's Platemail",
        "classes": "itemsprite plainsmansplate48",
        "e": "0|0|0|0|-24|-24|+20|+60|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "173": {
        "name": "Reinforced Platemail",
        "classes": "itemsprite reinforcedplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|+80|-8|-8|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "174": {
        "name": "Silversteel Platemail",
        "classes": "itemsprite silversteelplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+75|+75|+75|+75|+40|0|0|0|0|0|0|0|0|0|1|0"
    },
    "175": {
        "name": "Spiked Platemail",
        "classes": "itemsprite spikedplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+50|+90|0|-4|-4|-4|-4|+10|0|0|0|0|0|0|0|0|0|1|0"
    },
    "176": {
        "name": "Thickened Platemail",
        "classes": "itemsprite thickenedplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|+16|-4|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "177": {
        "name": "Under-Padded Platemail",
        "classes": "itemsprite paddedplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-8|-8|-8|+80|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "178": {
        "name": "Upland Platemail",
        "classes": "itemsprite uplandplate48",
        "e": "0|0|0|0|-24|+60|-24|+20|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|1|0"
    },
    "179": {
        "name": "Vanguard's Platemail",
        "classes": "itemsprite vanguardplate48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-4|-4|-4|-4|+5|0|0|0|0|0|0|0|0|0|1|0"
    },
    "186": {
        "name": "Hides",
        "classes": "base pelts_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "193": {
        "name": "Arterium",
        "classes": "itemsprite arterium_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "194": {
        "name": "Earthblood",
        "classes": "itemsprite earthblood_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "195": {
        "name": "Deepsilver",
        "classes": "itemsprite deepsilver_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "196": {
        "name": "Pyrestone",
        "classes": "itemsprite pyrestone_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "197": {
        "name": "Trove",
        "classes": "itemsprite trove_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "198": {
        "name": "Rainbowstone",
        "classes": "itemsprite rainbowstone_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "199": {
        "name": "Claristrine",
        "classes": "itemsprite claristrine_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "200": {
        "name": "Elven Tears",
        "classes": "itemsprite elventears_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "201": {
        "name": "Svelaugh Sand",
        "classes": "itemsprite svelaughsand_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "202": {
        "name": "Aeghris",
        "classes": "itemsprite aeghris_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "203": {
        "name": "Daera",
        "classes": "itemsprite daera_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "204": {
        "name": "Flektrine",
        "classes": "itemsprite flektrine_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "205": {
        "name": "Almhurin",
        "classes": "itemsprite almhurin_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "206": {
        "name": "Silversoil",
        "classes": "itemsprite silversoil_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "207": {
        "name": "Obsidian",
        "classes": "itemsprite obsidian_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "208": {
        "name": "Goldstone",
        "classes": "itemsprite goldstone_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "209": {
        "name": "Night Diamond",
        "classes": "itemsprite nightdiamond_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "210": {
        "name": "Iceheart",
        "classes": "itemsprite iceheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "211": {
        "name": "Amar Shards",
        "classes": "itemsprite amarshards_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "212": {
        "name": "Silversteel",
        "classes": "itemsprite silversteel_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "224": {
        "name": "Pale Cedar Wood",
        "classes": "itemsprite palecedarwood_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "225": {
        "name": "Lemonwood Bough",
        "classes": "itemsprite lemonwoodbough_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "226": {
        "name": "Larken Wood",
        "classes": "itemsprite larkenwood_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "227": {
        "name": "Berbane Leaves",
        "classes": "itemsprite berbaneleaves_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "228": {
        "name": "Ysanberries",
        "classes": "itemsprite ysanberries_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "229": {
        "name": "Mabri Fruit",
        "classes": "itemsprite mabrifruit_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "230": {
        "name": "Punfruit",
        "classes": "itemsprite punfruit_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "231": {
        "name": "Baleberries",
        "classes": "itemsprite baleberries_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "232": {
        "name": "Ebony Wood",
        "classes": "itemsprite ebonywood_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "233": {
        "name": "Dyallom Gall",
        "classes": "itemsprite dyallomgall_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "234": {
        "name": "Warpwood Shoot",
        "classes": "itemsprite warpwoodshoot_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "235": {
        "name": "Ironstem Root",
        "classes": "itemsprite ironstem_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "236": {
        "name": "Miner's Bane",
        "classes": "itemsprite minersbane_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "237": {
        "name": "Snowbell Flowers",
        "classes": "itemsprite snowbell_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "238": {
        "name": "Toadcap Fungus",
        "classes": "itemsprite toadcap_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "239": {
        "name": "Furzion Seedpod",
        "classes": "itemsprite furzionseed_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "240": {
        "name": "Vistrok Flower",
        "classes": "itemsprite vistrokflower_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "241": {
        "name": "Rockweed Root",
        "classes": "itemsprite rockweed_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "242": {
        "name": "Suntree Haft",
        "classes": "itemsprite suntreehaft_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "243": {
        "name": "Brownback Moss",
        "classes": "itemsprite brownbackmoss_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "244": {
        "name": "Silverthorn",
        "classes": "itemsprite silverthorn_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "245": {
        "name": "Sharproot",
        "classes": "itemsprite sharproot_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "246": {
        "name": "Ancient Oak",
        "classes": "itemsprite ancientoak_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "247": {
        "name": "Rahan Palm Wood",
        "classes": "itemsprite rahanpalmwood_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "248": {
        "name": "Queen's Hair Leaves",
        "classes": "itemsprite queenshairleaves_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "249": {
        "name": "Spidertree Leaves",
        "classes": "itemsprite spidertreeleaves_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "250": {
        "name": "Brascan Seeds",
        "classes": "itemsprite braslanseeds_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "251": {
        "name": "Giant Palm Leaves",
        "classes": "itemsprite giantpalmleaves_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "252": {
        "name": "Desert Flame",
        "classes": "itemsprite desertflame_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "253": {
        "name": "Grapes",
        "classes": "base grapes_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "255": {
        "name": "Corrupted Human Tentacle",
        "classes": "base tentacle_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "256": {
        "name": "Corrupted Elf Tentacle",
        "classes": "base tentacle_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "257": {
        "name": "Corrupted Dwarf Tentacle",
        "classes": "base tentacle_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "258": {
        "name": "Corrupted Orc Tentacle",
        "classes": "base tentacle_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "259": {
        "name": "Prestige Scrap",
        "classes": "scrolls s0",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "260": {
        "name": "Prestige Parchment",
        "classes": "scrolls s1",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "261": {
        "name": "Prestige Page",
        "classes": "scrolls s2",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "262": {
        "name": "Prestige Scroll",
        "classes": "scrolls s3",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "263": {
        "name": "Prestige Codex",
        "classes": "scrolls s4",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "264": {
        "name": "Prestige Book",
        "classes": "scrolls s5",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "265": {
        "name": "Prestige Tome",
        "classes": "scrolls s6",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "267": {
        "name": "Heart of Darkness",
        "classes": "hod l",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "268": {
        "name": "Lesser Mana Crystal",
        "classes": "item3 s48 i268",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "269": {
        "name": "Mana Crystal",
        "classes": "item3 s48 i269",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "270": {
        "name": "Greater Mana Crystal",
        "classes": "item3 s48 i270",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "271": {
        "name": "Shard of Nightfall",
        "classes": "item3 s48 i271",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "272": {
        "name": "Essence of Twilight",
        "classes": "item3 s48 i272",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "273": {
        "name": "Shard of Dawn",
        "classes": "item3 s48 i273",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "274": {
        "name": "Essence of Daybreak",
        "classes": "item3 s48 i274",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "275": {
        "name": "Shard of Putrescence",
        "classes": "item3 s48 i275",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "276": {
        "name": "Essence of Necrosis",
        "classes": "item3 s48 i276",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "277": {
        "name": "Shard of Life",
        "classes": "item3 s48 i277",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "278": {
        "name": "Essence of Anima",
        "classes": "item3 s48 i278",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "279": {
        "name": "Shard of Haste",
        "classes": "item3 s48 i279",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "280": {
        "name": "Esssence of Alacrity",
        "classes": "item3 s48 i280",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "281": {
        "name": "Shard of Ferocity",
        "classes": "item3 s48 i281",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "282": {
        "name": "Essence of Brutality",
        "classes": "item3 s48 i282",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "283": {
        "name": "Shard of Fortitude",
        "classes": "item3 s48 i283",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "284": {
        "name": "Essence of Protection",
        "classes": "item3 s48 i284",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "290": {
        "name": "Air Salt",
        "classes": "base elemairsalt_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "291": {
        "name": "Earth Salt",
        "classes": "base elemearthsalt_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "292": {
        "name": "Fire Salt",
        "classes": "base elemfiresalt_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "293": {
        "name": "Water Salt",
        "classes": "base elemwatersalt_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "295": {
        "name": "Mana",
        "classes": "base mana_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "296": {
        "name": "Research",
        "classes": "base research_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "298": {
        "name": "Alligator Heart",
        "classes": "animsprite alliheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "299": {
        "name": "Alligator Skull",
        "classes": "animsprite alliskull_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "300": {
        "name": "Alligator Entrail",
        "classes": "animsprite allientrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "301": {
        "name": "Alligator Vertebra",
        "classes": "animsprite alliverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "302": {
        "name": "Anaconda Heart",
        "classes": "animsprite anaheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "303": {
        "name": "Anaconda Fang",
        "classes": "animsprite anafangs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "304": {
        "name": "Anaconda Skin",
        "classes": "animsprite anaskin_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "305": {
        "name": "Arctic Wolf Heart",
        "classes": "animsprite arcwolfheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "306": {
        "name": "Arctic Wolf Entrail",
        "classes": "animsprite arcwolfentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "307": {
        "name": "Arctic Wolf Fur",
        "classes": "animsprite arcwolffur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "308": {
        "name": "Arctic Wolf Tooth",
        "classes": "animsprite arcwolfteeth_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "309": {
        "name": "Baboon Heart",
        "classes": "animsprite babheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "310": {
        "name": "Baboon Entrail",
        "classes": "animsprite babentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "311": {
        "name": "Baboon Fur",
        "classes": "animsprite babfurs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "312": {
        "name": "Black Bear Heart",
        "classes": "animsprite blackbheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "313": {
        "name": "Black Bear Tooth",
        "classes": "animsprite blackbteeth_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "314": {
        "name": "Black Bear Fur",
        "classes": "animsprite blackbfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "315": {
        "name": "Black Bear Vertebra",
        "classes": "animsprite blackbverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "316": {
        "name": "Black Panther Heart",
        "classes": "animsprite blackpheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "317": {
        "name": "Black Panther Vertebra",
        "classes": "animsprite blackpverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "318": {
        "name": "Black Panther Fur",
        "classes": "animsprite blackpfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "319": {
        "name": "Brown Bear Heart",
        "classes": "animsprite brownbheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "320": {
        "name": "Brown Bear Fur",
        "classes": "animsprite brownbfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "321": {
        "name": "Brown Bear Vertebra",
        "classes": "animsprite brownbverti_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "322": {
        "name": "Cobra Heart",
        "classes": "animsprite cobraheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "323": {
        "name": "Cobra Brain",
        "classes": "animsprite cobrabrains_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "324": {
        "name": "Coral Snake Heart",
        "classes": "animsprite coralheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "325": {
        "name": "Coral Snake Brain",
        "classes": "animsprite coralbrains_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "326": {
        "name": "Cyclops Heart",
        "classes": "animsprite cyclopsheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "327": {
        "name": "Cyclops Entrail",
        "classes": "animsprite cyclopsentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "328": {
        "name": "Elephant Heart",
        "classes": "animsprite elephantheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "329": {
        "name": "Elephant Hide",
        "classes": "animsprite elephanthide_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "330": {
        "name": "Elephant Rib",
        "classes": "animsprite elephantribs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "331": {
        "name": "Elephant Vertebra",
        "classes": "animsprite elephantverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "332": {
        "name": "Fire Salamander Heart",
        "classes": "animsprite firesalheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "333": {
        "name": "Fire Salamander Entrail",
        "classes": "animsprite firesalent_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "334": {
        "name": "Fire Salamander Skin",
        "classes": "animsprite firesalskin_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "335": {
        "name": "Gharial Heart",
        "classes": "animsprite gharheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "336": {
        "name": "Gharial Tooth",
        "classes": "animsprite gharteeth_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "337": {
        "name": "Giant Beetle Heart",
        "classes": "animsprite giantcara_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "338": {
        "name": "Giant Beetle Carapace",
        "classes": "animsprite giantbeetheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "339": {
        "name": "Giant Rat Heart",
        "classes": "animsprite giantratheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "340": {
        "name": "Giant Rat Rib",
        "classes": "animsprite giantratribs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "341": {
        "name": "Giant Rat Fur",
        "classes": "animsprite giantratfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "342": {
        "name": "Giant Scorpion Heart",
        "classes": "animsprite giantscorpheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "343": {
        "name": "Giant Scorpion Exoskeleton",
        "classes": "animsprite giantscorpexo_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "344": {
        "name": "Giant Scorpion Stinger",
        "classes": "animsprite giantscorpstinger_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "345": {
        "name": "Giant Scuttler Heart",
        "classes": "animsprite giantscuttheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "346": {
        "name": "Giant Scuttler Exoskeleton",
        "classes": "animsprite giantscuttexo_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "347": {
        "name": "Giant Snake Heart",
        "classes": "animsprite giantsnakeheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "348": {
        "name": "Giant Snake Vertebra",
        "classes": "animsprite giantsnakeverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "349": {
        "name": "Giant Snake Fang",
        "classes": "animsprite giantsnakefangs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "350": {
        "name": "Giant Spider Heart",
        "classes": "animsprite giantspiderheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "351": {
        "name": "Giant Spider Fangs",
        "classes": "animsprite giantspiderfangs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "352": {
        "name": "Golden Monkey Heart",
        "classes": "animsprite goldenmonkeyheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "353": {
        "name": "Golden Monkey Entrail",
        "classes": "animsprite goldenmonkeyent_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "354": {
        "name": "Golden Monkey Fur",
        "classes": "animsprite goldenmonkeyfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "355": {
        "name": "Ice Salamander Heart",
        "classes": "animsprite icesalheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "356": {
        "name": "Ice Salamander Entrail",
        "classes": "animsprite icesalentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "357": {
        "name": "Ice Salamander Skin",
        "classes": "animsprite icesalskin_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "358": {
        "name": "Jaguar Heart",
        "classes": "animsprite jaguarheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "359": {
        "name": "Jaguar Fur",
        "classes": "animsprite jaguarfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "360": {
        "name": "Jaguar Vertebra",
        "classes": "animsprite jaguarverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "361": {
        "name": "Leopard Heart",
        "classes": "animsprite leopardheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "362": {
        "name": "Leopard Entrail",
        "classes": "animsprite babentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "363": {
        "name": "Leopard Vertebra",
        "classes": "animsprite leopardverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "364": {
        "name": "Lion Heart",
        "classes": "animsprite lionheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "365": {
        "name": "Lion Skull",
        "classes": "animsprite lionskull_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "366": {
        "name": "Lion Tooth",
        "classes": "animsprite lionteeth_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "367": {
        "name": "Lion Vertebra",
        "classes": "animsprite lionverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "368": {
        "name": "Mammoth Heart",
        "classes": "animsprite mammothheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "369": {
        "name": "Mammoth Rib",
        "classes": "animsprite mammothribs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "370": {
        "name": "Mammoth Tusk",
        "classes": "animsprite mammothtusk_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "371": {
        "name": "Mammoth Vertebra",
        "classes": "animsprite mammothverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "372": {
        "name": "Massive Scarab Heart",
        "classes": "animsprite scarabheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "373": {
        "name": "Massive Scarab Carapace",
        "classes": "animsprite scarabcara_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "374": {
        "name": "Poisonous Crawler Heart",
        "classes": "animsprite poisoncrawlerheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "375": {
        "name": "Poisonous Crawler Exoskeleton",
        "classes": "animsprite poisoncrawlerexo_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "376": {
        "name": "Poisonous Crawler Mandible",
        "classes": "animsprite poisoncrawlermand_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "377": {
        "name": "Polar Bear Heart",
        "classes": "animsprite polarbearheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "378": {
        "name": "Polar Bear Fur",
        "classes": "animsprite polarbearfurs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "379": {
        "name": "Puma Heart",
        "classes": "animsprite pumaheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "380": {
        "name": "Puma Rib",
        "classes": "animsprite pumaribs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "381": {
        "name": "Puma Vertebra",
        "classes": "animsprite pumaverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "382": {
        "name": "Puma Fur",
        "classes": "animsprite pumafur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "383": {
        "name": "Rhinoceros Heart",
        "classes": "animsprite rhinoheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "384": {
        "name": "Rhinoceros Rib",
        "classes": "animsprite rhinoribs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "385": {
        "name": "Rhinoceros Hide",
        "classes": "animsprite rhinohide_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "386": {
        "name": "Salamander Heart",
        "classes": "animsprite salaheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "387": {
        "name": "Salamander Entrail",
        "classes": "animsprite salaentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "388": {
        "name": "Salamander Fang",
        "classes": "animsprite salafangs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "389": {
        "name": "Saurian Heart",
        "classes": "animsprite saurheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "390": {
        "name": "Saurian Entrail",
        "classes": "animsprite saurentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "391": {
        "name": "Saurian Rib",
        "classes": "animsprite saurribs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "392": {
        "name": "Saurian Vertebra",
        "classes": "animsprite saurverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "393": {
        "name": "Scaled Charger Heart",
        "classes": "animsprite scaledcharheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "394": {
        "name": "Scaled Charger Scale",
        "classes": "animsprite scaledcharscale_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "395": {
        "name": "Scaled Charger Vertebra",
        "classes": "animsprite scaledcharverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "396": {
        "name": "Scritcher Heart",
        "classes": "animsprite scritcherheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "397": {
        "name": "Scritcher Carapace",
        "classes": "animsprite scritchercara_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "398": {
        "name": "Simien Wolf Heart",
        "classes": "animsprite simienwolfheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "399": {
        "name": "Simien Wolf Fur",
        "classes": "animsprite simienwolffur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "400": {
        "name": "Snow Leopard Heart",
        "classes": "animsprite snleopardheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "401": {
        "name": "Snow Leopard Fur",
        "classes": "animsprite snleopardfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "402": {
        "name": "Snow Leopard Vertebra",
        "classes": "animsprite snleopardverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "403": {
        "name": "Tiger Heart",
        "classes": "animsprite tigerheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "404": {
        "name": "Tiger Tooth",
        "classes": "animsprite tigerteeth_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "405": {
        "name": "Tiger Skull",
        "classes": "animsprite tigerskull_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "406": {
        "name": "White Tiger Heart",
        "classes": "animsprite snowtigerheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "407": {
        "name": "White Tiger Vertebra",
        "classes": "animsprite snowtigerverte_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "408": {
        "name": "Wild Dog Heart",
        "classes": "animsprite wilddogheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "409": {
        "name": "Wild Dog Bone",
        "classes": "animsprite wilddogbones_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "410": {
        "name": "Wild Dog Entrail",
        "classes": "animsprite wilddogentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "411": {
        "name": "Wild Dog Fur",
        "classes": "animsprite wilddogfur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "412": {
        "name": "Wolf Heart",
        "classes": "animsprite wolfheart_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "413": {
        "name": "Wolf Entrail",
        "classes": "animsprite wolfentrails_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "414": {
        "name": "Wolf Tooth",
        "classes": "animsprite wolfteeth_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "415": {
        "name": "Wolf Fur",
        "classes": "animsprite wolffur_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "416": {
        "name": "Herbs",
        "classes": "itemsprite herbs_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "417": {
        "name": "Minerals",
        "classes": "base crystals_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "420": {
        "name": "Wine",
        "classes": "base wine_48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "422": {
        "name": "Elven Thoroughbred",
        "classes": "itemsprite elvishhorse48",
        "e": "0|0|0|0|0|0|0|0|0|0|-20|0|-20|-20|0|0|0|0|0|0|0|0|+15|0|0|0|0|0|1|0|0|0|0"
    },
    "423": {
        "name": "Battlebred",
        "classes": "itemsprite humanwarhorse48",
        "e": "0|-5|-5|-5|-10|-5|-15|0|0|0|-20|-20|0|-20|0|0|0|+20|+20|+20|+20|0|0|0|0|0|0|0|1|0|0|0|0"
    },
    "424": {
        "name": "Dwarven Battle Mule",
        "classes": "itemsprite dwarvenwarmule48",
        "e": "0|0|0|0|0|+20|+20|0|0|0|0|-20|-20|-20|0|0|0|0|0|0|0|-5|-10|0|0|0|0|0|1|0|0|0|0"
    },
    "425": {
        "name": "War Wolf",
        "classes": "itemsprite warwolf48",
        "e": "0|0|0|0|0|+20|0|0|0|0|-20|-20|-20|0|0|0|0|0|0|0|0|0|+10|+20|0|0|0|0|1|0|0|0|0"
    },
    "426": {
        "name": "Soul-Forged Blade",
        "classes": "tournsprite tourneysword48",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0"
    },
    "427": {
        "name": "Armored Charger",
        "classes": "item3 s48 i427",
        "e": "0|0|-10|-10|0|-5|-5|0|0|0|0|0|0|0|0|0|0|+10|+10|+10|+10|+5|+5|-30|-5|0|0|0|1|0|0|0|0"
    },
    "428": {
        "name": "Demonheart Thoroughbred",
        "classes": "item3 s48 i428",
        "e": "0|0|0|0|0|0|0|0|0|0|-40|+10|-40|-40|0|0|0|+5|+5|+5|+5|+10|+25|0|0|0|0|0|1|0|0|0|0"
    },
    "429": {
        "name": "Demonheart Battlebred",
        "classes": "item3 s48 i429",
        "e": "0|0|0|0|0|0|-5|0|0|0|-40|-40|+10|-40|0|0|0|+30|+30|+30|+30|+10|+10|0|0|0|0|0|1|0|0|0|0"
    },
    "430": {
        "name": "Demonheart Battle Mule",
        "classes": "item3 s48 i430",
        "e": "0|+20|+20|+20|+20|+30|+30|+20|0|0|+10|-40|-40|-40|0|0|0|+5|+5|+5|+5|+5|0|0|0|0|0|0|1|0|0|0|0"
    },
    "431": {
        "name": "Demonheart War Wolf",
        "classes": "item3 s48 i431",
        "e": "0|0|0|0|0|+30|0|0|0|0|-40|-40|-40|+10|0|0|0|+5|+5|+5|+5|+10|+20|+30|0|0|0|0|1|0|0|0|0"
    },
    "432": {
        "name": "Flanged Mace",
        "classes": "item3 s48 i432",
        "e": "0|0|0|-10|-10|+5|+5|0|0|0|0|0|0|0|0|0|0|0|-5|-10|0|+10|0|0|0|0|1|0|0|0|0|0|0"
    },
    "433": {
        "name": "Goldstone Warhammer",
        "classes": "item3 s48 i433",
        "e": "0|0|0|0|-5|+10|+10|-5|0|0|+5|-5|-5|+5|+10|0|0|0|0|0|0|-5|+10|+10|0|0|1|0|0|0|0|0|0"
    },
    "434": {
        "name": "Morningstar",
        "classes": "item3 s48 i434",
        "e": "0|0|0|-5|-5|+5|+5|0|0|0|0|0|0|0|0|0|0|+15|+15|-10|-10|+8|0|0|0|0|1|0|0|0|0|0|0"
    },
    "435": {
        "name": "Spiked Flail",
        "classes": "item3 s48 i435",
        "e": "0|+10|+10|-15|-15|0|-15|+10|0|0|0|0|0|0|0|0|0|-10|-10|-10|+5|+35|0|0|0|0|1|0|0|0|0|0|0"
    },
    "436": {
        "name": "Titan Hammer",
        "classes": "item3 s48 i436",
        "e": "0|0|-5|0|-5|0|0|0|0|0|0|0|0|0|0|0|0|+10|+10|-10|-10|+20|-5|-10|0|0|1|0|0|0|0|0|0"
    },
    "437": {
        "name": "Yura Warmace",
        "classes": "item3 s48 i437",
        "e": "0|0|-5|0|-5|0|0|0|0|0|0|0|0|0|0|0|0|-10|-10|+10|+10|+20|-5|-10|0|0|1|0|0|0|0|0|0"
    },
    "438": {
        "name": "Decimation Mallet",
        "classes": "item3 s48 i438",
        "e": "0|-5|0|-5|-5|0|0|0|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+19|-5|0|0|0|1|0|0|0|0|0|0"
    },
    "439": {
        "name": "Ridersbane Pick",
        "classes": "item3 s48 i439",
        "e": "0|0|0|0|0|+10|0|+10|0|0|0|0|0|0|0|0|0|-10|+32|0|0|+3|0|0|0|0|1|0|0|0|0|0|0"
    },
    "440": {
        "name": "Possessed Falchion",
        "classes": "item3 s48 i440",
        "e": "0|0|0|0|0|0|0|0|-10|+30|0|0|0|0|0|+30|0|0|0|0|0|+5|0|0|0|0|1|0|0|0|0|0|+45"
    },
    "441": {
        "name": "Consecrated Scepter",
        "classes": "item3 s48 i441",
        "e": "0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|0|+30|0|0|0|0|0|+5|0|0|0|0|1|0|0|0|0|0|+45"
    },
    "442": {
        "name": "Jeweled Lightblade",
        "classes": "item3 s48 i442",
        "e": "0|0|0|0|+10|-5|-5|+10|0|0|-5|+5|+5|-5|+10|0|0|0|0|0|0|-5|+10|+10|0|0|1|0|0|0|0|0|0"
    },
    "443": {
        "name": "Twilight Greatsword",
        "classes": "item3 s48 i443",
        "e": "0|0|0|0|0|0|0|0|-25|+65|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|1|0|0|0|0|0|0"
    },
    "444": {
        "name": "Dawnbreaker Greatsword",
        "classes": "item3 s48 i444",
        "e": "0|0|0|0|0|0|0|0|+65|-25|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|1|0|0|0|0|0|0"
    },
    "445": {
        "name": "Viper Estoc",
        "classes": "item3 s48 i445",
        "e": "0|0|0|+25|0|0|0|0|0|0|0|0|0|0|+30|0|0|-4|-4|-4|+6|-4|+15|+10|0|0|1|0|0|0|0|0|0"
    },
    "446": {
        "name": "Deathspike Rapier",
        "classes": "item3 s48 i446",
        "e": "0|0|+10|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-10|-10|-10|-10|+10|+20|0|0|0|1|0|0|0|0|0|0"
    },
    "447": {
        "name": "Penumbral Falchion",
        "classes": "item3 s48 i447",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|-5|+50|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|1|0|0|0|0|0|+145"
    },
    "448": {
        "name": "Resplendent Scepter",
        "classes": "item3 s48 i448",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+50|-5|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|1|0|0|0|0|0|+145"
    },
    "449": {
        "name": "Greatsword of Dusk",
        "classes": "item3 s48 i449",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|0|+150|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+5|0|+70|0|1|0|0|0|0|0|+5"
    },
    "450": {
        "name": "Greatsword of Dawn",
        "classes": "item3 s48 i450",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+150|0|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|1|0|0|0|0|0|+5"
    },
    "451": {
        "name": "Serpent's Kiss",
        "classes": "item3 s48 i451",
        "e": "0|+10|+10|+10|+10|+10|+10|+10|0|0|0|0|0|0|+100|0|0|+10|+10|+10|+10|+5|+60|+30|+10|0|1|0|0|0|0|0|0"
    },
    "452": {
        "name": "Deathwish Flail",
        "classes": "item3 s48 i452",
        "e": "0|+50|+50|0|0|0|0|+50|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+170|0|0|+10|0|1|0|0|0|0|0|0"
    },
    "453": {
        "name": "Bulwark Battle Edge",
        "classes": "item3 s48 i453",
        "e": "0|+60|+60|+60|+60|+60|+60|+60|0|0|0|0|0|0|0|0|0|+80|+80|+80|+80|+5|0|0|+10|0|1|0|0|0|0|0|0"
    },
    "454": {
        "name": "Bane of Kerberos",
        "classes": "item3 s48 i454",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+50|+100|+75|+125|+200|0|0|0|0|1|0|0|0|0|0|+50"
    },
    "455": {
        "name": "Hailstorm Bow",
        "classes": "item3 s48 i455",
        "e": "0|+5|+5|-5|-5|0|+5|+5|0|0|0|0|0|0|0|0|0|+15|+15|-10|-10|+4|+2|0|0|0|0|1|0|0|0|0|+10"
    },
    "456": {
        "name": "Rockpiercer Recurve Bow",
        "classes": "item3 s48 i456",
        "e": "0|-5|+5|+5|+5|0|-5|+5|0|0|0|0|0|0|0|0|0|-10|-10|+15|+15|+4|+2|0|0|0|0|1|0|0|0|0|+10"
    },
    "457": {
        "name": "Siege Bow",
        "classes": "item3 s48 i457",
        "e": "0|+10|+10|-15|-15|0|-15|+10|0|-5|0|0|0|0|0|0|0|+5|-5|-5|-5|+35|-10|-5|0|0|0|1|0|0|0|0|0"
    },
    "458": {
        "name": "Gilded Longbow",
        "classes": "item3 s48 i458",
        "e": "0|0|0|0|+10|-5|-5|+10|0|0|-5|+5|+5|-5|+50|0|0|0|0|0|0|-5|+10|+10|0|0|0|1|0|0|0|0|0"
    },
    "459": {
        "name": "Ornate Shortbow",
        "classes": "item3 s48 i459",
        "e": "0|0|0|0|-5|+10|+10|-5|0|0|+5|-5|-5|+5|+50|0|0|0|0|0|0|-5|+10|+10|0|0|0|1|0|0|0|0|0"
    },
    "460": {
        "name": "Flatbow",
        "classes": "item3 s48 i460",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|-20|-20|0|+10|+10|+10|+10|0|0|0|0|0|0|1|0|0|0|0|-20"
    },
    "461": {
        "name": "Nightstalker Greatbow",
        "classes": "item3 s48 i461",
        "e": "0|0|0|0|0|0|0|0|-25|+65|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|0|1|0|0|0|0|0"
    },
    "462": {
        "name": "Dawnhunter Greatbow",
        "classes": "item3 s48 i462",
        "e": "0|0|0|0|0|0|0|0|+65|-25|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|0|1|0|0|0|0|0"
    },
    "463": {
        "name": "Wicked Bow",
        "classes": "item3 s48 i463",
        "e": "0|0|0|0|0|0|0|0|-10|+30|0|0|0|0|0|+30|0|0|0|0|0|+5|0|0|0|0|0|1|0|0|0|0|+45"
    },
    "464": {
        "name": "Celestial Bow",
        "classes": "item3 s48 i464",
        "e": "0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|0|+30|0|0|0|0|0|+5|0|0|0|0|0|1|0|0|0|0|+45"
    },
    "465": {
        "name": "Sniper's Bow",
        "classes": "item3 s48 i465",
        "e": "0|-10|+5|-10|-10|+5|-10|+5|0|-10|0|0|0|0|0|0|0|+85|-30|-30|-30|+5|0|0|0|0|0|1|0|0|0|0|0"
    },
    "466": {
        "name": "Reaper Longbow",
        "classes": "item3 s48 i466",
        "e": "0|0|-5|0|-5|0|0|+35|0|0|0|0|0|0|0|0|0|+10|+10|-10|-10|+20|-5|-10|0|0|0|1|0|0|0|0|0"
    },
    "467": {
        "name": "Hellfire Recurve",
        "classes": "item3 s48 i467",
        "e": "0|0|+35|0|-5|0|0|-5|0|0|0|0|0|0|0|0|0|-10|-10|+10|+10|+20|-5|-10|0|0|0|1|0|0|0|0|0"
    },
    "468": {
        "name": "Slayer's Shortbow",
        "classes": "item3 s48 i468",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-10|-10|-10|-10|+10|+20|0|0|0|0|1|0|0|0|0|0"
    },
    "469": {
        "name": "Viper Shortbow",
        "classes": "item3 s48 i469",
        "e": "0|0|0|+25|0|0|0|0|0|0|0|0|0|0|+30|0|0|-6|-6|-6|-6|-4|+15|+5|0|0|0|1|0|0|0|0|0"
    },
    "470": {
        "name": "Death's Reach",
        "classes": "item3 s48 i470",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|0|+50|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|0|1|0|0|0|0|+145"
    },
    "471": {
        "name": "Hand of the Heavens",
        "classes": "item3 s48 i471",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+50|0|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|0|1|0|0|0|0|+145"
    },
    "472": {
        "name": "Blackout Greatbow",
        "classes": "item3 s48 i472",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|-5|+150|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|0|1|0|0|0|0|+5"
    },
    "473": {
        "name": "Aurora Greatbow",
        "classes": "item3 s48 i473",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+150|-5|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|0|1|0|0|0|0|+5"
    },
    "474": {
        "name": "Cobrastrike Shortbow",
        "classes": "item3 s48 i474",
        "e": "0|+10|+10|+45|+10|+10|+10|+10|0|0|0|0|0|0|+100|0|0|+10|+10|+10|+10|+5|+60|+30|+10|0|0|1|0|0|0|0|0"
    },
    "475": {
        "name": "Heartpiercer",
        "classes": "item3 s48 i475",
        "e": "0|+50|+50|0|0|0|0|+50|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+170|0|0|+10|0|0|1|0|0|0|0|0"
    },
    "476": {
        "name": "Citadel Bow",
        "classes": "item3 s48 i476",
        "e": "0|+60|+60|+60|+60|+60|+60|+60|0|0|0|0|0|0|0|0|0|+80|+80|+80|+80|+5|0|0|+10|0|0|1|0|0|0|0|0"
    },
    "477": {
        "name": "Hunter of Kerberos",
        "classes": "item3 s48 i477",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+75|+50|+100|+125|+200|0|0|0|0|0|1|0|0|0|0|+50"
    },
    "478": {
        "name": "Becs De Corbin",
        "classes": "item3 s48 i478",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-12|+8|-12|+12|+8|0|0|0|1|0|0|0|0|0|0|0"
    },
    "479": {
        "name": "Planson",
        "classes": "item3 s48 i479",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-12|-12|+8|+12|+8|0|0|0|1|0|0|0|0|0|0|0"
    },
    "480": {
        "name": "Winged Spear",
        "classes": "item3 s48 i480",
        "e": "0|0|0|-12|-12|+8|+2|+8|0|0|0|0|0|0|0|0|0|+5|-5|+30|-20|+5|0|0|0|1|0|0|0|0|0|0|0"
    },
    "481": {
        "name": "Grim Spontoon",
        "classes": "item3 s48 i481",
        "e": "0|0|0|+15|+15|0|0|0|0|0|0|0|0|0|0|0|0|+6|-6|-6|+6|-6|+8|0|0|1|0|0|0|0|0|0|0"
    },
    "482": {
        "name": "Naginata",
        "classes": "item3 s48 i482",
        "e": "0|+5|+5|-6|-6|+16|+16|-6|0|0|0|0|0|0|0|0|0|0|0|0|0|+20|-5|-5|0|1|0|0|0|0|0|0|0"
    },
    "483": {
        "name": "Spear of the Asp",
        "classes": "item3 s48 i483",
        "e": "0|0|0|+25|0|0|0|0|0|0|0|0|0|0|+30|0|0|+15|-5|-5|0|-10|+15|0|0|1|0|0|0|0|0|0|0"
    },
    "484": {
        "name": "Wastelander Angon",
        "classes": "item3 s48 i484",
        "e": "0|-5|0|+2|+2|+10|-5|+10|0|0|0|0|0|0|+5|0|0|+5|+5|+5|+5|0|+6|-5|0|1|0|0|0|0|0|0|0"
    },
    "485": {
        "name": "Jeweled Labrys",
        "classes": "item3 s48 i485",
        "e": "0|0|0|0|+10|-5|-5|+10|0|0|-5|+5|+5|-5|+10|0|0|0|0|0|0|-5|+10|+10|0|1|0|0|0|0|0|0|0"
    },
    "486": {
        "name": "Opulent Glaive",
        "classes": "item3 s48 i486",
        "e": "0|0|0|0|-5|+10|+10|-5|0|0|+5|-5|-5|+5|+10|0|0|0|0|0|0|-5|+10|+10|0|1|0|0|0|0|0|0|0"
    },
    "487": {
        "name": "Moonbeam Poleaxe",
        "classes": "item3 s48 i487",
        "e": "0|0|0|0|0|0|0|0|-25|+65|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|1|0|0|0|0|0|0|0"
    },
    "488": {
        "name": "Sunburst Halberd",
        "classes": "item3 s48 i488",
        "e": "0|0|0|0|0|0|0|0|+65|-25|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|1|0|0|0|0|0|0|0"
    },
    "489": {
        "name": "Fiendish Lance",
        "classes": "item3 s48 i489",
        "e": "0|0|0|0|0|0|0|0|-10|+30|0|0|0|0|0|+30|0|0|0|0|0|+5|0|0|0|1|0|0|0|0|0|0|+45"
    },
    "490": {
        "name": "Radiant Bardiche",
        "classes": "item3 s48 i490",
        "e": "0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|0|+30|0|0|0|0|0|+5|0|0|0|1|0|0|0|0|0|0|+45"
    },
    "491": {
        "name": "Boreal Billhook",
        "classes": "item3 s48 i491",
        "e": "0|+45|0|-5|-5|0|0|0|0|-5|0|0|0|0|0|0|0|+10|+10|-10|-10|+25|-5|-10|0|1|0|0|0|0|0|0|0"
    },
    "492": {
        "name": "War Scythe",
        "classes": "item3 s48 i492",
        "e": "0|0|+45|-5|-5|0|0|0|0|-5|0|0|0|0|0|0|0|-10|-10|+10|+10|+25|-5|-10|0|1|0|0|0|0|0|0|0"
    },
    "493": {
        "name": "Death's-Head Lance",
        "classes": "item3 s48 i493",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|-5|+50|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|1|0|0|0|0|0|0|+145"
    },
    "494": {
        "name": "Divine Verdict",
        "classes": "item3 s48 i494",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+50|-5|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|1|0|0|0|0|0|0|+145"
    },
    "495": {
        "name": "Mooncrusher Poleaxe",
        "classes": "item3 s48 i495",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|0|+150|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|1|0|0|0|0|0|0|+5"
    },
    "496": {
        "name": "Helios Halberd",
        "classes": "item3 s48 i496",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+150|0|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|1|0|0|0|0|0|0|+5"
    },
    "497": {
        "name": "Venomstrike Spear",
        "classes": "item3 s48 i497",
        "e": "0|+10|+10|+10|+10|+10|+10|+10|0|0|0|0|0|0|+100|0|0|+10|+10|+10|+10|+5|+60|+30|+10|1|0|0|0|0|0|0|0"
    },
    "498": {
        "name": "Mancleaver",
        "classes": "item3 s48 i498",
        "e": "0|+50|+50|0|0|0|0|+50|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+170|0|0|+10|1|0|0|0|0|0|0|0"
    },
    "499": {
        "name": "Angon of the Aegis",
        "classes": "item3 s48 i499",
        "e": "0|+60|+60|+60|+60|+60|+60|+60|0|0|0|0|0|0|0|0|0|+80|+80|+80|+80|+5|0|0|+10|1|0|0|0|0|0|0|0"
    },
    "500": {
        "name": "Torment of Kerberos",
        "classes": "item3 s48 i500",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+50|+125|+100|+75|+200|0|0|0|1|0|0|0|0|0|0|+50"
    },
    "501": {
        "name": "Lionsmane Armour",
        "classes": "item3 s48 i501",
        "e": "0|0|+10|0|0|0|0|0|+10|0|0|0|0|0|+50|0|0|-10|-10|+10|+10|+5|0|0|0|0|0|0|0|1|0|0|0"
    },
    "502": {
        "name": "Reinforced Tunic",
        "classes": "item3 s48 i502",
        "e": "0|+5|-5|-5|-5|+5|-5|+5|0|0|0|0|0|0|0|0|0|+30|+30|+30|+30|-15|0|-10|0|0|0|0|0|1|0|0|0"
    },
    "503": {
        "name": "Warding Leather",
        "classes": "item3 s48 i503",
        "e": "0|0|0|0|0|0|0|0|0|+5|0|0|0|0|0|0|0|0|0|0|-6|0|0|0|+45|0|0|0|0|1|0|0|+45"
    },
    "504": {
        "name": "Ornate Tunic",
        "classes": "item3 s48 i504",
        "e": "0|0|0|0|+10|-5|-5|+10|0|0|-5|+5|+5|-5|+30|0|0|0|0|0|0|-5|+10|+10|0|0|0|0|0|1|0|0|0"
    },
    "505": {
        "name": "Snakeskin Tunic",
        "classes": "item3 s48 i505",
        "e": "0|0|0|+25|0|0|0|0|0|0|0|0|0|0|+15|0|0|+15|-5|-5|0|-10|+15|+5|0|0|0|0|0|1|0|0|0"
    },
    "506": {
        "name": "Ursine Leather",
        "classes": "item3 s48 i506",
        "e": "0|+35|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|-10|-10|-10|-10|+10|+20|0|0|0|0|0|0|1|0|0|0"
    },
    "507": {
        "name": "Peerless Leather",
        "classes": "item3 s48 i507",
        "e": "0|-10|-10|-10|+10|+5|+5|+5|0|0|0|0|0|0|+5|+5|0|+5|+5|+5|+5|0|0|0|0|0|0|0|0|1|0|0|+5"
    },
    "508": {
        "name": "Hellborn Tunic",
        "classes": "item3 s48 i508",
        "e": "0|0|0|0|0|0|0|0|-10|+30|0|0|0|0|0|+10|0|0|0|0|0|+5|0|0|0|0|0|0|0|1|0|0|+15"
    },
    "509": {
        "name": "Astral Leather",
        "classes": "item3 s48 i509",
        "e": "0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|0|+10|0|0|0|0|0|+5|0|0|0|0|0|0|0|1|0|0|+15"
    },
    "510": {
        "name": "Ornamented Leather",
        "classes": "item3 s48 i510",
        "e": "0|0|0|0|-5|+10|+10|-5|0|0|+5|-5|-5|+5|+30|0|0|0|0|0|0|-5|+10|+10|0|0|0|0|0|1|0|0|0"
    },
    "511": {
        "name": "Deathstalker Garb",
        "classes": "item3 s48 i511",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|-5|+50|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|0|0|0|1|0|0|+145"
    },
    "512": {
        "name": "Righteous Embrace",
        "classes": "item3 s48 i512",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+50|-5|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|0|0|0|1|0|0|+145"
    },
    "513": {
        "name": "Tunic of the Shrouded",
        "classes": "item3 s48 i513",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|0|+150|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|0|0|0|1|0|0|+5"
    },
    "514": {
        "name": "Tunic of Radiance",
        "classes": "item3 s48 i514",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+150|0|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|0|0|0|1|0|0|+5"
    },
    "515": {
        "name": "Cobra's Hood",
        "classes": "item3 s48 i515",
        "e": "0|+10|+10|+10|+10|+10|+10|+10|0|0|0|0|0|0|+100|0|0|+10|+10|+10|+10|+5|+60|+30|+10|0|0|0|0|1|0|0|0"
    },
    "516": {
        "name": "Sadist's Surcoat",
        "classes": "item3 s48 i516",
        "e": "0|+50|+50|0|0|0|0|+50|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+170|0|0|+10|0|0|0|0|1|0|0|0"
    },
    "517": {
        "name": "Redoubt Leather",
        "classes": "item3 s48 i517",
        "e": "0|+60|+60|+60|+60|+60|+60|+60|0|0|0|0|0|0|0|0|0|+80|+80|+80|+80|+5|0|0|+10|0|0|0|0|1|0|0|0"
    },
    "518": {
        "name": "Hide of Kerberos",
        "classes": "item3 s48 i518",
        "e": "0|+20|+20|+20|+20|+20|+20|+20|0|0|0|0|0|0|0|0|0|+125|+125|+125|+125|+30|0|0|0|0|0|0|0|1|0|0|+50"
    },
    "519": {
        "name": "Heirloom Brigadine",
        "classes": "item3 s48 i519",
        "e": "0|0|0|0|-5|+10|+10|-5|0|0|+5|-5|-5|+5|+30|0|0|0|0|0|0|-5|+10|+10|0|0|0|0|0|0|1|0|0"
    },
    "520": {
        "name": "Jewel Encrusted Doublet",
        "classes": "item3 s48 i520",
        "e": "0|0|0|0|+10|-5|-5|+10|0|0|-5|+5|+5|-5|+30|0|0|0|0|0|0|-5|+10|+10|0|0|0|0|0|0|1|0|0"
    },
    "521": {
        "name": "Peerless Lamellar",
        "classes": "item3 s48 i521",
        "e": "0|-10|-10|-10|+10|+5|+5|+5|0|0|0|0|0|0|+5|+5|0|+5|+5|+5|+5|0|0|0|0|0|0|0|0|0|1|0|+5"
    },
    "522": {
        "name": "Serpentscale Doublet",
        "classes": "item3 s48 i522",
        "e": "0|0|0|+25|0|0|0|0|0|0|0|0|0|0|+15|0|0|+15|-5|-5|0|-10|+15|+5|0|0|0|0|0|0|1|0|0"
    },
    "523": {
        "name": "Darkheart Ringmail",
        "classes": "item3 s48 i523",
        "e": "0|0|0|0|0|0|0|0|-10|+30|0|0|0|0|0|+10|0|0|0|0|0|+5|0|0|0|0|0|0|0|0|1|0|+15"
    },
    "524": {
        "name": "Exalted Ringmail",
        "classes": "item3 s48 i524",
        "e": "0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|0|+10|0|0|0|0|0|+5|0|0|0|0|0|0|0|0|1|0|+15"
    },
    "525": {
        "name": "Chainmail of Warding",
        "classes": "item3 s48 i525",
        "e": "0|0|0|0|0|0|0|0|0|+5|0|0|0|0|0|0|0|-6|0|0|0|0|0|0|+45|0|0|0|0|0|1|0|+45"
    },
    "526": {
        "name": "Siege Brigandine",
        "classes": "item3 s48 i526",
        "e": "0|+10|+10|-15|-15|0|-15|+10|0|0|0|0|0|0|0|0|0|+25|-5|-5|-5|+20|-5|-5|0|0|0|0|0|0|1|0|0"
    },
    "527": {
        "name": "Daystrider Chainmail",
        "classes": "item3 s48 i527",
        "e": "0|0|0|0|0|0|0|0|+65|-25|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|0|0|0|0|1|0|0"
    },
    "528": {
        "name": "Moonrise Chainmail",
        "classes": "item3 s48 i528",
        "e": "0|0|0|0|0|0|0|0|-25|+65|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|0|0|0|0|1|0|0"
    },
    "529": {
        "name": "Demonscale Mail",
        "classes": "item3 s48 i529",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|-5|+50|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|0|0|0|0|1|0|+145"
    },
    "530": {
        "name": "Sanctified Ringmail",
        "classes": "item3 s48 i530",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+50|-5|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|0|0|+15|0|0|0|0|0|1|0|+145"
    },
    "531": {
        "name": "Chainmail of Nightfall",
        "classes": "item3 s48 i531",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|0|+150|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|0|0|0|0|1|0|+5"
    },
    "532": {
        "name": "Scintillating Chainmail",
        "classes": "item3 s48 i532",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+150|0|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|+2|0|+70|0|0|0|0|0|1|0|+5"
    },
    "533": {
        "name": "Ouroboros Doublet",
        "classes": "item3 s48 i533",
        "e": "0|+10|+10|0|+10|+10|+10|+10|0|0|0|0|0|0|+100|0|0|+10|+10|+10|+10|+5|+60|+30|+10|0|0|0|0|0|1|0|0"
    },
    "534": {
        "name": "Deathgiver's Caress",
        "classes": "item3 s48 i534",
        "e": "0|+50|+50|0|0|0|0|+50|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+170|0|0|+10|0|0|0|0|0|1|0|0"
    },
    "535": {
        "name": "Mail of Impunity",
        "classes": "item3 s48 i535",
        "e": "0|+60|+60|+60|+60|+60|+60|+60|0|0|0|0|0|0|0|0|0|+80|+80|+80|+80|+5|0|0|+10|0|0|0|0|0|1|0|0"
    },
    "536": {
        "name": "Scale of Kerberos",
        "classes": "item3 s48 i536",
        "e": "0|+10|+10|+10|+10|+10|+10|+10|0|0|0|0|0|0|0|0|0|+150|+150|+150|+150|+30|0|0|+10|0|0|0|0|0|1|0|+50"
    },
    "537": {
        "name": "Adorned Platemail",
        "classes": "item3 s48 i537",
        "e": "0|0|0|0|-5|+10|+10|-5|0|0|+5|-5|-5|+5|+30|0|0|0|0|0|0|-5|+10|+10|0|0|0|0|0|0|0|1|0"
    },
    "538": {
        "name": "Trophy Platemail",
        "classes": "item3 s48 i538",
        "e": "0|0|0|0|+10|-5|-5|+10|0|0|-5|+5|+5|-5|+30|0|0|0|0|0|0|-5|+10|+10|0|0|0|0|0|0|0|1|0"
    },
    "539": {
        "name": "Peerless Platemail",
        "classes": "item3 s48 i539",
        "e": "0|-10|-10|-10|+10|+5|+5|+5|0|0|0|0|0|0|+5|+5|0|+5|+5|+5|+5|0|0|0|0|0|0|0|0|0|0|1|+5"
    },
    "540": {
        "name": "Deathadder Platemail",
        "classes": "item3 s48 i540",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|+15|0|0|+15|-5|-5|0|-10|+15|+5|0|0|0|0|0|0|0|1|0"
    },
    "541": {
        "name": "Platemail of the Void",
        "classes": "item3 s48 i541",
        "e": "0|0|0|0|0|0|0|0|-10|+30|0|0|0|0|0|+10|0|0|0|0|0|+5|0|0|0|0|0|0|0|0|0|1|+15"
    },
    "542": {
        "name": "Elysian Platemail",
        "classes": "item3 s48 i542",
        "e": "0|0|0|0|0|0|0|0|+30|-10|0|0|0|0|0|+10|0|0|0|0|0|+5|0|0|0|0|0|0|0|0|0|1|+15"
    },
    "543": {
        "name": "Lunar Platemail",
        "classes": "item3 s48 i543",
        "e": "0|0|0|0|0|0|0|0|-25|+65|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|0|0|0|0|0|1|0"
    },
    "544": {
        "name": "Solar Platemail",
        "classes": "item3 s48 i544",
        "e": "0|0|0|0|0|0|0|0|+65|-25|0|0|0|0|0|0|0|0|0|0|0|0|-10|0|+25|0|0|0|0|0|0|1|0"
    },
    "545": {
        "name": "Warding Platemail",
        "classes": "item3 s48 i545",
        "e": "0|0|0|0|0|0|0|0|0|+5|0|0|0|0|0|0|0|0|0|-6|0|0|0|0|+45|0|0|0|0|0|0|1|+45"
    },
    "546": {
        "name": "Titansbane Platemail",
        "classes": "item3 s48 i546",
        "e": "0|+30|+5|-10|-5|0|+5|-5|0|0|0|0|0|0|0|+65|0|-10|-10|-10|-10|+50|-10|-5|0|0|0|0|0|0|0|1|-5"
    },
    "547": {
        "name": "Pandemonium Plate",
        "classes": "item3 s48 i547",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|-5|+50|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|+2|0|+15|0|0|0|0|0|0|1|+145"
    },
    "548": {
        "name": "Aegis of Virtue",
        "classes": "item3 s48 i548",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+50|-5|0|0|0|0|0|+130|0|+5|+5|+5|+5|+25|0|0|+15|0|0|0|0|0|0|1|+145"
    },
    "549": {
        "name": "Gloom Ward Full Plate",
        "classes": "item3 s48 i549",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|0|+150|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|0|0|+70|0|0|0|0|0|0|1|+5"
    },
    "550": {
        "name": "Brightwalker Full Plate",
        "classes": "item3 s48 i550",
        "e": "0|+5|+5|+5|+5|+5|+5|+5|+150|0|0|0|0|0|0|+5|0|+5|+5|+5|+5|+10|0|0|+70|0|0|0|0|0|0|1|+5"
    },
    "551": {
        "name": "Basilisk's Gaze",
        "classes": "item3 s48 i551",
        "e": "0|+10|+10|+10|+10|+10|+10|+10|0|0|0|0|0|0|+100|0|0|+10|+10|+10|+10|+5|+60|+30|+10|0|0|0|0|0|0|1|0"
    },
    "552": {
        "name": "Breastplate of Bedlam",
        "classes": "item3 s48 i552",
        "e": "0|+50|+50|0|0|0|0|+50|0|0|0|0|0|0|0|0|0|+5|+5|+5|+5|+170|0|0|+10|0|0|0|0|0|0|1|0"
    },
    "553": {
        "name": "Impenetrable Platemail",
        "classes": "item3 s48 i553",
        "e": "0|+60|+60|+60|+60|+60|+60|+60|0|0|0|0|0|0|0|0|0|+80|+80|+80|+80|+5|0|0|+10|0|0|0|0|0|0|1|0"
    },
    "554": {
        "name": "Plate of Kerberos",
        "classes": "item3 s48 i554",
        "e": "0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|0|+175|+175|+175|+175|+150|0|0|0|0|0|0|0|0|0|1|+50"
    }
};
dt.cRecipes={
    "33": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "5",
                "i=4|1"
            ],
            [
                "5",
                "c=186"
            ],
            [
                "10",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "34": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "3",
                "c=417"
            ]
        ]
    },
    "35": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "3",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "36": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "5",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "37": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "3",
                "c=186"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "40": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=411"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "41": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=395"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "42": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=382"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "43": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=409"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "44": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=376"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "45": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "5",
                "i=1|1"
            ],
            [
                "5",
                "i=4|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "46": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=232"
            ],
            [
                "25",
                "i=2|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "47": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "5",
                "i=4|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "48": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=336"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "49": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "5",
                "i=1|1"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=193"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "50": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "51": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=359"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "52": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=409"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "53": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=314"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "54": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=207"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "55": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=414"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "56": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "5",
                "i=4|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "57": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=346"
            ]
        ]
    },
    "58": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=193"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "59": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=415"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "60": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=409"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "61": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=311"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "62": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=341"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "63": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=212"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "64": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "65": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "5",
                "i=4|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "67": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=334"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "68": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=397"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "69": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=357"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "70": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=232"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "25",
                "i=2|1"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "71": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=226"
            ],
            [
                "50",
                "i=2|1"
            ],
            [
                "10",
                "i=2|2"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "72": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=314"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "73": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=320"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "74": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=411"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "75": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=341"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "76": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "3",
                "c=186"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "77": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=304"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "78": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=362"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "79": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "80": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=415"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "81": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=370"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "82": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=382"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "83": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=320"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "84": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=395"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "85": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=226"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "86": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=311"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "87": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=224"
            ],
            [
                "1",
                "c=225"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "88": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=1|1"
            ],
            [
                "5",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "2",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "89": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=341"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "91": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=334"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "92": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=359"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "93": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "5",
                "i=1|1"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=246"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "94": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=382"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "95": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=247"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "96": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=357"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "97": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=246"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "98": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=311"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "99": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=232"
            ],
            [
                "25",
                "i=2|1"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "100": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "5",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=336"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "101": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=388"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "102": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=341"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "103": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=308"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "104": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=411"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "105": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "5",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "106": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=411"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "107": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=304"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "108": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "109": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=320"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "110": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=207"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "111": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=415"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "112": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=395"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "113": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=346"
            ]
        ]
    },
    "114": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=212"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "115": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=246"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "116": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "120": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "50",
                "i=2|1"
            ],
            [
                "1",
                "c=338"
            ],
            [
                "1",
                "c=375"
            ],
            [
                "1",
                "c=394"
            ]
        ]
    },
    "121": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=249"
            ]
        ]
    },
    "122": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=251"
            ]
        ]
    },
    "123": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=250"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "124": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=399"
            ]
        ]
    },
    "125": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=394"
            ]
        ]
    },
    "126": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=401"
            ]
        ]
    },
    "127": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=250"
            ]
        ]
    },
    "128": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=186"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "129": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=338"
            ]
        ]
    },
    "130": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=354"
            ]
        ]
    },
    "131": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=399"
            ]
        ]
    },
    "132": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=318"
            ]
        ]
    },
    "133": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=248"
            ]
        ]
    },
    "134": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=343"
            ]
        ]
    },
    "135": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "20",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ]
        ]
    },
    "136": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=210"
            ]
        ]
    },
    "137": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "50",
                "i=2|1"
            ]
        ]
    },
    "138": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=375"
            ]
        ]
    },
    "139": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "141": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=249"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "142": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=251"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "143": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "3",
                "c=417"
            ]
        ]
    },
    "144": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "20",
                "i=1|3"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "3",
                "c=417"
            ]
        ]
    },
    "145": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "146": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=307"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "147": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "148": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=338"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "149": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=375"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "150": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=354"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "151": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "152": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "50",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=250"
            ],
            [
                "1",
                "c=251"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "153": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=248"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "154": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=394"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "155": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "156": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=212"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "157": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=193"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "158": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=186"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "159": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=394"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "161": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "50",
                "i=2|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "162": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "20",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "163": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=251"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "164": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=193"
            ],
            [
                "1",
                "c=212"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "165": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=394"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "166": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=378"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "167": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "40",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "168": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=338"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "169": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=354"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "170": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "171": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=207"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "172": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=343"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "173": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=210"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "174": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=212"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "175": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "30",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=336"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "176": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "50",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "177": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=248"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "178": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=375"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "179": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=186"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "422": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "5",
                "c=416"
            ]
        ]
    },
    "423": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "5",
                "c=416"
            ]
        ]
    },
    "424": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "5",
                "c=417"
            ]
        ]
    },
    "425": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "5",
                "c=186"
            ]
        ]
    },
    "427": {
        "recipe": [
            [
                "1",
                "i=3|1"
            ],
            [
                "25",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=233"
            ],
            [
                "1",
                "c=416"
            ],
            [
                "3",
                "c=417"
            ]
        ]
    },
    "428": {
        "recipe": [
            [
                "40",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "5",
                "c=416"
            ],
            [
                "1",
                "c=422"
            ]
        ]
    },
    "429": {
        "recipe": [
            [
                "40",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "5",
                "c=416"
            ],
            [
                "1",
                "c=423"
            ]
        ]
    },
    "430": {
        "recipe": [
            [
                "40",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "5",
                "c=417"
            ],
            [
                "1",
                "c=424"
            ]
        ]
    },
    "431": {
        "recipe": [
            [
                "40",
                "i=4|1"
            ],
            [
                "5",
                "c=186"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=425"
            ]
        ]
    },
    "432": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=202"
            ],
            [
                "1",
                "c=312"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "433": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=379"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "434": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=344"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "435": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=350"
            ],
            [
                "3",
                "c=351"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "436": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=204"
            ],
            [
                "1",
                "c=326"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "437": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=205"
            ],
            [
                "1",
                "c=373"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "438": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=391"
            ],
            [
                "1",
                "c=392"
            ],
            [
                "5",
                "c=417"
            ]
        ]
    },
    "439": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=238"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "440": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "441": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=200"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "442": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=323"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "443": {
        "recipe": [
            [
                "2",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=203"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "444": {
        "recipe": [
            [
                "2",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=252"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "445": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=324"
            ],
            [
                "1",
                "c=349"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "446": {
        "recipe": [
            [
                "1",
                "i=3|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=313"
            ],
            [
                "1",
                "c=404"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "447": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=271"
            ],
            [
                "3",
                "c=275"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=440"
            ]
        ]
    },
    "448": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "3",
                "c=277"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=441"
            ]
        ]
    },
    "449": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=271"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=275"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=443"
            ]
        ]
    },
    "450": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=273"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=277"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=444"
            ]
        ]
    },
    "451": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "2",
                "c=279"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=445"
            ]
        ]
    },
    "452": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "2",
                "c=281"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=292"
            ],
            [
                "1",
                "c=435"
            ]
        ]
    },
    "453": {
        "recipe": [
            [
                "1",
                "c=57"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "2",
                "c=283"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=292"
            ]
        ]
    },
    "454": {
        "recipe": [
            [
                "1",
                "c=63"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=292"
            ]
        ]
    },
    "455": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "10",
                "i=2|1"
            ],
            [
                "1",
                "c=355"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "456": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=241"
            ],
            [
                "10",
                "i=2|1"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "457": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=329"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "458": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=352"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "459": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=358"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "460": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=298"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "461": {
        "recipe": [
            [
                "2",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=312"
            ],
            [
                "2",
                "c=416"
            ]
        ]
    },
    "462": {
        "recipe": [
            [
                "2",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=403"
            ],
            [
                "2",
                "c=416"
            ]
        ]
    },
    "463": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "464": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=200"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "465": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=383"
            ],
            [
                "1",
                "c=387"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "466": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=335"
            ],
            [
                "2",
                "c=416"
            ]
        ]
    },
    "467": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=332"
            ],
            [
                "1",
                "c=386"
            ],
            [
                "2",
                "c=416"
            ]
        ]
    },
    "468": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=342"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "469": {
        "recipe": [
            [
                "1",
                "i=3|4"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=322"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "470": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=271"
            ],
            [
                "3",
                "c=275"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=463"
            ]
        ]
    },
    "471": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "3",
                "c=277"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=464"
            ]
        ]
    },
    "472": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=271"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=275"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=461"
            ]
        ]
    },
    "473": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=273"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=277"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=462"
            ]
        ]
    },
    "474": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "2",
                "c=279"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=469"
            ]
        ]
    },
    "475": {
        "recipe": [
            [
                "1",
                "c=76"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "2",
                "c=281"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=290"
            ]
        ]
    },
    "476": {
        "recipe": [
            [
                "1",
                "c=87"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "2",
                "c=283"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=290"
            ]
        ]
    },
    "477": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=290"
            ],
            [
                "1",
                "c=466"
            ]
        ]
    },
    "478": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=236"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "479": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=205"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "480": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "4",
                "c=340"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "481": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=299"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "482": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=405"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "483": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "2",
                "c=303"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "484": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=204"
            ],
            [
                "1",
                "c=337"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "485": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=361"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "486": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=347"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "487": {
        "recipe": [
            [
                "2",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=400"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "488": {
        "recipe": [
            [
                "2",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=364"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "489": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "490": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=200"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "491": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=237"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "492": {
        "recipe": [
            [
                "1",
                "i=3|5"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=367"
            ],
            [
                "2",
                "c=417"
            ]
        ]
    },
    "493": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=271"
            ],
            [
                "3",
                "c=275"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=489"
            ]
        ]
    },
    "494": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "3",
                "c=277"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=490"
            ]
        ]
    },
    "495": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=271"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=275"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=487"
            ]
        ]
    },
    "496": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=273"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=277"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=488"
            ]
        ]
    },
    "497": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "2",
                "c=279"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=483"
            ]
        ]
    },
    "498": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "2",
                "c=281"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=492"
            ]
        ]
    },
    "499": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "2",
                "c=283"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=293"
            ],
            [
                "1",
                "c=484"
            ]
        ]
    },
    "500": {
        "recipe": [
            [
                "1",
                "c=114"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=293"
            ]
        ]
    },
    "501": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=364"
            ],
            [
                "1",
                "c=366"
            ]
        ]
    },
    "502": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "5",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=234"
            ]
        ]
    },
    "503": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=206"
            ],
            [
                "100",
                "i=2|1"
            ]
        ]
    },
    "504": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=323"
            ]
        ]
    },
    "505": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=348"
            ],
            [
                "1",
                "c=416"
            ]
        ]
    },
    "506": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=377"
            ]
        ]
    },
    "507": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "5",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=211"
            ]
        ]
    },
    "508": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=267"
            ]
        ]
    },
    "509": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=200"
            ]
        ]
    },
    "510": {
        "recipe": [
            [
                "1",
                "i=3|8"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=186"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=352"
            ]
        ]
    },
    "511": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=271"
            ],
            [
                "3",
                "c=275"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=508"
            ]
        ]
    },
    "512": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "3",
                "c=277"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=509"
            ]
        ]
    },
    "513": {
        "recipe": [
            [
                "1",
                "c=132"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=271"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=275"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "514": {
        "recipe": [
            [
                "1",
                "c=137"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=273"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=277"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "515": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "2",
                "c=279"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=505"
            ]
        ]
    },
    "516": {
        "recipe": [
            [
                "1",
                "c=139"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "2",
                "c=281"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "517": {
        "recipe": [
            [
                "1",
                "c=128"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "2",
                "c=283"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "518": {
        "recipe": [
            [
                "1",
                "c=123"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "519": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=312"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "520": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=301"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "521": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=211"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "522": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=302"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "523": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "524": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=200"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "525": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=206"
            ],
            [
                "100",
                "i=2|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "526": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=371"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "527": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=400"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "528": {
        "recipe": [
            [
                "1",
                "i=3|9"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=317"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "529": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=271"
            ],
            [
                "3",
                "c=275"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=523"
            ]
        ]
    },
    "530": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "3",
                "c=277"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=524"
            ]
        ]
    },
    "531": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=271"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=275"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=528"
            ]
        ]
    },
    "532": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=273"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=277"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=527"
            ]
        ]
    },
    "533": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "2",
                "c=279"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=522"
            ]
        ]
    },
    "534": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "2",
                "c=281"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=526"
            ]
        ]
    },
    "535": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "2",
                "c=283"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=521"
            ]
        ]
    },
    "536": {
        "recipe": [
            [
                "1",
                "c=156"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    },
    "537": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=305"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "538": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "20",
                "i=4|1"
            ],
            [
                "1",
                "c=208"
            ],
            [
                "1",
                "c=374"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "539": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "15",
                "i=1|3"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=211"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "540": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=302"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "541": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=267"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "542": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=200"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "543": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=398"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "544": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=364"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "545": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=206"
            ],
            [
                "100",
                "i=2|1"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "546": {
        "recipe": [
            [
                "1",
                "i=3|10"
            ],
            [
                "10",
                "i=4|1"
            ],
            [
                "1",
                "c=328"
            ],
            [
                "1",
                "c=369"
            ],
            [
                "1",
                "c=417"
            ]
        ]
    },
    "547": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=271"
            ],
            [
                "3",
                "c=275"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=541"
            ]
        ]
    },
    "548": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=273"
            ],
            [
                "3",
                "c=277"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=542"
            ]
        ]
    },
    "549": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=271"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=275"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=543"
            ]
        ]
    },
    "550": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "3",
                "c=273"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=277"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=544"
            ]
        ]
    },
    "551": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "2",
                "c=279"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=540"
            ]
        ]
    },
    "552": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "2",
                "c=281"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=283"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=546"
            ]
        ]
    },
    "553": {
        "recipe": [
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=279"
            ],
            [
                "1",
                "c=281"
            ],
            [
                "2",
                "c=283"
            ],
            [
                "1",
                "c=284"
            ],
            [
                "1",
                "c=291"
            ],
            [
                "1",
                "c=539"
            ]
        ]
    },
    "554": {
        "recipe": [
            [
                "1",
                "c=174"
            ],
            [
                "100",
                "i=4|1"
            ],
            [
                "1",
                "c=272"
            ],
            [
                "1",
                "c=274"
            ],
            [
                "1",
                "c=276"
            ],
            [
                "1",
                "c=278"
            ],
            [
                "1",
                "c=280"
            ],
            [
                "1",
                "c=282"
            ],
            [
                "1",
                "c=291"
            ]
        ]
    }
};
dt.cEffects={
    "33": {
        "effects": [
            [
                "+25",
                "Move Speed"
            ]
        ]
    },
    "34": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "-5",
                "in Arctic"
            ],
            [
                "-5",
                "in Desert"
            ],
            [
                "-5",
                "in Jungle"
            ]
        ]
    },
    "35": {
        "effects": [
            [
                "-8",
                "Attack"
            ],
            [
                "+24",
                "Cav Def"
            ],
            [
                "+24",
                "Spear Def"
            ],
            [
                "+24",
                "Bow Def"
            ],
            [
                "+24",
                "Sword Def"
            ]
        ]
    },
    "36": {
        "effects": [
            [
                "+25",
                "Move Speed"
            ]
        ]
    },
    "37": {
        "effects": [
            [
                "+5",
                "Magic Res"
            ]
        ]
    },
    "40": {
        "effects": [
            [
                "+60",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "41": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "42": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "43": {
        "effects": [
            [
                "+30",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "44": {
        "effects": [
            [
                "-12",
                "Attack"
            ],
            [
                "-14",
                "Cav Def"
            ],
            [
                "-14",
                "Spear Def"
            ],
            [
                "-14",
                "Bow Def"
            ],
            [
                "-14",
                "Sword Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+40",
                "on Mountains"
            ],
            [
                "+50",
                "in Forests"
            ],
            [
                "-8",
                "for Dwarves"
            ],
            [
                "+10",
                "for Orcs"
            ]
        ]
    },
    "45": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "-4",
                "on Hills"
            ],
            [
                "-2",
                "on Plains"
            ],
            [
                "-4",
                "in Forests"
            ],
            [
                "+6",
                "for Dwarves"
            ]
        ]
    },
    "46": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+120",
                "vs Undead"
            ]
        ]
    },
    "47": {
        "effects": [
            [
                "-2",
                "Attack"
            ],
            [
                "-4",
                "on Hills"
            ],
            [
                "-2",
                "on Plains"
            ],
            [
                "-2",
                "on Mountains"
            ],
            [
                "+6",
                "for Elves"
            ]
        ]
    },
    "48": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "-12",
                "Cav Def"
            ],
            [
                "+90",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "-48",
                "Sword Def"
            ]
        ]
    },
    "49": {
        "effects": [
            [
                "+30",
                "Attack"
            ],
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+30",
                "for Dwarves"
            ],
            [
                "-8",
                "for Elves"
            ],
            [
                "-4",
                "for Humans"
            ],
            [
                "+10",
                "for Orcs"
            ]
        ]
    },
    "50": {
        "effects": [
            [
                "-16",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ]
        ]
    },
    "51": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ]
        ]
    },
    "52": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "+30",
                "in Jungle"
            ]
        ]
    },
    "53": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ]
        ]
    },
    "54": {
        "effects": [
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Sword Def"
            ],
            [
                "+125",
                "Magic Res"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    },
    "55": {
        "effects": [
            [
                "+80",
                "Attack"
            ],
            [
                "-40",
                "Cav Def"
            ],
            [
                "-24",
                "Spear Def"
            ],
            [
                "-28",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "-24",
                "for Dwarves"
            ],
            [
                "-32",
                "for Elves"
            ],
            [
                "-24",
                "for Humans"
            ],
            [
                "+60",
                "for Orcs"
            ]
        ]
    },
    "56": {
        "effects": [
            [
                "-2",
                "Cav Def"
            ],
            [
                "-2",
                "Spear Def"
            ],
            [
                "-2",
                "Bow Def"
            ],
            [
                "-2",
                "Sword Def"
            ],
            [
                "-4",
                "on Plains"
            ],
            [
                "-2",
                "on Mountains"
            ],
            [
                "-2",
                "in Forests"
            ],
            [
                "+6",
                "for Orcs"
            ]
        ]
    },
    "57": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ]
        ]
    },
    "58": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ],
            [
                "+140",
                "vs Monsterous"
            ]
        ]
    },
    "59": {
        "effects": [
            [
                "+80",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "60": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "+30",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "61": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "62": {
        "effects": [
            [
                "+20",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ]
        ]
    },
    "63": {
        "effects": [
            [
                "+150",
                "Attack"
            ],
            [
                "+50",
                "Cav Def"
            ],
            [
                "+25",
                "Spear Def"
            ],
            [
                "+75",
                "Sword Def"
            ]
        ]
    },
    "64": {
        "effects": [
            [
                "+40",
                "Attack"
            ],
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "65": {
        "effects": [
            [
                "-2",
                "Attack"
            ],
            [
                "-2",
                "on Hills"
            ],
            [
                "-4",
                "on Mountains"
            ],
            [
                "-2",
                "in Forests"
            ],
            [
                "+6",
                "for Humans"
            ]
        ]
    },
    "67": {
        "effects": [
            [
                "+30",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "68": {
        "effects": [
            [
                "+40",
                "Cav Def"
            ],
            [
                "+40",
                "Spear Def"
            ],
            [
                "+40",
                "Bow Def"
            ],
            [
                "+40",
                "Sword Def"
            ],
            [
                "-32",
                "in Arctic"
            ],
            [
                "-32",
                "in Desert"
            ],
            [
                "-32",
                "in Jungle"
            ]
        ]
    },
    "69": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "+30",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "70": {
        "effects": [
            [
                "-8",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-16",
                "in Day"
            ],
            [
                "-12",
                "vs Monsterous"
            ],
            [
                "+90",
                "vs Undead"
            ]
        ]
    },
    "71": {
        "effects": [
            [
                "-24",
                "Attack"
            ],
            [
                "-12",
                "Cav Def"
            ],
            [
                "-12",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "-12",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "-36",
                "in Arctic"
            ],
            [
                "-36",
                "in Desert"
            ],
            [
                "-36",
                "in Jungle"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+120",
                "for Elves"
            ]
        ]
    },
    "72": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+135",
                "vs Monsterous"
            ]
        ]
    },
    "73": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ]
        ]
    },
    "74": {
        "effects": [
            [
                "+60",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "75": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+120",
                "vs Animals"
            ]
        ]
    },
    "76": {
        "effects": [
            [
                "+75",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "-20",
                "Move Speed"
            ],
            [
                "-32",
                "in Arctic"
            ],
            [
                "-32",
                "in Desert"
            ],
            [
                "-32",
                "in Jungle"
            ]
        ]
    },
    "77": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "+30",
                "in Jungle"
            ]
        ]
    },
    "78": {
        "effects": [
            [
                "-8",
                "Attack"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "+30",
                "in Jungle"
            ]
        ]
    },
    "79": {
        "effects": [
            [
                "-8",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+3",
                "for Elves"
            ]
        ]
    },
    "80": {
        "effects": [
            [
                "+80",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "81": {
        "effects": [
            [
                "+30",
                "Attack"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+30",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "82": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "83": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ]
        ]
    },
    "84": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "85": {
        "effects": [
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "+20",
                "on Hills"
            ],
            [
                "-12",
                "on Plains"
            ],
            [
                "+20",
                "in Forests"
            ],
            [
                "-36",
                "vs Monsterous"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "86": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "87": {
        "effects": [
            [
                "+25",
                "Cav Def"
            ],
            [
                "+25",
                "Spear Def"
            ],
            [
                "+25",
                "Bow Def"
            ],
            [
                "+25",
                "Sword Def"
            ],
            [
                "-32",
                "in Arctic"
            ],
            [
                "-32",
                "in Desert"
            ],
            [
                "-32",
                "in Jungle"
            ],
            [
                "+15",
                "for Elves"
            ]
        ]
    },
    "88": {
        "effects": [
            [
                "+6",
                "Attack"
            ],
            [
                "-15",
                "Move Speed"
            ]
        ]
    },
    "89": {
        "effects": [
            [
                "+20",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ]
        ]
    },
    "91": {
        "effects": [
            [
                "+30",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "92": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ]
        ]
    },
    "93": {
        "effects": [
            [
                "-16",
                "Attack"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+120",
                "vs Animals"
            ]
        ]
    },
    "94": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "95": {
        "effects": [
            [
                "+20",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ],
            [
                "+40",
                "in Desert"
            ],
            [
                "-16",
                "in Jungle"
            ]
        ]
    },
    "96": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "+30",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "97": {
        "effects": [
            [
                "-16",
                "Attack"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+150",
                "vs Monsterous"
            ]
        ]
    },
    "98": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "99": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+60",
                "vs Undead"
            ]
        ]
    },
    "100": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-8",
                "Cav Def"
            ],
            [
                "+60",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-32",
                "Sword Def"
            ]
        ]
    },
    "101": {
        "effects": [
            [
                "-24",
                "Cav Def"
            ],
            [
                "+40",
                "Spear Def"
            ],
            [
                "+20",
                "Bow Def"
            ],
            [
                "-24",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "+40",
                "vs Animals"
            ]
        ]
    },
    "102": {
        "effects": [
            [
                "+20",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ]
        ]
    },
    "103": {
        "effects": [
            [
                "-8",
                "Attack"
            ],
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "+20",
                "in Arctic"
            ],
            [
                "-4",
                "in Desert"
            ],
            [
                "-4",
                "in Jungle"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "104": {
        "effects": [
            [
                "+60",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "105": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "+40",
                "Cav Def"
            ],
            [
                "+40",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+10",
                "for Orcs"
            ],
            [
                "+60",
                "vs Monsterous"
            ]
        ]
    },
    "106": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-12",
                "Cav Def"
            ],
            [
                "-12",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "-12",
                "Sword Def"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "-4",
                "in Arctic"
            ],
            [
                "-4",
                "in Desert"
            ],
            [
                "+20",
                "in Jungle"
            ],
            [
                "+90",
                "vs Animals"
            ]
        ]
    },
    "107": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "+30",
                "in Jungle"
            ]
        ]
    },
    "108": {
        "effects": [
            [
                "-16",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ]
        ]
    },
    "109": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ]
        ]
    },
    "110": {
        "effects": [
            [
                "-8",
                "Sword Def"
            ],
            [
                "+50",
                "Magic Res"
            ],
            [
                "-8",
                "vs Monsterous"
            ],
            [
                "+20",
                "vs Undead"
            ]
        ]
    },
    "111": {
        "effects": [
            [
                "+80",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ]
        ]
    },
    "112": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "113": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ]
        ]
    },
    "114": {
        "effects": [
            [
                "+50",
                "Attack"
            ],
            [
                "+25",
                "Cav Def"
            ],
            [
                "+25",
                "Sword Def"
            ]
        ]
    },
    "115": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "+60",
                "Sword Def"
            ],
            [
                "-2",
                "in Arctic"
            ],
            [
                "-2",
                "in Desert"
            ],
            [
                "+10",
                "in Jungle"
            ]
        ]
    },
    "116": {
        "effects": [
            [
                "+6",
                "Attack"
            ],
            [
                "-15",
                "Move Speed"
            ]
        ]
    },
    "120": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+15",
                "for Orcs"
            ],
            [
                "+120",
                "vs Monsterous"
            ],
            [
                "+120",
                "vs Animals"
            ],
            [
                "+120",
                "vs Undead"
            ]
        ]
    },
    "121": {
        "effects": [
            [
                "-12",
                "Cav Def"
            ],
            [
                "-12",
                "Spear Def"
            ],
            [
                "+120",
                "Bow Def"
            ],
            [
                "-12",
                "Sword Def"
            ]
        ]
    },
    "122": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "+40",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "123": {
        "effects": [
            [
                "-8",
                "Attack"
            ],
            [
                "+100",
                "Cav Def"
            ],
            [
                "+100",
                "Spear Def"
            ],
            [
                "+100",
                "Bow Def"
            ],
            [
                "+100",
                "Sword Def"
            ],
            [
                "-40",
                "Move Speed"
            ]
        ]
    },
    "124": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+15",
                "for Orcs"
            ]
        ]
    },
    "125": {
        "effects": [
            [
                "+20",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+40",
                "for Elves"
            ]
        ]
    },
    "126": {
        "effects": [
            [
                "+40",
                "in Arctic"
            ],
            [
                "-24",
                "in Desert"
            ],
            [
                "-24",
                "in Jungle"
            ]
        ]
    },
    "127": {
        "effects": [
            [
                "-12",
                "Cav Def"
            ],
            [
                "+120",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "-12",
                "Sword Def"
            ]
        ]
    },
    "128": {
        "effects": [
            [
                "-2",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ]
        ]
    },
    "129": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ],
            [
                "+40",
                "for Orcs"
            ]
        ]
    },
    "130": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "+40",
                "in Jungle"
            ]
        ]
    },
    "131": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "for Elves"
            ],
            [
                "+30",
                "for Orcs"
            ]
        ]
    },
    "132": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+90",
                "at Night"
            ],
            [
                "-12",
                "in Day"
            ]
        ]
    },
    "133": {
        "effects": [
            [
                "-16",
                "Cav Def"
            ],
            [
                "-16",
                "Spear Def"
            ],
            [
                "-16",
                "Bow Def"
            ],
            [
                "-16",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+140",
                "vs Animals"
            ]
        ]
    },
    "134": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ],
            [
                "+40",
                "for Humans"
            ]
        ]
    },
    "135": {
        "effects": [
            [
                "+24",
                "Cav Def"
            ],
            [
                "-6",
                "Spear Def"
            ],
            [
                "-6",
                "Bow Def"
            ],
            [
                "-6",
                "Sword Def"
            ]
        ]
    },
    "136": {
        "effects": [
            [
                "-12",
                "Cav Def"
            ],
            [
                "-12",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "+120",
                "Sword Def"
            ]
        ]
    },
    "137": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "-12",
                "at Night"
            ],
            [
                "+90",
                "in Day"
            ]
        ]
    },
    "138": {
        "effects": [
            [
                "+60",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ],
            [
                "+40",
                "for Dwarves"
            ]
        ]
    },
    "139": {
        "effects": [
            [
                "+100",
                "Attack"
            ],
            [
                "-12",
                "Cav Def"
            ],
            [
                "-12",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "-12",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ]
        ]
    },
    "141": {
        "effects": [
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ]
        ]
    },
    "142": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "+30",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "143": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "+16",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ]
        ]
    },
    "144": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "+30",
                "Cav Def"
            ],
            [
                "+30",
                "Spear Def"
            ],
            [
                "+30",
                "Bow Def"
            ],
            [
                "+30",
                "Sword Def"
            ],
            [
                "-40",
                "Move Speed"
            ],
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "-8",
                "in Jungle"
            ]
        ]
    },
    "145": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+25",
                "in Desert"
            ],
            [
                "+25",
                "in Jungle"
            ]
        ]
    },
    "146": {
        "effects": [
            [
                "+30",
                "in Arctic"
            ],
            [
                "-24",
                "in Desert"
            ],
            [
                "-24",
                "in Jungle"
            ]
        ]
    },
    "147": {
        "effects": [
            [
                "-2",
                "Attack"
            ],
            [
                "+4",
                "Cav Def"
            ],
            [
                "+4",
                "Spear Def"
            ],
            [
                "+4",
                "Bow Def"
            ],
            [
                "+4",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "-4",
                "in Arctic"
            ],
            [
                "-4",
                "in Desert"
            ],
            [
                "-4",
                "in Jungle"
            ]
        ]
    },
    "148": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ]
        ]
    },
    "149": {
        "effects": [
            [
                "+60",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "150": {
        "effects": [
            [
                "-8",
                "in Arctic"
            ],
            [
                "-8",
                "in Desert"
            ],
            [
                "+30",
                "in Jungle"
            ]
        ]
    },
    "151": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ]
        ]
    },
    "152": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "+25",
                "Cav Def"
            ],
            [
                "+25",
                "Spear Def"
            ],
            [
                "+25",
                "Bow Def"
            ],
            [
                "+25",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+50",
                "for Humans"
            ]
        ]
    },
    "153": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+50",
                "vs Animals"
            ]
        ]
    },
    "154": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "155": {
        "effects": [
            [
                "+80",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ]
        ]
    },
    "156": {
        "effects": [
            [
                "+40",
                "Attack"
            ],
            [
                "+75",
                "Cav Def"
            ],
            [
                "+75",
                "Spear Def"
            ],
            [
                "+75",
                "Bow Def"
            ],
            [
                "+75",
                "Sword Def"
            ]
        ]
    },
    "157": {
        "effects": [
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ]
        ]
    },
    "158": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ]
        ]
    },
    "159": {
        "effects": [
            [
                "+20",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ]
        ]
    },
    "161": {
        "effects": [
            [
                "+50",
                "Cav Def"
            ],
            [
                "+50",
                "Spear Def"
            ],
            [
                "+50",
                "Bow Def"
            ],
            [
                "+25",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "-16",
                "in Arctic"
            ],
            [
                "-16",
                "in Desert"
            ],
            [
                "-16",
                "in Jungle"
            ]
        ]
    },
    "162": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "+16",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ]
        ]
    },
    "163": {
        "effects": [
            [
                "-12",
                "in Arctic"
            ],
            [
                "+30",
                "in Desert"
            ],
            [
                "-12",
                "in Jungle"
            ]
        ]
    },
    "164": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "+50",
                "Cav Def"
            ],
            [
                "+70",
                "Spear Def"
            ],
            [
                "+70",
                "Bow Def"
            ],
            [
                "+50",
                "Sword Def"
            ],
            [
                "-50",
                "Move Speed"
            ],
            [
                "+30",
                "on Hills"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-20",
                "in Arctic"
            ],
            [
                "-20",
                "in Desert"
            ],
            [
                "-20",
                "in Jungle"
            ],
            [
                "+40",
                "for Dwarves"
            ],
            [
                "-24",
                "for Elves"
            ],
            [
                "-24",
                "for Humans"
            ],
            [
                "-24",
                "for Orcs"
            ]
        ]
    },
    "165": {
        "effects": [
            [
                "+20",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ]
        ]
    },
    "166": {
        "effects": [
            [
                "+30",
                "in Arctic"
            ],
            [
                "-36",
                "in Desert"
            ],
            [
                "-36",
                "in Jungle"
            ]
        ]
    },
    "167": {
        "effects": [
            [
                "-2",
                "Attack"
            ],
            [
                "+4",
                "Cav Def"
            ],
            [
                "+4",
                "Spear Def"
            ],
            [
                "+4",
                "Bow Def"
            ],
            [
                "+4",
                "Sword Def"
            ],
            [
                "-20",
                "Move Speed"
            ],
            [
                "-5",
                "in Arctic"
            ],
            [
                "-5",
                "in Desert"
            ],
            [
                "-5",
                "in Jungle"
            ]
        ]
    },
    "168": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "-24",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ]
        ]
    },
    "169": {
        "effects": [
            [
                "-12",
                "in Arctic"
            ],
            [
                "-12",
                "in Desert"
            ],
            [
                "+30",
                "in Jungle"
            ]
        ]
    },
    "170": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+10",
                "Move Speed"
            ]
        ]
    },
    "171": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-6",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+200",
                "Magic Res"
            ],
            [
                "+35",
                "vs Undead"
            ]
        ]
    },
    "172": {
        "effects": [
            [
                "-24",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "173": {
        "effects": [
            [
                "+80",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "-8",
                "Sword Def"
            ]
        ]
    },
    "174": {
        "effects": [
            [
                "+40",
                "Attack"
            ],
            [
                "+75",
                "Cav Def"
            ],
            [
                "+75",
                "Spear Def"
            ],
            [
                "+75",
                "Bow Def"
            ],
            [
                "+75",
                "Sword Def"
            ]
        ]
    },
    "175": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ],
            [
                "+90",
                "vs Monsterous"
            ],
            [
                "+50",
                "vs Animals"
            ]
        ]
    },
    "176": {
        "effects": [
            [
                "-4",
                "Cav Def"
            ],
            [
                "+16",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ]
        ]
    },
    "177": {
        "effects": [
            [
                "-8",
                "Cav Def"
            ],
            [
                "-8",
                "Spear Def"
            ],
            [
                "-8",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ]
        ]
    },
    "178": {
        "effects": [
            [
                "+60",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "-24",
                "on Mountains"
            ],
            [
                "-24",
                "in Forests"
            ]
        ]
    },
    "179": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "-4",
                "Sword Def"
            ]
        ]
    },
    "422": {
        "effects": [
            [
                "+15",
                "Move Speed"
            ],
            [
                "-20",
                "for Dwarves"
            ],
            [
                "-20",
                "for Humans"
            ],
            [
                "-20",
                "for Orcs"
            ]
        ]
    },
    "423": {
        "effects": [
            [
                "+20",
                "Cav Def"
            ],
            [
                "+20",
                "Spear Def"
            ],
            [
                "+20",
                "Bow Def"
            ],
            [
                "+20",
                "Sword Def"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "-15",
                "on Mountains"
            ],
            [
                "-10",
                "in Forests"
            ],
            [
                "-5",
                "in Arctic"
            ],
            [
                "-5",
                "in Desert"
            ],
            [
                "-5",
                "in Jungle"
            ],
            [
                "-20",
                "for Dwarves"
            ],
            [
                "-20",
                "for Elves"
            ],
            [
                "-20",
                "for Orcs"
            ]
        ]
    },
    "424": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "+20",
                "on Hills"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "-20",
                "for Elves"
            ],
            [
                "-20",
                "for Humans"
            ],
            [
                "-20",
                "for Orcs"
            ]
        ]
    },
    "425": {
        "effects": [
            [
                "+10",
                "Move Speed"
            ],
            [
                "+20",
                "Carry"
            ],
            [
                "+20",
                "on Hills"
            ],
            [
                "-20",
                "for Dwarves"
            ],
            [
                "-20",
                "for Elves"
            ],
            [
                "-20",
                "for Humans"
            ]
        ]
    },
    "427": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "-30",
                "Carry"
            ],
            [
                "-5",
                "Magic Res"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "-10",
                "in Desert"
            ],
            [
                "-10",
                "in Jungle"
            ]
        ]
    },
    "428": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+25",
                "Move Speed"
            ],
            [
                "-40",
                "for Dwarves"
            ],
            [
                "+10",
                "for Elves"
            ],
            [
                "-40",
                "for Humans"
            ],
            [
                "-40",
                "for Orcs"
            ]
        ]
    },
    "429": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+30",
                "Cav Def"
            ],
            [
                "+30",
                "Spear Def"
            ],
            [
                "+30",
                "Bow Def"
            ],
            [
                "+30",
                "Sword Def"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "-40",
                "for Dwarves"
            ],
            [
                "-40",
                "for Elves"
            ],
            [
                "+10",
                "for Humans"
            ],
            [
                "-40",
                "for Orcs"
            ]
        ]
    },
    "430": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+30",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "+30",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ],
            [
                "+20",
                "in Arctic"
            ],
            [
                "+20",
                "in Desert"
            ],
            [
                "+20",
                "in Jungle"
            ],
            [
                "+10",
                "for Dwarves"
            ],
            [
                "-40",
                "for Elves"
            ],
            [
                "-40",
                "for Humans"
            ],
            [
                "-40",
                "for Orcs"
            ]
        ]
    },
    "431": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+20",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+30",
                "on Hills"
            ],
            [
                "-40",
                "for Dwarves"
            ],
            [
                "-40",
                "for Elves"
            ],
            [
                "-40",
                "for Humans"
            ],
            [
                "+10",
                "for Orcs"
            ]
        ]
    },
    "432": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "-10",
                "in Forests"
            ],
            [
                "-10",
                "in Jungle"
            ]
        ]
    },
    "433": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "for Dwarves"
            ],
            [
                "-5",
                "for Elves"
            ],
            [
                "-5",
                "for Humans"
            ],
            [
                "+5",
                "for Orcs"
            ],
            [
                "+10",
                "vs Animals"
            ]
        ]
    },
    "434": {
        "effects": [
            [
                "+8",
                "Attack"
            ],
            [
                "+15",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "+15",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "-5",
                "in Jungle"
            ]
        ]
    },
    "435": {
        "effects": [
            [
                "+35",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-15",
                "on Mountains"
            ],
            [
                "-15",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "-15",
                "in Jungle"
            ]
        ]
    },
    "436": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "-5",
                "in Desert"
            ]
        ]
    },
    "437": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "-5",
                "in Desert"
            ]
        ]
    },
    "438": {
        "effects": [
            [
                "+19",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "-5",
                "in Arctic"
            ],
            [
                "-5",
                "in Jungle"
            ]
        ]
    },
    "439": {
        "effects": [
            [
                "+3",
                "Attack"
            ],
            [
                "+32",
                "Cav Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ]
        ]
    },
    "440": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-10",
                "in Day"
            ],
            [
                "+30",
                "vs Monsterous"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "441": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "at Night"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+30",
                "vs Monsterous"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "442": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-5",
                "for Dwarves"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+5",
                "for Humans"
            ],
            [
                "-5",
                "for Orcs"
            ],
            [
                "+10",
                "vs Animals"
            ]
        ]
    },
    "443": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "+65",
                "at Night"
            ],
            [
                "-25",
                "in Day"
            ]
        ]
    },
    "444": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "-25",
                "at Night"
            ],
            [
                "+65",
                "in Day"
            ]
        ]
    },
    "445": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-4",
                "Cav Def"
            ],
            [
                "-4",
                "Spear Def"
            ],
            [
                "-4",
                "Bow Def"
            ],
            [
                "+6",
                "Sword Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+25",
                "in Jungle"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "446": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+20",
                "Move Speed"
            ],
            [
                "+10",
                "in Desert"
            ]
        ]
    },
    "447": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "at Night"
            ],
            [
                "-5",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "448": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ],
            [
                "+50",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "449": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+5",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "at Night"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "450": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "451": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+60",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+10",
                "in Jungle"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "452": {
        "effects": [
            [
                "+170",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+50",
                "on Plains"
            ],
            [
                "+50",
                "in Arctic"
            ],
            [
                "+50",
                "in Desert"
            ]
        ]
    },
    "453": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+80",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+60",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+60",
                "in Arctic"
            ],
            [
                "+60",
                "in Desert"
            ],
            [
                "+60",
                "in Jungle"
            ]
        ]
    },
    "454": {
        "effects": [
            [
                "+200",
                "Attack"
            ],
            [
                "+100",
                "Cav Def"
            ],
            [
                "+75",
                "Spear Def"
            ],
            [
                "+50",
                "Bow Def"
            ],
            [
                "+125",
                "Sword Def"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    },
    "455": {
        "effects": [
            [
                "+4",
                "Attack"
            ],
            [
                "+15",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "+15",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "-5",
                "in Jungle"
            ],
            [
                "+10",
                "vs Undead"
            ]
        ]
    },
    "456": {
        "effects": [
            [
                "+4",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "+15",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+15",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "-5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+10",
                "vs Undead"
            ]
        ]
    },
    "457": {
        "effects": [
            [
                "+35",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "-5",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "-5",
                "Carry"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-15",
                "on Mountains"
            ],
            [
                "-15",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "-15",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ]
        ]
    },
    "458": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-5",
                "for Dwarves"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+5",
                "for Humans"
            ],
            [
                "-5",
                "for Orcs"
            ],
            [
                "+50",
                "vs Animals"
            ]
        ]
    },
    "459": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "for Dwarves"
            ],
            [
                "-5",
                "for Elves"
            ],
            [
                "-5",
                "for Humans"
            ],
            [
                "+5",
                "for Orcs"
            ],
            [
                "+50",
                "vs Animals"
            ]
        ]
    },
    "460": {
        "effects": [
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "-20",
                "vs Monsterous"
            ],
            [
                "-20",
                "vs Animals"
            ],
            [
                "-20",
                "vs Undead"
            ]
        ]
    },
    "461": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "+65",
                "at Night"
            ],
            [
                "-25",
                "in Day"
            ]
        ]
    },
    "462": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "-25",
                "at Night"
            ],
            [
                "+65",
                "in Day"
            ]
        ]
    },
    "463": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-10",
                "in Day"
            ],
            [
                "+30",
                "vs Monsterous"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "464": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "at Night"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+30",
                "vs Monsterous"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "465": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-30",
                "Cav Def"
            ],
            [
                "-30",
                "Spear Def"
            ],
            [
                "+85",
                "Bow Def"
            ],
            [
                "-30",
                "Sword Def"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "-10",
                "on Mountains"
            ],
            [
                "-10",
                "in Forests"
            ],
            [
                "-10",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "-10",
                "in Jungle"
            ],
            [
                "-10",
                "at Night"
            ]
        ]
    },
    "466": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "+35",
                "on Plains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "-5",
                "in Desert"
            ]
        ]
    },
    "467": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+35",
                "in Desert"
            ]
        ]
    },
    "468": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+20",
                "Move Speed"
            ]
        ]
    },
    "469": {
        "effects": [
            [
                "-4",
                "Attack"
            ],
            [
                "-6",
                "Cav Def"
            ],
            [
                "-6",
                "Spear Def"
            ],
            [
                "-6",
                "Bow Def"
            ],
            [
                "-6",
                "Sword Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+5",
                "Carry"
            ],
            [
                "+25",
                "in Jungle"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "470": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "at Night"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "471": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "472": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "at Night"
            ],
            [
                "-5",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "473": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ],
            [
                "+150",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "474": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+60",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+45",
                "in Jungle"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "475": {
        "effects": [
            [
                "+170",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+50",
                "on Plains"
            ],
            [
                "+50",
                "in Arctic"
            ],
            [
                "+50",
                "in Desert"
            ]
        ]
    },
    "476": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+80",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+60",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+60",
                "in Arctic"
            ],
            [
                "+60",
                "in Desert"
            ],
            [
                "+60",
                "in Jungle"
            ]
        ]
    },
    "477": {
        "effects": [
            [
                "+200",
                "Attack"
            ],
            [
                "+50",
                "Cav Def"
            ],
            [
                "+100",
                "Spear Def"
            ],
            [
                "+75",
                "Bow Def"
            ],
            [
                "+125",
                "Sword Def"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    },
    "478": {
        "effects": [
            [
                "+8",
                "Attack"
            ],
            [
                "+8",
                "Cav Def"
            ],
            [
                "-12",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "+12",
                "Sword Def"
            ]
        ]
    },
    "479": {
        "effects": [
            [
                "+8",
                "Attack"
            ],
            [
                "-12",
                "Cav Def"
            ],
            [
                "+8",
                "Spear Def"
            ],
            [
                "-12",
                "Bow Def"
            ],
            [
                "+12",
                "Sword Def"
            ]
        ]
    },
    "480": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "+30",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "-20",
                "Sword Def"
            ],
            [
                "+8",
                "on Hills"
            ],
            [
                "+8",
                "on Plains"
            ],
            [
                "+2",
                "on Mountains"
            ],
            [
                "-12",
                "in Forests"
            ],
            [
                "-12",
                "in Jungle"
            ]
        ]
    },
    "481": {
        "effects": [
            [
                "-6",
                "Attack"
            ],
            [
                "-6",
                "Cav Def"
            ],
            [
                "-6",
                "Spear Def"
            ],
            [
                "+6",
                "Bow Def"
            ],
            [
                "+6",
                "Sword Def"
            ],
            [
                "+8",
                "Move Speed"
            ],
            [
                "+15",
                "in Forests"
            ],
            [
                "+15",
                "in Jungle"
            ]
        ]
    },
    "482": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-5",
                "Carry"
            ],
            [
                "+16",
                "on Hills"
            ],
            [
                "-6",
                "on Plains"
            ],
            [
                "+16",
                "on Mountains"
            ],
            [
                "-6",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "-6",
                "in Jungle"
            ]
        ]
    },
    "483": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-5",
                "Spear Def"
            ],
            [
                "+15",
                "Bow Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+25",
                "in Jungle"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "484": {
        "effects": [
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+6",
                "Move Speed"
            ],
            [
                "-5",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+2",
                "in Forests"
            ],
            [
                "-5",
                "in Arctic"
            ],
            [
                "+2",
                "in Jungle"
            ],
            [
                "+5",
                "vs Animals"
            ]
        ]
    },
    "485": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-5",
                "for Dwarves"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+5",
                "for Humans"
            ],
            [
                "-5",
                "for Orcs"
            ],
            [
                "+10",
                "vs Animals"
            ]
        ]
    },
    "486": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "for Dwarves"
            ],
            [
                "-5",
                "for Elves"
            ],
            [
                "-5",
                "for Humans"
            ],
            [
                "+5",
                "for Orcs"
            ],
            [
                "+10",
                "vs Animals"
            ]
        ]
    },
    "487": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "+65",
                "at Night"
            ],
            [
                "-25",
                "in Day"
            ]
        ]
    },
    "488": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "-25",
                "at Night"
            ],
            [
                "+65",
                "in Day"
            ]
        ]
    },
    "489": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-10",
                "in Day"
            ],
            [
                "+30",
                "vs Monsterous"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "490": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "at Night"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+30",
                "vs Monsterous"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "491": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+45",
                "in Arctic"
            ],
            [
                "-5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ]
        ]
    },
    "492": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+45",
                "in Desert"
            ],
            [
                "-5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ]
        ]
    },
    "493": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "at Night"
            ],
            [
                "-5",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "494": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ],
            [
                "+50",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "495": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "at Night"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "496": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "497": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+60",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+10",
                "in Jungle"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "498": {
        "effects": [
            [
                "+170",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+50",
                "on Plains"
            ],
            [
                "+50",
                "in Arctic"
            ],
            [
                "+50",
                "in Desert"
            ]
        ]
    },
    "499": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+80",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+60",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+60",
                "in Arctic"
            ],
            [
                "+60",
                "in Desert"
            ],
            [
                "+60",
                "in Jungle"
            ]
        ]
    },
    "500": {
        "effects": [
            [
                "+200",
                "Attack"
            ],
            [
                "+125",
                "Cav Def"
            ],
            [
                "+100",
                "Spear Def"
            ],
            [
                "+50",
                "Bow Def"
            ],
            [
                "+75",
                "Sword Def"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    },
    "501": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+10",
                "in Day"
            ],
            [
                "+50",
                "vs Animals"
            ]
        ]
    },
    "502": {
        "effects": [
            [
                "-15",
                "Attack"
            ],
            [
                "+30",
                "Cav Def"
            ],
            [
                "+30",
                "Spear Def"
            ],
            [
                "+30",
                "Bow Def"
            ],
            [
                "+30",
                "Sword Def"
            ],
            [
                "-10",
                "Carry"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "-5",
                "in Desert"
            ],
            [
                "-5",
                "in Jungle"
            ]
        ]
    },
    "503": {
        "effects": [
            [
                "-6",
                "Sword Def"
            ],
            [
                "+45",
                "Magic Res"
            ],
            [
                "+5",
                "at Night"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "504": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-5",
                "for Dwarves"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+5",
                "for Humans"
            ],
            [
                "-5",
                "for Orcs"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "505": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-5",
                "Spear Def"
            ],
            [
                "+15",
                "Bow Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+5",
                "Carry"
            ],
            [
                "+25",
                "in Jungle"
            ],
            [
                "+15",
                "vs Animals"
            ]
        ]
    },
    "506": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "+20",
                "Move Speed"
            ],
            [
                "+35",
                "in Arctic"
            ]
        ]
    },
    "507": {
        "effects": [
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-10",
                "in Arctic"
            ],
            [
                "-10",
                "in Desert"
            ],
            [
                "-10",
                "in Jungle"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Animals"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "508": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-10",
                "in Day"
            ],
            [
                "+10",
                "vs Monsterous"
            ],
            [
                "+15",
                "vs Undead"
            ]
        ]
    },
    "509": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "at Night"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+10",
                "vs Monsterous"
            ],
            [
                "+15",
                "vs Undead"
            ]
        ]
    },
    "510": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "for Dwarves"
            ],
            [
                "-5",
                "for Elves"
            ],
            [
                "-5",
                "for Humans"
            ],
            [
                "+5",
                "for Orcs"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "511": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "at Night"
            ],
            [
                "-5",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "512": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ],
            [
                "+50",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "513": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "at Night"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "514": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "515": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+60",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+10",
                "in Jungle"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "516": {
        "effects": [
            [
                "+170",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+50",
                "on Plains"
            ],
            [
                "+50",
                "in Arctic"
            ],
            [
                "+50",
                "in Desert"
            ]
        ]
    },
    "517": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+80",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+60",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+60",
                "in Arctic"
            ],
            [
                "+60",
                "in Desert"
            ],
            [
                "+60",
                "in Jungle"
            ]
        ]
    },
    "518": {
        "effects": [
            [
                "+30",
                "Attack"
            ],
            [
                "+125",
                "Cav Def"
            ],
            [
                "+125",
                "Spear Def"
            ],
            [
                "+125",
                "Bow Def"
            ],
            [
                "+125",
                "Sword Def"
            ],
            [
                "+20",
                "on Hills"
            ],
            [
                "+20",
                "on Plains"
            ],
            [
                "+20",
                "on Mountains"
            ],
            [
                "+20",
                "in Forests"
            ],
            [
                "+20",
                "in Arctic"
            ],
            [
                "+20",
                "in Desert"
            ],
            [
                "+20",
                "in Jungle"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    },
    "519": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "for Dwarves"
            ],
            [
                "-5",
                "for Elves"
            ],
            [
                "-5",
                "for Humans"
            ],
            [
                "+5",
                "for Orcs"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "520": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-5",
                "for Dwarves"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+5",
                "for Humans"
            ],
            [
                "-5",
                "for Orcs"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "521": {
        "effects": [
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-10",
                "in Arctic"
            ],
            [
                "-10",
                "in Desert"
            ],
            [
                "-10",
                "in Jungle"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Animals"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "522": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-5",
                "Spear Def"
            ],
            [
                "+15",
                "Bow Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+5",
                "Carry"
            ],
            [
                "+25",
                "in Jungle"
            ],
            [
                "+15",
                "vs Animals"
            ]
        ]
    },
    "523": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-10",
                "in Day"
            ],
            [
                "+10",
                "vs Monsterous"
            ],
            [
                "+15",
                "vs Undead"
            ]
        ]
    },
    "524": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "at Night"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+10",
                "vs Monsterous"
            ],
            [
                "+15",
                "vs Undead"
            ]
        ]
    },
    "525": {
        "effects": [
            [
                "-6",
                "Bow Def"
            ],
            [
                "+45",
                "Magic Res"
            ],
            [
                "+5",
                "at Night"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "526": {
        "effects": [
            [
                "+20",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-5",
                "Spear Def"
            ],
            [
                "+25",
                "Bow Def"
            ],
            [
                "-5",
                "Sword Def"
            ],
            [
                "-5",
                "Move Speed"
            ],
            [
                "-5",
                "Carry"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-15",
                "on Mountains"
            ],
            [
                "-15",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "-15",
                "in Jungle"
            ]
        ]
    },
    "527": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "-25",
                "at Night"
            ],
            [
                "+65",
                "in Day"
            ]
        ]
    },
    "528": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "+65",
                "at Night"
            ],
            [
                "-25",
                "in Day"
            ]
        ]
    },
    "529": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "at Night"
            ],
            [
                "-5",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "530": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ],
            [
                "+50",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "531": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "at Night"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "532": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "533": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+60",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "534": {
        "effects": [
            [
                "+170",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+50",
                "on Plains"
            ],
            [
                "+50",
                "in Arctic"
            ],
            [
                "+50",
                "in Desert"
            ]
        ]
    },
    "535": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+80",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+60",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+60",
                "in Arctic"
            ],
            [
                "+60",
                "in Desert"
            ],
            [
                "+60",
                "in Jungle"
            ]
        ]
    },
    "536": {
        "effects": [
            [
                "+30",
                "Attack"
            ],
            [
                "+150",
                "Cav Def"
            ],
            [
                "+150",
                "Spear Def"
            ],
            [
                "+150",
                "Bow Def"
            ],
            [
                "+150",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+10",
                "in Jungle"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    },
    "537": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+5",
                "for Dwarves"
            ],
            [
                "-5",
                "for Elves"
            ],
            [
                "-5",
                "for Humans"
            ],
            [
                "+5",
                "for Orcs"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "538": {
        "effects": [
            [
                "-5",
                "Attack"
            ],
            [
                "+10",
                "Move Speed"
            ],
            [
                "+10",
                "Carry"
            ],
            [
                "-5",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "-5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-5",
                "for Dwarves"
            ],
            [
                "+5",
                "for Elves"
            ],
            [
                "+5",
                "for Humans"
            ],
            [
                "-5",
                "for Orcs"
            ],
            [
                "+30",
                "vs Animals"
            ]
        ]
    },
    "539": {
        "effects": [
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "-10",
                "in Arctic"
            ],
            [
                "-10",
                "in Desert"
            ],
            [
                "-10",
                "in Jungle"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Animals"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "540": {
        "effects": [
            [
                "-10",
                "Attack"
            ],
            [
                "-5",
                "Cav Def"
            ],
            [
                "-5",
                "Spear Def"
            ],
            [
                "+15",
                "Bow Def"
            ],
            [
                "+15",
                "Move Speed"
            ],
            [
                "+5",
                "Carry"
            ],
            [
                "+15",
                "vs Animals"
            ]
        ]
    },
    "541": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+30",
                "at Night"
            ],
            [
                "-10",
                "in Day"
            ],
            [
                "+10",
                "vs Monsterous"
            ],
            [
                "+15",
                "vs Undead"
            ]
        ]
    },
    "542": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "-10",
                "at Night"
            ],
            [
                "+30",
                "in Day"
            ],
            [
                "+10",
                "vs Monsterous"
            ],
            [
                "+15",
                "vs Undead"
            ]
        ]
    },
    "543": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "+65",
                "at Night"
            ],
            [
                "-25",
                "in Day"
            ]
        ]
    },
    "544": {
        "effects": [
            [
                "-10",
                "Move Speed"
            ],
            [
                "+25",
                "Magic Res"
            ],
            [
                "-25",
                "at Night"
            ],
            [
                "+65",
                "in Day"
            ]
        ]
    },
    "545": {
        "effects": [
            [
                "-6",
                "Spear Def"
            ],
            [
                "+45",
                "Magic Res"
            ],
            [
                "+5",
                "at Night"
            ],
            [
                "+45",
                "vs Undead"
            ]
        ]
    },
    "546": {
        "effects": [
            [
                "+50",
                "Attack"
            ],
            [
                "-10",
                "Cav Def"
            ],
            [
                "-10",
                "Spear Def"
            ],
            [
                "-10",
                "Bow Def"
            ],
            [
                "-10",
                "Sword Def"
            ],
            [
                "-10",
                "Move Speed"
            ],
            [
                "-5",
                "Carry"
            ],
            [
                "-5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "-5",
                "in Forests"
            ],
            [
                "+30",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "-10",
                "in Jungle"
            ],
            [
                "+65",
                "vs Monsterous"
            ],
            [
                "-5",
                "vs Undead"
            ]
        ]
    },
    "547": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+2",
                "Move Speed"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+50",
                "at Night"
            ],
            [
                "-5",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "548": {
        "effects": [
            [
                "+25",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+15",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "-5",
                "at Night"
            ],
            [
                "+50",
                "in Day"
            ],
            [
                "+130",
                "vs Monsterous"
            ],
            [
                "+145",
                "vs Undead"
            ]
        ]
    },
    "549": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "at Night"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "550": {
        "effects": [
            [
                "+10",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+70",
                "Magic Res"
            ],
            [
                "+5",
                "on Hills"
            ],
            [
                "+5",
                "on Plains"
            ],
            [
                "+5",
                "on Mountains"
            ],
            [
                "+5",
                "in Forests"
            ],
            [
                "+5",
                "in Arctic"
            ],
            [
                "+5",
                "in Desert"
            ],
            [
                "+5",
                "in Jungle"
            ],
            [
                "+150",
                "in Day"
            ],
            [
                "+5",
                "vs Monsterous"
            ],
            [
                "+5",
                "vs Undead"
            ]
        ]
    },
    "551": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+10",
                "Cav Def"
            ],
            [
                "+10",
                "Spear Def"
            ],
            [
                "+10",
                "Bow Def"
            ],
            [
                "+10",
                "Sword Def"
            ],
            [
                "+60",
                "Move Speed"
            ],
            [
                "+30",
                "Carry"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+10",
                "on Hills"
            ],
            [
                "+10",
                "on Plains"
            ],
            [
                "+10",
                "on Mountains"
            ],
            [
                "+10",
                "in Forests"
            ],
            [
                "+10",
                "in Arctic"
            ],
            [
                "+10",
                "in Desert"
            ],
            [
                "+10",
                "in Jungle"
            ],
            [
                "+100",
                "vs Animals"
            ]
        ]
    },
    "552": {
        "effects": [
            [
                "+170",
                "Attack"
            ],
            [
                "+5",
                "Cav Def"
            ],
            [
                "+5",
                "Spear Def"
            ],
            [
                "+5",
                "Bow Def"
            ],
            [
                "+5",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+50",
                "on Plains"
            ],
            [
                "+50",
                "in Arctic"
            ],
            [
                "+50",
                "in Desert"
            ]
        ]
    },
    "553": {
        "effects": [
            [
                "+5",
                "Attack"
            ],
            [
                "+80",
                "Cav Def"
            ],
            [
                "+80",
                "Spear Def"
            ],
            [
                "+80",
                "Bow Def"
            ],
            [
                "+80",
                "Sword Def"
            ],
            [
                "+10",
                "Magic Res"
            ],
            [
                "+60",
                "on Hills"
            ],
            [
                "+60",
                "on Plains"
            ],
            [
                "+60",
                "on Mountains"
            ],
            [
                "+60",
                "in Forests"
            ],
            [
                "+60",
                "in Arctic"
            ],
            [
                "+60",
                "in Desert"
            ],
            [
                "+60",
                "in Jungle"
            ]
        ]
    },
    "554": {
        "effects": [
            [
                "+150",
                "Attack"
            ],
            [
                "+175",
                "Cav Def"
            ],
            [
                "+175",
                "Spear Def"
            ],
            [
                "+175",
                "Bow Def"
            ],
            [
                "+175",
                "Sword Def"
            ],
            [
                "+50",
                "vs Undead"
            ]
        ]
    }
};
dt.iClasses = {"i=4|1":["name resIcon ico-gold","Gold"],"i=1|1":["name resIcon ico-wood","Wood"],"i=1|2":["name resIcon ico-clay","Clay"],"i=1|3":["name resIcon ico-iron","Iron"],"i=1|4":["name resIcon ico-stone","Stone"],"i=1|5":["name resIcon ico-food","Food"],"i=2|1":["name resIcon ico-mana","Mana"],"i=2|2":["name resIcon ico-research","Research"],"i=3|1":["name resIcon ico-horses","Horses"],"i=3|2":["name resIcon ico-livestock","Livestock"],"i=3|3":["name resIcon ico-swords","Swords"],"i=3|4":["name resIcon ico-bows","Bows"],"i=3|5":["name resIcon ico-spears","Spears"],"i=3|6":["name resIcon ico-saddles","Saddles"],"i=3|7":["name resIcon ico-books","Books"],"i=3|8":["name resIcon ico-leather","Leather Armour"],"i=3|9":["name resIcon ico-chainmail","Chainmail"],"i=3|10":["name resIcon ico-plate","Plate Armour"],"i=3|12":["name resIcon ico-beer","Beer"],"i=3|11":["name resIcon ico-siege","Siege Blocks"]};
dt.cpi = {1:"i=3|1",2:"i=3|2",3:"i=3|3",4:"i=3|4",5:"i=3|5",6:"i=3|6",7:"i=3|7",8:"i=3|8",9:"i=3|9",10:"i=3|10",11:"i=3|11",12:"i=3|12",17:"i=1|1",19:"i=1|2",21:"i=1|3",23:"i=1|4",25:"i=1|5",183:"i=4|1",295:"i=2|1",296:"i=2|2"};
dt.icp = {"i=1|1":17,"i=1|2":19,"i=1|3":21,"i=1|4":23,"i=1|5":25,"i=2|1":295,"i=2|2":296,"i=3|1":1,"i=3|2":2,"i=3|3":3,"i=3|4":4,"i=3|5":5,"i=3|6":6,"i=3|7":7,"i=3|8":8,"i=3|9":9,"i=3|10":10,"i=3|11":11,"i=3|12":12,"i=4|1":183};
dt.equipmentTypes = {
    "Mounts":[33,34,35,36,37,422,423,424,425,427,428,429,430,431],
    "Spears":[91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,478,479,480,481,482,483,484,485,486,487,488,489,490,491,492,500,497,499,495,496,493,494,498],
    "Swords":[40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,432,433,434,435,436,437,438,439,440,441,442,443,444,445,446,453,454,452,447,448,449,450,451],
    "Bows":[67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,455,456,457,458,459,460,461,462,463,464,465,466,467,468,469,475,476,472,473,470,471,477,474],
    "Leather Armour":[120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,501,502,503,504,505,506,507,508,509,510,518,517,513,514,516,515,511,512],
    "Chainmail":[141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,519,520,521,522,523,524,525,526,527,528,536,535,533,529,530,534,532,531],
    "Plate Armour":[161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,537,538,539,540,541,542,543,544,545,546,554,553,551,547,548,549,550,552]
    };
dt.npcFamilies = {"Alligators":[128,129,130,131], "Anacondas":[132,133,134,135], "Arctic Wolves":[184,185,186,187], "Baboons":[250,251,252,253], "Black Bears":[180,181,182,183], "Black Panthers":[259,260,261,262], "Brown Bears":[102,103,104,105], "Cobras":[136,137,138,139], "Coral Snakes":[140,141,142,143], "Cyclopes":[271,272,273,274], "Dwarves":[50,51,60,61,70,71,80,81,90,91], "Elephants":[164,165,166,167], "Elves":[48,49,58,59,68,69,78,79,88,89], "Escaped Air Elementals":[282], "Escaped Fire Elementals":[280], "Escaped Earth Elementals":[281], "Escaped Water Elementals":[283], "Fire Salamanders":[220,221,222,223], "Gharials":[267,268,269,270], "Ghosts":[293,294,295,296], "Giant Beetles":[242,243,244,245], "Giant Rats":[238,239,240,241], "Giant Scorpions":[156,157,158,159], "Giant Scuttlers":[148,149,150,151], "Giant Snakes":[233,234,235,236,237], "Giant Spiders":[228,229,230,231,232], "Golden Monkeys":[246,247,248,249], "Humans":[46,47,56,57,66,67,76,77,86,87], "Ice Salamanders":[224,225,226,227], "Jaguars":[192,193,194,195], "Leopards":[196,197,198,199], "Lions":[204,205,206,207], "Mammoths":[168,169,170,171], "Massive Scarabs":[254,255,256,257], "Mercenaries":[54,55,64,65,74,75,84,85], "Moumiyas":[297,298,299,300], "Orcs":[45,52,53,62,63,72,73,82,83,92,94], "Poisonous Crawlers":[144,145,146,147], "Polar Bears":[124,125,126,127], "Pumas":[200,201,202,203], "Rhinoceros":[172,173,174,175], "Roaming Trolls":[275,276,277,278], "Salamanders":[216,217,218,219], "Saurians":[263,264,265,266], "Scaled Chargers":[160,161,162,163], "Scritchers":[152,153,154,155], "Serpentkin":[117,118,119,120,121], "Simien Wolves":[176,177,178,179], "Skeletons":[285,286,287,288], "Snow Leopards":[188,189,190,191], "Tigers":[208,209,210,211], "White Tigers":[212,213,214,215], "Wild Dogs":[98,99,100,101], "Wolves":[107,108,109,110], "Zombies":[289,290,291,292], "Revenants":[301,302,303,304], "Wraith":[305,306,307,308], "Undead Dragons":[309,310,311,312], "Strigoi":[313,314,315,316], "Necromancers":[317,318], "Liches":[319,320], "Apaxu":[386,387,388], "Arakvar":[111,114,115,390,391], "Argiri":[710,711], "Atagek":[326,393,394,395], "Azure Throne":[396,397,398,399,400], "Barbarians":[401,402,403,404,405,406], "Bayne's Irregulars":[407,408,409,410,411,412], "Blood Reavers":[413,414,415,416], "Brotherhood of Kerala":[417,418,419,420], "Circle Of Five":[421,422,423,424,425,426], "Clan Bealagh":[428,429,430], "Clan Dollogh":[701], "Clan Moedagh":[431,432,433], "Clan Reashag":[435,436], "Council of Illyria":[122,123], "Crimson Dawn":[437,438,439], "Crimson Skulls":[440,441,442,443], "Daedalons":[444,445,446], "Demersae":[447], "Dragonkin":[448], "Drangu Mundas":[449,450,451,452,453], "Drek-Hhakral":[707,708,709], "Duchy of Keppen":[455], "Dyadans":[460], "Eaglesbrood":[461], "Empirium":[463,464,465,466], "Forbidden Empire":[467,468,469,470,471,472], "Foshnu Ob Wulpor":[473,474,475,476], "Fratri Hominis":[346,478,479,480,481], "Free Orcs":[726], "Fyrgis":[482,483,484], "Giantkind":[485,486], "Guardians of Zera":[487,488,489], "Guul-Hai":[728], "Harpies":[], "Hashashin":[492,493,494], "Heirs of Voyldarn":[732], "Houergerrt":[704], "Illians":[495,496,497], "Illyria Trade Council":[498,499], "Jannu":[500,501,502], "Kadu":[503,504,505], "Kanchalka":[353,507,508,509], "Kapikami":[725], "Kartur-Hhakral":[703], "Keepers of Golden Fields":[510,511,512], "Kildarran":[513,514,515,516], "Kingdom of Larn":[356,518,519,520], "Kingdom of Tal":[521,522,523], "Lannigolds":[524,525,526,527,528,529], "Llwcharion":[717,718], "Lords Of The West":[530,531,532], "Lyrians":[360,534,535,536], "Marauding Skullsplitters":[361,538,539,540], "Melders":[321,322,323,324,362,542,543,544,545], "Mellorians":[729,730], "Nordlanders":[363,551], "Northmen":[552,553,554], "Order of Allembine":[279,555,556], "Outlaws":[558,559,560,561], "Overoad Traders":[714,715,716], "Parlan":[562,563,564,565,566,567], "Parvacones":[568,569,570], "Pax Orcana":[571,572,573,574], "Raja Metu":[575,576,577,578,579], "Seelie Court":[580,581,582], "Sillar":[583,584,585,586], "Sirens":[587,588,589], "Skarakar":[590,591,592], "Sma Uruk":[593,594,595,596,597], "South Oceans Company":[720], "Sslipentin":[284,598,599,600,601], "Sultanate of Kazim":[602,603,604,605,606], "Sundog Gith":[607,608,609,610], "Sylvans":[611,612,613,614,615], "Tal Colonists":[721,722], "Tantarkim":[616,617,618], "Tenaril":[619,620,621,622], "Tewhirrus":[623,624,625], "The College of Silence":[], "The Lost Clans":[727], "The New Light":[712,713], "The Pirate Kings":[705,706], "The Tears Eternal":[702], "The Tower":[731], "Tien Zao":[626,627,628,629,630], "Tinkers":[631,632,633,634,635], "Tipu Khan":[636,637,638,639], "Trappers":[640], "Treggar's Crows":[641,642,643], "Triumvirate":[644,645,646,647], "Turyns":[648,649,650,651], "Udaiman":[652,653,654], "Udan Tebriz":[384,656], "Undying Flame":[657,658,659,660], "Unseelie Court":[661,662,663,664,665], "Virten":[700], "Wen Kun Dynasty":[666,667,668], "Windseekers":[669,670], "Yuraquncha":[723,724], "Zau Brulk":[671,672,673,674]};
dt.cBuildings={33:{name:"Draught Horse",bldgs:[21,62]},34:{name:"Heavy Warhorse",bldgs:[21,62]},35:{name:"Nimble Warhorse",bldgs:[21,62]},36:{name:"Riding Horse",bldgs:[21,62]},37:{name:"Steady Warhorse",bldgs:[21,62]},40:{name:"Adventurer's Sword",bldgs:[32,60]},41:{name:"Barbarian Sword",bldgs:[32,60]},42:{name:"Battle Sword",bldgs:[32,60]},43:{name:"Bone Handled Sword",bldgs:[32,60]},44:{name:"Chitin-Crafted Shortsword",bldgs:[32,60]},45:{name:"Dwarven Battle Axe",bldgs:[32,60]},46:{name:"Ebony-Hilt Sword",bldgs:[32,60]},47:{name:"Elven Sword",bldgs:[32,60]},48:{name:"Fang-Barbed Sword",bldgs:[32,60]},49:{name:"War Axe",bldgs:[32,60]},50:{name:"Light Sword",bldgs:[32,60]},51:{name:"Longsword",bldgs:[32,60]},52:{name:"Machete",bldgs:[32,60]},53:{name:"Mountain-Tribe's Sword",bldgs:[32,60]},54:{name:"Obsidian Blade",bldgs:[32,60]},55:{name:"Orc Marauder's Sword",bldgs:[32,60]},56:{name:"Orcish Sword",bldgs:[32,60]},57:{name:"Razor-Edged Sword",bldgs:[32,60]},58:{name:"Reinforced Sword",bldgs:[32,60]},59:{name:"Sabre",bldgs:[32,60]},60:{name:"Scimitar",bldgs:[32,60]},61:{name:"Seaxe",bldgs:[32,60]},62:{name:"Short Sword",bldgs:[32,60]},63:{name:"Silversteel Sword",bldgs:[32,60]},64:{name:"Spiked Sword",bldgs:[32,60]},65:{name:"Traditional Sword",bldgs:[32,60]},67:{name:"Arctic Bow",bldgs:[28,74]},68:{name:"Chitin-Cored Bow",bldgs:[28,74]},69:{name:"Desert Bow",bldgs:[28,74]},70:{name:"Ebony Bow",bldgs:[28,74]},71:{name:"Elven Spirit Bow",bldgs:[28,74]},72:{name:"Hero's Bow",bldgs:[28,74]},73:{name:"High-Power Bow",bldgs:[28,74]},74:{name:"Hillsman's Bow",bldgs:[28,74]},75:{name:"Hunter's Bow",bldgs:[28,74]},76:{name:"Composite Bow",bldgs:[28,74]},77:{name:"Jungle Bow",bldgs:[28,74]},78:{name:"Leopard Gut Bow",bldgs:[28,74]},79:{name:"Light Bow",bldgs:[28,74]},80:{name:"Long-Draw Bow",bldgs:[28,74]},81:{name:"Mammoth Tusk Bow",bldgs:[28,74]},82:{name:"Marksman's Bow",bldgs:[28,74]},83:{name:"Mountain Bow",bldgs:[28,74]},84:{name:"Plains Bow",bldgs:[28,74]},85:{name:"Poacher's Bow",bldgs:[28,74]},86:{name:"Rapid-Draw Bow",bldgs:[28,74]},87:{name:"Three-Wood Bow",bldgs:[28,74]},88:{name:"War Bow",bldgs:[28,74]},89:{name:"Woodsman's Bow",bldgs:[28,74]},91:{name:"Arctic Spear",bldgs:[31,73]},92:{name:"Battle Spear",bldgs:[31,73]},93:{name:"Boar Spear",bldgs:[31,73]},94:{name:"Defender's Spear",bldgs:[31,73]},95:{name:"Desert Pike",bldgs:[31,73]},96:{name:"Desert Spear",bldgs:[31,73]},97:{name:"Dragon Spear",bldgs:[31,73]},98:{name:"Duelling Spear",bldgs:[31,73]},99:{name:"Ebony Spear",bldgs:[31,73]},100:{name:"Fang-Barbed Spear",bldgs:[31,73]},101:{name:"Fang-Tipped Spear",bldgs:[31,73]},102:{name:"Forester's Spear",bldgs:[31,73]},103:{name:"Harpoon-Spear",bldgs:[31,73]},104:{name:"Hill-Tribe Spear",bldgs:[31,73]},105:{name:"Iron-Banded Spear",bldgs:[31,73]},106:{name:"Jungle Hunter's Spear",bldgs:[31,73]},107:{name:"Jungle Spear",bldgs:[31,73]},108:{name:"Light Spear",bldgs:[31,73]},109:{name:"Mountain Spear",bldgs:[31,73]},110:{name:"Obsidian-Tipped Spear",bldgs:[31,73]},111:{name:"Pike",bldgs:[31,73]},112:{name:"Plainsman's Spear",bldgs:[31,73]},113:{name:"Razor-Edged Spear",bldgs:[31,73]},114:{name:"Silversteel Spear",bldgs:[31,73]},115:{name:"Trident",bldgs:[31,73]},116:{name:"War Spear",bldgs:[31,73]},120:{name:"Animal-Scale Armour",bldgs:[34,76]},121:{name:"Cloth-Backed Leather",bldgs:[34,76]},122:{name:"Desert Armour",bldgs:[34,76]},123:{name:"Extra Heavy Armour",bldgs:[34,76]},124:{name:"Extra Light Armour",bldgs:[34,76]},125:{name:"Forester's Armour",bldgs:[34,76]},126:{name:"Fur-Lined Armour",bldgs:[34,76]},127:{name:"Hardened Leather",bldgs:[34,76]},128:{name:"Heavy Leather",bldgs:[34,76]},129:{name:"Highland Armour",bldgs:[34,76]},130:{name:"Jungle Armour",bldgs:[34,76]},131:{name:"Light Leather",bldgs:[34,76]},132:{name:"Midnight Armour",bldgs:[34,76]},133:{name:"Over-Padded Armour",bldgs:[34,76]},134:{name:"Plainsman's Armour",bldgs:[34,76]},135:{name:"Reinforced Leather",bldgs:[34,76]},136:{name:"Splintmail",bldgs:[34,76]},137:{name:"Sun-Burnished Armour",bldgs:[34,76]},138:{name:"Upland Armour",bldgs:[34,76]},139:{name:"Vanguard's Armour",bldgs:[34,76]},141:{name:"Cloth-Backed Chainmail",bldgs:[32,75]},142:{name:"Desert Chainmail",bldgs:[32,75]},143:{name:"Double-Weave Chainmail",bldgs:[32,75]},144:{name:"Extra Heavy Chainmail",bldgs:[32,75]},145:{name:"Extra Light Chainmail",bldgs:[32,75]},146:{name:"Fur-Lined Chainmail",bldgs:[32,75]},147:{name:"Heavy Chain Armour",bldgs:[32,75]},148:{name:"Highland Chainmail",bldgs:[32,75]},149:{name:"Hillsman's Chainmail",bldgs:[32,75]},150:{name:"Jungle Chainmail",bldgs:[32,75]},151:{name:"Light Chain Armour",bldgs:[32,75]},152:{name:"Master-Crafted Chainmail",bldgs:[32,75]},153:{name:"Over-Padded Chainmail",bldgs:[32,75]},154:{name:"Plainsman's Chainmail",bldgs:[32,75]},155:{name:"Reinforced Chainmail",bldgs:[32,75]},156:{name:"Silversteel Chainmail",bldgs:[32,75]},157:{name:"Thick-Ring Chainmail",bldgs:[32,75]},158:{name:"Vanguard's Chainmail",bldgs:[32,75]},159:{name:"Woodsman's Chainmail",bldgs:[32,75]},161:{name:"Ancient Pattern Armour",bldgs:[33,61]},162:{name:"Chain-Edged Platemail",bldgs:[33,61]},163:{name:"Desert Platemail",bldgs:[33,61]},164:{name:"Dwarven Champion's",bldgs:[33,61]},165:{name:"Forester's Platemail",bldgs:[33,61]},166:{name:"Fur-Lined Platemail",bldgs:[33,61]},167:{name:"Heavy Platemail",bldgs:[33,61]},168:{name:"Highland Platemail",bldgs:[33,61]},169:{name:"Jungle Platemail",bldgs:[33,61]},170:{name:"Light Platemail",bldgs:[33,61]},171:{name:"Obsidian Platemail",bldgs:[33,61]},172:{name:"Plainsman's Platemail",bldgs:[33,61]},173:{name:"Reinforced Platemail",bldgs:[33,61]},174:{name:"Silversteel Platemail",bldgs:[33,61]},175:{name:"Spiked Platemail",bldgs:[33,61]},176:{name:"Thickened Platemail",bldgs:[33,61]},177:{name:"Under-Padded Platemail",bldgs:[33,61]},178:{name:"Upland Platemail",bldgs:[33,61]},179:{name:"Vanguard's Platemail",bldgs:[33,61]},259:{name:"Prestige Scrap",bldgs:[7]},260:{name:"Prestige Parchment",bldgs:[7]},261:{name:"Prestige Page",bldgs:[7]},262:{name:"Prestige Scroll",bldgs:[7]},263:{name:"Prestige Codex",bldgs:[7]},264:{name:"Prestige Book",bldgs:[7]},265:{name:"Prestige Tome",bldgs:[7]},422:{name:"Elven Thoroughbred",bldgs:[21,62]},423:{name:"Battlebred",bldgs:[21,62]},424:{name:"Dwarven Battle Mule",bldgs:[21,62]},425:{name:"War Wolf",bldgs:[21,62]},426:{name:"Soul-Forged Blade",bldgs:[]}};
